// Copyright (c) 2010, SAS Institute Inc., Cary, NC, USA. All Rights Reserved

//Script used to convert Web Infrastructure Platform's notification templates
//to use string template technology from v930. The script should be called during 
//loadContent target of SDW install when the app server is up and running.
//The script reads the source notification templates(.html,.sms,.txt) from SAS Content Server, 
//converts the templates and uploads the new templates(with extension .st) to the Content Server.
//The source templates will then be deleted from the Content Server

//Attributes:
//
//1. resultproperty
//	 Name of an Ant property which will be defined by this script
//	 to specify an int value representing the return code.
//	 This attribute is required.
//	  0 : success
//	 -1 : Unable to create DAV Resource
//	 -2 : BufferedReader creation failure
// 	 -3 : BufferedWriter creation failure
//	 -4 : Pattern Matching failure
//	 -5 : Failure while uploading contents to DAV
//	 -6 : Failure while deleting resource in DAV
//2. outputproperty
//	 Name of an Ant property which will be defined by this script
//	 to specify a message summarizing its completing status.
//	 This attribute is optional 
//3. protocol
//	 Protocol used to access the content server.
//	 This attribute is required.
//	 (e.g. http)
//4. host
//	 Machine name of the content server
//	 This attribute is required.
//	 (e.g. dxxxxx.na.sas.com)
//5. port
//	 Port to access content server
//	 This attribute is required.
//6. basepath
//	 Base path of the content server
//	 This attribute is required.
//	 (e.g. /SASContentServer/repository/default/sasdav)
//7. username
//	 Admin username of the content server
//	 This attribute is required.
//	 (e.g. sasadm@saspw)
//8. password
//	 Password to access content server
//	 This attribute is required.

import java.io.BufferedReader
import java.io.BufferedWriter
import java.io.IOException
import java.io.InputStreamReader
import java.io.OutputStreamWriter
import java.rmi.RemoteException
import java.util.regex.Pattern			
import com.sas.services.webdav.DAVResource
import com.sas.services.webdav.DAVException

AntBuilder ant = new AntBuilder()

logger =
{
	level, msg -> ant.echo(level:level, message:msg);
}

//Name of the ant property which will contain a return status message
String outputprop_msg = attributes["outputproperty"]

//Name of the ant property which will contain a return status code
String outputprop_rc = attributes["resultproperty"]

//Protocol to access content server
def String protocol = attributes["protocol"]

//Machine name to access content server
def String host = attributes["host"]

//Content server Port
def String port = attributes["port"]

//Base Path
def String basePath = attributes["basepath"]

//URL to access Content Server
def String url = protocol + "://" + host + ":" + port + basePath

//Admin user name to access Content Server
def String username = attributes["username"]

//Password to access content server
def String password = attributes["password"]

//DAV Path to source notification templates
def String davPath = basePath + "/Templates/notification"

//List of source template names
def fileNames = ["SAS_Email_Message", "SAS_CommentAdded", "BIDAlert_default", "BIDAlert_linkOnly"]

//Extensions of source templates
def extensions = ["html", "txt", "sms"]

//Temporary Folder Location
def tempFolderLoc = project.getProperty("config.lev.web.temp.dir")

BufferedReader reader = null
BufferedWriter writer = null
DAVResource resource = null
String returnMsg = null
int rc

try
{	
	logger("debug", "creating dav resource")
	rc = -1
	returnMsg = "Unable to create DAV Resource"
	resource = new DAVResource(url, username, password)
	
	//delete directories in the temp folder loc if already exists
	if(tempFolderLoc != null && tempFolderLoc.size() > 0)
	{
		File temp = new File(tempFolderLoc + "/SASContentServer")
		if(temp.exists())
		{
			temp.delete()
			logger("debug", "Deleting already existing temporary folders")
		}
	}
	//get a list of subdirectories
	List cList = resource.getChildrenList(davPath)
	logger("debug", "Children list for path " + davPath + " - " + cList)
	
	if(cList.size() > 0)
	{
		cList.each{subDir->
			//get a list of file names
			resource.getChildrenList(subDir).each{templates->
				fileNames.each{fileName->
					def index = templates.indexOf(fileName)
					if(index >= 0)
					{
						def name = templates.substring(index)
						def tempFolder
						if(tempFolderLoc != null && tempFolderLoc.size() > 0)
						{
							tempFolder = tempFolderLoc + templates
						}
						File file = new File(tempFolder)
						File folder = file.getParentFile();
						//create directories if they do not exist
						if(!folder.exists())
						{
							folder.mkdirs();
						}	
						//create empty file if the file does not exist
						if(!file.exists())
						{
							file.createNewFile()
						}
						resource.setPath(templates)
						//retrieve contents to the given file
						resource.getContents(file)	
						logger("debug", "Getting contents from " + file)
					}
				}
			}
		}
		logger("debug", "Reading templates to local folder complete")
		
		//recurse through directories in the given path
		new File(tempFolderLoc + davPath).eachDirRecurse{path->
			fileNames.each{fileName->
				extensions.each{ext->
					File template = new File(path.getPath() + "/" + fileName + "." + ext)
					
					if(template.exists())
					{
						rc = -2
						returnMsg = "Error while creating BufferedReader for " + template
						reader = new BufferedReader(new InputStreamReader(new FileInputStream(template), "UTF-8"))
						File stFile = new File(path.getPath() + "/" + fileName + ".st");
						
						rc = -3;
						returnMsg = "Error while creating BufferedWriter for " + stFile
						if(stFile.exists())
						{
							writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(stFile, true), "UTF-8"))
						}
						else
						{
							writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(stFile), "UTF-8"))
							writer.append("\$! ---------------------------------------------------------------- !\$\n")
							writer.append("\$! SAS Notification Template: " + fileName + "                      !\$\n")
							writer.append("\$! Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA,      !\$\n")
							writer.append("\$! All Rights Reserved                                              !\$\n")
							writer.append("\$!                                                                  !\$\n")
							writer.append("\$! ---------------------------------------------------------------- !\$\n\n")
						}
						
						writer.append("\$!========================" + ext.toUpperCase() + "======================================!\$\n\n")
						writer.append("\$if(_SAS_TEMPLATE_" + ext.toUpperCase() + ")\$\n\n")
						
						String line
						StringBuffer sb = new StringBuffer()
						
						rc = -4;
						returnMsg = "Error while doing Pattern Matching"
						
						while(true)
						{
							line = reader.readLine();
							if(line == null)
							{
								break;
							}
							def m = Pattern.compile(/%\w+/).matcher(line)
							StringBuffer sb1 = new StringBuffer();
							while(m.find())
							{
								str = line.substring(m.start(), m.end())
								str1 = str.substring(1, str.length())
								m.appendReplacement(sb1, "\\\$" + str1 + "\\\$")
							}
							m.appendTail(sb1)
							if(sb1.length() > 0)
							{
								line = sb1.toString()
							}
							sb.append(line).append("\n")
						}
						writer.append(sb.toString())
						writer.append("\n\$endif\$\n\n")
						
						reader.close()
						writer.close()
						template.delete()
					}
				}
			}
			
			fileNames.each{fileName->
				def uploadPath = davPath + "/" + path.name + "/" + fileName + ".st"
				logger("debug", "upload path " + uploadPath)
				File uploadFile = new File(path.getPath() + "/" + fileName + ".st")
				
				if(uploadFile.exists())
				{
					resource.setPath(uploadPath)
					FileInputStream fis = new FileInputStream(uploadFile)
					
					rc = -5
					returnMsg = "Could not upload " + uploadFile + " contents to DAV"
					
					resource.setContents(fis)
					fis.close()
				}
			}
			
			fileNames.each{fileName->
				extensions.each{ext->
					def path1 = davPath + "/" + path.name + "/" + fileName + "." + ext
					resource.setPath(path1);
					
					rc = -6
					returnMsg = "Could not delete " + path1 + " in DAV"
					
					resource.delete()
				}
			}
		}
		logger("debug", "New Templates uploaded to DAV and Source Templates are deleted")
	}
	else{
		logger("info", "There are no notification templates to migrate.")
	}
	
	resource.close()
	
	//delete files created in the temporary folder location
	if(tempFolderLoc != null && tempFolderLoc.size() > 0)
	{
		File temp = new File(tempFolderLoc + "/SASContentServer")
		if(temp.exists())
		{
			temp.delete()	
			logger("debug", "Temporary files deleted")
		}
	}
	
	rc = 0
	returnMsg = "Migrating notification templates completed"
}
catch(DAVException e)
{
	logger("error", e.getMessage())
}
catch(RemoteException e)
{	
	logger("error", e.getMessage())
}
catch(IOException ex)
{
	logger("error", ex.getMessage())
}
finally
{
	if(outputprop_msg != null && outputprop_msg.trim().length() > 0)
	{
		project.setNewProperty(outputprop_msg, returnMsg)
	}	
	project.setNewProperty(outputprop_rc, "" + rc)
	
	logger("info", returnMsg)
}