// Copyright (c) 2010, SAS Institute Inc., Cary, NC, USA. All Rights Reserved

// Script used to read the Host, Port, CommunicationProtocol
// and/or Service attributes of a TCPIPConnection which represents
// the Connection URI associated with the specified SoftwareComponent.

// Attributes:
// 
// 1. resultproperty
//    Name of an Ant property which will be defined by this script
//    to specify an int value representing the return code.
//    This attribute is required.
//     0 : success
//    -1 : initialization failure
//    -2 : Metadata Server connection failure
//    -3 : TCPIPConnection search failure
//    -4 : TCPIPConnection search returned no results
//    -5 : TCPIPConnection search returned more than one result
//    -6 : read TCPIPConnection attribute failure
// 2. outputproperty
//    Name of an Ant property which will be defined by this script
//    to specify a message summarizing its completion status.
//    This attribute is optional.
// 3. outputpropertyfqid
//    Name of an Ant property which will be defined by this script
//    to specify the FQID of the TCPIPConnection object which was
//    read.
//    This attribute is required.
//    (e.g. "A5ZBO87U.AV00000G")
// 4. softwarecomponentname
//    The name of the SoftwareComponent whose PublicType is 'Application'
//    which is associated to a DeployedComponent whose Name is 'Registered SAS Application'
//    which is associated to a TCPIPConnection whose name is 'Connection URI'.
//    This name will be used in an XMLSelect to get the TCPIPConnection to update its
//    Host, Port, CommunicationProtocol, Service attributes.
//    This XMLSelect should uniquely identify one TCPIPConnection when used in the 
//    XMLSelect where xxx is the name of the SoftwareComponent.
//    "TCPIPConnection[@Name='Connection URI'][Source/DeployedComponent[@Name='Registered SAS Application']/DescriptiveComponent/SoftwareComponent[@Name='xxx' and @PublicType='Application']]"
//    This attribute is required.
//    (e.g. "Web Infra Platform Services 9.2")
// 5. outputpropertyprotocol
//    TCIPConnection communication protocol
//    This attribute is required.
//    (e.g. "http", "https")
// 6. outputpropertyhost
//    TCPIPConnection host.
//    This attribute is required.
//    (e.g. "xxx.yyy.com")
// 7. outputpropertyport
//    TCPIPConnection port.
//    This attribute is required.
//    (e.g. "1234")
// 8. outputpropertyservice
//    TCPIPConnection servlet context.
//    This attribute is required.
//    (e.g. "/SASWIPServices")
// 9. host
//    Metadata Server host.
//    If this attribute is not specified, then the host will default
//    to the value of the "SMU.host.metadata" property.
//    This attribute is optional.
//    (e.g. "xxxx.xx.xx.com")
//10. port
//    Metadata Server port.
//    If this attribute is not specified, then the port will default
//    to the value of the "SMU.port.metadata" property.
//    (e.g. "8561")
//11. user
//    Metadata Server connection user ID.
//    If this attribute is not specified, then the user ID will default
//    to the value of the "SMU.user" property.
//    This attribute is optional.
//    (e.g. "myuserid")
//12. password
//    Metadata Server connection user password.
//    If this attribute is not specified, then the password will default
//    to the value of the "SMU.password" property.
//    This attribute is optional.
//    (e.g. "mypassword")


import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.List;

import com.sas.metadata.remote.MdException;
import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdOMRConnection;
import com.sas.metadata.remote.MdObjectStore;
import com.sas.metadata.remote.MdOMIUtil;
import com.sas.metadata.remote.MetadataObjects;
import com.sas.metadata.remote.TCPIPConnection;

int rc = -1;
String returnMsg = "Unable to initialize.";

AntBuilder ant = new AntBuilder();

logger =
{
   level, msg -> ant.echo(level:level, message:msg);
}


// name of the Ant property which will be set a return status message.
String outputproperty_msg = attributes["outputproperty"];

// name of the Ant property which will be set to the return code.
//  0 : success
// -1 : initialization failure
// -2 : Metadata Server connection failure
// -3 : TCPIPConnection search failure
// -4 : TCPIPConnection search returned no results
// -5 : TCPIPConnection search returned more than one result
// -6 : read TCPIPConnection attribute failure
String outputproperty_returncode = attributes["resultproperty"];

// Name of the SoftwareComponent which is associated to a
// TCPIPConnection via a Connection URI.
String softwareComponentName = attributes["softwarecomponentname"];

// Metadata Server connection user ID.
// If this attribute is not specified, then the user ID will default
// to the value of the "SMU.user" property.
String metadataUser = attributes["user"];

// Metadata Server connection user password.
// If this attribute is not specified, then the password will default
// to the value of the "SMU.password" property.
String metadataPassword = attributes["password"];

// Metadata Server host.
// If this attribute is not specified, then the host will default
// to the value of the "SMU.host.metadata" property.
String metadataHost = attributes["host"];

// Metadata Server port.
// If this attribute is not specified, then the port will default
// to the value of the "SMU.port.metadata" property.
String metadataPort = attributes["port"];

// name of the Ant property which will be set to the FQID of the
// TCPIPConnection metadata object which was updated
// (e.g. "A5ZBO87U.AV00000G")
String outputproperty_fqid = attributes["outputpropertyfqid"];

// communication protocol (e.g. "http", "https")
String outputproperty_protocol = attributes["outputpropertyprotocol"];

// host (e.g. "xxx.yyy.com")
String outputproperty_host = attributes["outputpropertyhost"];

// port (e.g. "1234")
String outputproperty_port = attributes["outputpropertyport"];


// servlet context (e.g. "/SASLogon")
String outputproperty_service = attributes["outputpropertyservice"];




MdFactory mdFactory = null;

try
{
   rc = -2;
   returnMsg = "Unable to connect to the Metadata Server";
   // create connection to the Metadata Server

   mdFactory = new MdFactoryImpl(false);
   MdOMRConnection mdOMRConnection = mdFactory.getConnection();
   mdOMRConnection.makeOMRConnection(
      (metadataHost == null)
         ? project.getProperty("SMU.host.metadata")
         : metadataHost,
      (metadataPort == null)
         ? project.getProperty("SMU.port.metadata")
         : metadataPort,
      (metadataUser == null)
         ? project.getProperty("SMU.user")
         : metadataUser,
      (metadataPassword == null)
         ? project.getProperty("SMU.password")
         : metadataPassword);
   MdObjectStore objectStore = mdFactory.createObjectStore();
   MdOMIUtil mdOMIUtil = mdFactory.getOMIUtil();


   rc = -3;
   // query TCPIPConnection from the Metadata Server

   String options = "<XMLSelect search=\"TCPIPConnection[@Name='Connection URI'][Source/DeployedComponent[@Name='Registered SAS Application']/DescriptiveComponent/SoftwareComponent[@Name='" + softwareComponentName + "' and @PublicType='Application']]\"/>";
   returnMsg = "Search failed for: " + options;
   List tcpipConnections = mdOMIUtil.getMetadataObjectsSubset(
      objectStore,
      mdOMIUtil.getFoundationReposID(),
      MetadataObjects.TCPIPCONNECTION,
      MdOMIUtil.OMI_GET_METADATA
         | MdOMIUtil.OMI_ALL_SIMPLE
         | MdOMIUtil.OMI_TEMPLATE
         | MdOMIUtil.OMI_XMLSELECT,
      options);

   if (tcpipConnections.size() > 1)
   {
      rc = -5;
      returnMsg = "The search " + options + " returned multiple URI connections.";
      // search returned multiple connection URIs
      throw new IllegalStateException(returnMsg);
   }
   else if (tcpipConnections.isEmpty())
   {
      // search did not find a connection URI
      rc = -4;
      returnMsg = "The search " + options + " did not return a URI connection.";
   }
   else
   {
      rc = -6;
      returnMsg = "Unable to read TCPIPConnection attribute.";
      // read TCPIPConnection attributes

      TCPIPConnection tcpipConnection = (TCPIPConnection) tcpipConnections.get(0);
      String fqid = tcpipConnection.getFQID();
      String protocol = tcpipConnection.getCommunicationProtocol();
      String host = tcpipConnection.getHostName();
      String port = tcpipConnection.getPort();
      String service = tcpipConnection.getService();

      rc = 0;
      returnMsg = "The Connection URI represented by the TCPIPConnection[ID=" + fqid + "] for software component '" + softwareComponentName + "' has been successfully read.";
      // success

      // set a property whose value specifies the FQID of the
      // updated TCPIPConnection.
      project.setNewProperty(outputproperty_fqid, fqid);
      project.setNewProperty(outputproperty_protocol, protocol);
      project.setNewProperty(outputproperty_host, host);
      project.setNewProperty(outputproperty_port, port);
      project.setNewProperty(outputproperty_service, service);
   }

}
catch (Exception e)
{
   logger("error", e.getMessage());
   sw = new StringWriter();
   pw = new PrintWriter(sw);
   e.printStackTrace(pw);
   logger("error", sw.toString());
}
finally
{
   if (null != outputproperty_msg && outputproperty_msg.trim().length() > 0)
   {
      project.setNewProperty(outputproperty_msg, returnMsg);
   }
   project.setNewProperty(outputproperty_returncode, "" + rc);

   logger("info", returnMsg);

   if (null != mdFactory)
   {
      try
      {
         mdFactory.dispose();
      }
      catch (RemoteException e)
      {
         logger("error", e.getMessage());
      }
   }

}