// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.util
import org.apache.log4j.Logger
import org.apache.log4j.PropertyConfigurator

// **************************************************************************************************************
// Class: CommandProcess
// Purpose: Encapsulate the attributes and methods used to run an external command using the Ant exec task
//			This class makes use of the Groovy AntBuilder class to construct the environment that allows the
//			Ant exec task to work. 
// Methods:
//		run - Run the provided command on the target operating system
//			  Method input parameters:
//					command_directory - this is the directory that the Ant exec task will change to before executing
//										the command.
//					command_string	  - this is the command to be executed (e.g. fully-qualified name of shell/bat
//										script).
//					command_options	  - this is a String array containing all options for the command (e.g. ["-dir",
//										"/path/to/file/used/by/the.command"])
//					outputproperty_name - Ant outputproperty name (default: "ant_outputproperty") 
//					resultproperty_name - Ant resultproperty name (default: "ant_resultproperty")
//					command_input	  - Input string used for response to the command (optional)
//			  Method output:
//					int exitValue     - Return code from the command
//					process_output	  - Ant exec outputproperty (fetch via getter)
//					process_exitValue - Ant exec resultproperty in Integer form	(fetch via getter)
// ****************************************************************************************************************
class CommandProcess {
		final static Logger logger = Logger.getLogger("com.sas.config");	// Instantiate singleton of log4j object for logging
        // Local class fields available for getting:
        String process_output
        int    process_exitValue
        String line_separator 
        
        // *************************************************************************************
        // CTOR: Initialize properties used to run the command and obtain output
        public CommandProcess() {
			line_separator = java.lang.System.getProperty("line.separator")
        }   // END CTOR
        
        // *************************************************************************************
        // Method to run the provided command on the target operating system
        public int run( String command_directory, 
        				String command_string, 
        				String[] command_options, 
        				String outputproperty_name="ant_outputproperty", 
        				String resultproperty_name="ant_resultproperty", 
        				String command_input="" ) {
        	
        	// Print input property values for the process:
			logger.info("CommandProcess-run: ------------> Command processing begins... ")
			logger.info("CommandProcess-run: ---> Command directory: \"${command_directory}\"")  
			StringBuffer commandToRun = new StringBuffer(command_string+" ")
			command_options.each { opt -> commandToRun.append(opt+" ") }           
			logger.info("CommandProcess-run: ---> Command to run: \"${commandToRun.toString()}\"")
			if (command_input != "")
				logger.info("CommandProcess-run: ---> Command input string: \"${command_input}\"") 
			logger.info("CommandProcess-run: ---> Output property name: ${outputproperty_name}")
			logger.info("CommandProcess-run: ---> Result property name: ${resultproperty_name}")
			
			// Run the process using the Ant exec task:
			def ant = new AntBuilder()	// Construct the AntBuilder object to run the Ant exec task			
			ant.exec( executable:command_string, 
	                  dir:command_directory,
	                  inputstring:command_input,
	                  outputproperty:"${outputproperty_name}", 
	                  resultproperty:"${resultproperty_name}" ) 
	                {
	                  	command_options.each { opt -> arg(value:opt) }	// Create individual arg(...) values for each command option
	                }
	                
	        // Obtain output from the process and print it:
	        process_output = ant.project.properties."${outputproperty_name}"
	        process_exitValue = Integer.valueOf(ant.project.properties."${resultproperty_name}")
	        String [] outputLines = process_output.split("[${line_separator}]+")
	        logger.info("CommandProcess-run: ------------> Begin Command Output <------------")
	        outputLines.each { line-> logger.info("${line}") } 
	        logger.info("CommandProcess-run: ------------> End   Command Output <------------")
	        logger.info("CommandProcess-run: ------------> Command processing ended; Exit value: ${process_exitValue} ")
	        ant = null	// Ensure a new instance gets created for next process
	        return process_exitValue                                
        }       // END run method

		/*******************************************************************************************************************
		** For testing: **      
		public final static void main(String [] args) {
	        StringBuffer options_string
	        String [] options
	        int rc
	        CommandProcess cmdProc = new CommandProcess()
	  		//String command = "C:\\was70\\bin\\manageprofiles.bat"
	  		//String directory = "C:\\was70\\bin"
	        //String [] options  = ["-response", "C:\\junk\\create.DmgrProfile.CELL.response.MyDmgr01.properties"]
	        String directory = "."
	        String command = "C:\\junk\\with spaces\\test.bat"
	                
	        // RUN # 1
	        options = ["dir", "C:\\junk\\with spaces", "copy", "C:\\junk\\with spaces\\test.bat", "C:\\junk\\with spaces\\test.bat.bak1"]
	        options_string = new StringBuffer()
	        options.each { opt -> options_string.append(opt+" ") } 
	        rc = cmdProc.run(directory, command, options, "out1", "rc1")
	        println "Main: RUN #1: Command \"${command} " + options_string.toString() + "\"" 
	
			// RUN # 2  
			options = ["type", "C:\\junk\\with spaces\\test.bat", "copy", "C:\\junk\\with spaces\\test.bat", "C:\\junk\\with spaces\\test.bat.bak2"]
			options_string = new StringBuffer()
			options.each { opt -> options_string.append(opt+" ") } 
			rc = cmdProc.run(directory, command, options, "out2", "rc2")
			println "Main: RUN #2: Command \"${command} " + options_string.toString() + "\"" 
	     }	// END main method
	     *******************************************************************************************************************/
}	// End CommandProcess class
