//*************************************************************************************************************************************
// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved
//
// Class:	CredentialsDialogPrep
//
// Purpose: Encapsulate the behavior and attributes used to prepare and process the dialog to solicit userId and password  
//			credentials from the user.  These credentials are required to configure appserver resources for the SAS webapps.
//
//*************************************************************************************************************************************

package com.sas.config.util

import java.util.Enumeration;
import java.util.HashMap;
import java.util.ArrayList;
import org.apache.log4j.Logger;
import com.sas.config.util.ResourceProperties;

class CredentialsDialogPrep  {
    
    final static Logger logger = Logger.getLogger("com.sas.config");    // Instantiate singleton of log4j object for logging
    int m_rc                        // Method return code
    
    // Attributes from the CTOR parameters 
    def taskRsrcNameTable           // Map of tasks (e.g. "DataSource.create") and resource names (e.g. "SASServer1-SharedServices" or could be wildcard "*") that require prompting for credentials
    def listTasksNeedingCredentials // List of tasks from the "tasks to be executed" that require credentials
    String credentialsFile          // Fully-qualified file name of Credentials file
    boolean debugFlag               // DEBUG flag
    String promptMode               // Prompting mode "GRAPHIC" or "TEXT"
	boolean hasScope = true			// Set by default.  Determine if credential names include scope
        
    // Attributes used to process the Credentials file
    ResourceProperties credentialsProps = null  // Credentials properties  
    File credentialsFileObj = null      // File object used by Properties class
    FileInputStream fis = null      // FileInputStream object used by Properties class to read the file
    def uniqMap = [:]               // A map containing the unique resources and the associated unique integer identifier

    //*******************************************************************************************************************************
    // Method: CTOR
    // Input: 
    //      tasks - list of tasks needing credentials, each list item in the form:  <resourceType>.<operation>_<scope>_<resourceName>
    //      file  - file name of the Credentials properties file
    //      promptMode - "TEXT" (default) or "GRAPHIC"
    //*******************************************************************************************************************************
    CredentialsDialogPrep(def tasks, String file, Boolean debug=false, String _promptMode="TEXT") {
		logger.info("CredentialsDialogPrep: Prepare for credentials dialog, if necessary, using \"${file}\" properties file...")
        taskRsrcNameTable = tasks;
        credentialsFile = file;
        debugFlag = debug;
        promptMode = _promptMode;
        credentialsProps = new ResourceProperties(credentialsFile)
    }   // End CTOR
	
    //**********************************************************************************************************
    // Method:  getTaskRsrcNameTable - return the provided map of tasks and resource names requiring credentials
    //**********************************************************************************************************
    String [] getTaskRsrcNameTable() {
        return taskRsrcNameTable;
    }   // End getTaskRsrcNameTable method
	
	boolean getHasScope() {
		return hasScope;
	} 
	
	void setHasScope(boolean _hasScope) {
		hasScope = _hasScope;
	}

    //**********************************************************************************************************************
    // Method:  processCredentials - prompt for credentials for the list of tasks provided
    // Purpose: Determine credentials prompting required for the list of tasks to be run. 
    // Input:   listTasksNeedingCredentials - Groovy list containing a task in the form:
    //          <resourceType>.<operation>_<scope>_<resourceName> (e.g. "Datasource.create_SERVER_*" or "LoginModule.create_CELL_SAS-Trusted-user")
    //          <resourceType>.<operation>_<resourceName> (e.g. "Datasource.create_SASServer1-SharedServices")
    // Output:  ArrayList - a list where each entry in the list is a map containing the contents used to display the text   
    //          (titleText) when prompting for userId and password 
    //**********************************************************************************************************************
    public ArrayList processCredentials(def _listTasksNeedingCredentials) {
        logger.info("CredentialsDialogPrep-processCredentials: Determine if all required credentials are available...")
        listTasksNeedingCredentials = _listTasksNeedingCredentials
        int cred_RC = 0     // Return code from this method
        def contents = []   // List of contents (titleText, userId & password) to be displayed by CredentialsDialog
        if (debugFlag)  {  
            logger.debug("**DEBUG** CredentialsDialogPrep-processCredentials: Type of class for listTasksNeedingCredentials = ${listTasksNeedingCredentials.getClass()}")
            logger.debug("**DEBUG** CredentialsDialogPrep-processCredentials: listTasksNeedingCredentials=${listTasksNeedingCredentials}")
        }
        createUniqueMap()   // Create the "uniqMap" attribute with key names (common string up to "_userid" & "_passwd") with a unique integer identifier as values from the  
        
        // At this point we have map containing all of the unique key names, which is the full key name without the "_userid" and "_passwd" portion
        // Now construct the list of maps, which is input to the CredentialsDialog program 
        if (debugFlag)
            logger.debug("**DEBUG** CredentialsDialogPrep-processCredentials: All unique property names requiring credentials: ${uniqMap}")
        def keys = uniqMap.keySet()     // Get the set of unique property names
        keys.each { key ->          // Process each unique property name
            def contentsParm = null // A map with parameters for the CredentialsDialog program
			contentsParm = createDialogParm(uniqMap, key)
            if (contentsParm)
                contents.add(contentsParm)
        }   // End construct dialog parameter list
        
        // Now, we have a map containing the lists of input parameters for the CredentialsDialog class
        // Solicit the required userIds and passwords using the CredentialsDialog program 
        logger.info("CredentialsDialogPrep-processCredentials: Determine if credentials need to be solicited for those resources that require authentication...")
        if (debugFlag) 
            logger.debug("**DEBUG** CredentialsDialogPrep-processCredentials: Count of parameter maps input to CredentialsDialog: " + contents.size())
        return contents             // Return the list of maps containing contents that the CredentialsDialog class requires for prompting
    }   // End processCredentials method
    
	public void handleBadCredentialKey(msg, key) {
		String message = "invalid key found in credentials file:  $key.  $msg"
		println(message)
		logger.error("**SEVERE** CredentialsDialogPrep-createUniqueMap: $message")
		throw new java.lang.IllegalArgumentException(message);
	}
	
    //**********************************************************************************************************************
    // Method:  createUniqueMap
    // Purpose: Build a Map which contains the common string from the "_userid" and "_passwd" keys from the Credentials.file 
    //          and values of a unique integer identifier
    // Output:  Map - contains key (common string from "_userid" and "_passwd" keys) and value (unique integer identifier)
    //          e.g. ["DataSource.create_SERVER_SASServer1-SharedServices":0, "LoginModule.create_CELL_*":1]
    //**********************************************************************************************************************
    public void createUniqueMap() {

        // Table of ResourceTypes and operations that require authentication credentials
        def tasksNeedingCredentials = taskRsrcNameTable // To eliminate a Groovy bug(?) Had to use a local method variable and copy from the class variable, otherwise HashMap datatype gets lost
        uniqMap = [:]                           // Initialize the class attribute "uniqMap" map.  It will constructed by this method
        int uniqId = 0                          // Identifier used as a unique id for the common key-part in the Credentials.properties file 
                
        // Read each of the credentials.properties and prepare a Map containing the unique property name prefixes and a corresponding identifier
        String taskToExec                       // resourceType.operation portion of a credentials property key
        String scope                            // scope portion of a credentials property key
        String resourceName                     // resourceName portion of a credentials property key
        def credentialsKey                      // Credentials key used to build input map for createDialogParm method
        
        // Process each key in the Credentials.properties file:
		def keys = credentialsProps.keySet()    // Get the set of unique keys (property names) from the credentials.properties file
        keys.each { key ->                      // Process each property name from the credentials.properties file
            int lastUnderbar = key.lastIndexOf("_") // Underbar before "userid", "passwd" or "titleText"
            if ( lastUnderbar == -1 )
				handleBadCredentialKey("All keys must contain underbars.", key)
            int firstUnderbar = key.indexOf("_") // Underbar before "userid", "passwd" or "titleText"
            if ( firstUnderbar == -1 )
				handleBadCredentialKey("All keys must contain underbars.", key)
            credentialsKey = key.substring(0,lastUnderbar)  // Isolate the unique key portion less the "userid" or "passwd" or "titleText"
            taskToExec = key.substring(0, firstUnderbar) // resourceType.operation (e.g. "DataSource.create")
			int nextUnderbar = firstUnderbar
            if (hasScope) {
				int resourceUnderbar = key.indexOf("_", firstUnderbar+1)
				if ( resourceUnderbar == -1 || resourceUnderbar == lastUnderbar)
					handleBadCredentialKey("All keys must be of the form type.operation_scope_resource_key.", key)
				nextUnderbar = resourceUnderbar
				scope = key.substring(firstUnderbar+1, resourceUnderbar + 1)
            }
            else
                scope = ""
            resourceName = key.substring(nextUnderbar+1, lastUnderbar) // resourceName (e.g. "SASServer1-SharedServices" or "SAS-Trusted-user")
            if (debugFlag)  {
                logger.debug("**DEBUG** CredentialsDialogPrep-createUniqueMap: Isolated unique key from credentials file: ${key}")
                logger.debug("**DEBUG** CredentialsDialogPrep-createUniqueMap: resourceType.operation=${taskToExec}; scope=${scope}; resourceName=${resourceName}")
                logger.debug("**DEBUG** CredentialsDialogPrep-createUniqueMap: Type of class for taskToExec = ${taskToExec.getClass()}")
            }
            def compareKey = credentialsKey     // Assume credentials key includes explicit resourceName for comparator
            if (tasksNeedingCredentials.get(taskToExec).equals("*"))    // If task portion of credentials key indicates wildcard resourceName,
                compareKey = taskToExec+"_"+scope+"*"   // Comparator in listTasksNeedingCredentials list should include wildcard resourceName
            if ( listTasksNeedingCredentials.any { it.equals(compareKey) } )  { // If credentials key matches one in the listTasksNeedingCredentials,
                // This unique credentials key is one that requires prompting for credentials, so add it to the uniqMap input to createDialogParm
                if (debugFlag)
                    logger.debug("**DEBUG** CredentialsDialogPrep-createUniqueMap: Task \"${credentialsKey}\" needs credentials.")
                if (!uniqMap.containsKey(credentialsKey))  {    // If the unique property name doesn't already exist in the uniqMap
                    if (debugFlag)
                        logger.debug("**DEBUG** CredentialsDialogPrep-createUniqueMap: Add unique name \"${credentialsKey}\" with id \"${uniqId}\" to unique map.")
                    uniqMap.put(credentialsKey,uniqId++)        // Place the unique property name along with its unique identifier into the uniqMap
                }
            }
        }   // END each key entry in the Credentials.properties file
    }   // END createUniqueMap method

    //********************************************************************************************************************
    // Method:  getUniqueMap
    // Purpose: Getter method to return the reference to the unique map built by the createUniqueMap method
    // Output:  Map - contains key (common string from "_userid" and "_passwd" keys) and value (unique integer identifier)
    //          e.g. ["DataSource.create_SERVER__SASServer1-SharedServices":0, "LoginModule.create_CELL_*":1]
    //********************************************************************************************************************
    public Map getUniqueMap() { 
        return uniqMap
    }
    
    //**************************************************************************************************
    // Method:  createDialogParm
    // Purpose: Build a Map which can be added to the parameter list for CredentialsDialog program
    //**************************************************************************************************
    public Map createDialogParm(def uniqMap, def uniqKey) {
        if (debugFlag)
            logger.debug("**DEBUG** CredentialsDialogPrep-createDialogParm: uniqKey=${uniqKey}")
        def mapElement = [:]                            // Map constructed for each unique key name
        def titleTextKey = uniqKey+"_titleText"
        def titleTextValue = credentialsProps.getProperty(titleTextKey)
        def useridKey = uniqKey+"_userid"
        def useridValue = credentialsProps.getProperty(useridKey)
        def passwdKey = uniqKey+"_passwd"
        def passwdValue = credentialsProps.getProperty(passwdKey)
        if (useridValue == "" || passwdValue == "") {   // If either userid or password is not set, setup for prompting
            if (debugFlag)
                logger.debug("**DEBUG** CredentialsDialogPrep-createDialogParm: ${useridKey}=${useridValue}; ${passwdKey}=${passwdValue}; ${titleTextKey}=${titleTextValue}")
            mapElement.put('titleText', titleTextValue)
            def id = uniqMap.get(uniqKey)
            mapElement.put('userid',"uid"+id)
            mapElement.put('passwd',"pwd"+id)
            return mapElement
        } else {
            return null
        }
    }   // End createDialogParm method
    
    //**********************************************************************************************************************
    // Method:  processDialogResults
    // Purpose: Process the results map produced by the CredentialsDialog class containing the results from prompting for 
    //          credentials.
    // Input:   resultsMap - list containing map entries of [ ["titleText":"xxxx","userid":"uid0","passwd":"pwd0"], ... ]
    //              where each item in the list 
    // Output:  m_rc - Method return code 
    //              1 = successful update of Credentials.properties file with solicited credentials
    //              anything else means update fo Credentials.properties file failed.
    //**********************************************************************************************************************
    public int processDialogResults(def resultsMap) {
        // At this point, prompting for credentials for null items in the credentials.properties file has been completed.
        // Run through the unique key names and obtain the prompting responses for userid and passwd values from the dialog
        // using the unique identifier to access the results map
        if (debugFlag)  {
            logger.debug("**DEBUG** CredentialsDialogPrep-processDialogResults: CredentialsDialog results: resultsMap=${resultsMap}")
            logger.debug("**DEBUG** CredentialsDialogPrep-processDialogResults: CredentialsDialog results: uniqMap=${uniqMap}")
        }
        int uniqId = 0
        def rspId = ""
        // Run through the "uniqMap" created in createUniqMap method and fetch the unique integer identifier for each key 
        def keys = uniqMap.keySet()
        keys.each { key ->
            uniqId = uniqMap.get(key)   // Get the unique identifier number associated with the responses for this key
            if (debugFlag)
                logger.debug("**DEBUG** CredentialsDialogPrep-processDialogResults: Processing key=${key}, value=${uniqId}")
            rspId = "uid"+uniqId        // Obtain the response identifier for the "userid" response
            if (resultsMap.containsKey(rspId))  {
                def userid = resultsMap.get(rspId)
                def useridKey = key + '_userid'
				credentialsProps.setProperty(useridKey, userid)
            }
            rspId = "pwd"+uniqId        // Obtain the response identifier for the "passwd" response
            if (resultsMap.containsKey(rspId))  {
                def passwd = resultsMap.get(rspId)
                def encodedPasswd = com.sas.util.SasPasswordString.encode(com.sas.util.SasPasswordString.SAS002_ENCODING, passwd)
                def passwdKey = key + '_passwd'
				credentialsProps.setProperty(passwdKey, encodedPasswd)
            }
        }   // END run through the "uniqMap" keys 
        m_rc = credentialsProps.writeProps()      // Update the credentials.properties file with the results from prompting
        logger.info("CredentialsDialogPrep-processDialogResults: \"${credentialsFile}\" file has been updated with solicited credentials.")
        if (m_rc != 1) {
            logger.error("**SEVERE** Attempt to write updated \"${credentialsFile}\" file FAILED RC=${m_rc}; Unable to continue without required userIds/passwords for resources.")
            m_rc = -99
        }
        return m_rc
    }   // END processDialogResults method

    //**********************************************************************************************************************
    // Method:  scanCommandsForCredentials
    // Purpose: Scan the command line or script file for a list of commands that require credentials.
    // Input:   execType - Execution type RUN or FILE
    //          singleTaskLine - if execType is RUN, string containing command line
    //          taskFile - if execType is FILE, the file
    // Output:  List of resources that may need credentials for which prompting will be necessary
    //**********************************************************************************************************************
    public def scanCommandsForCredentials(String execType, String singleTaskLine, File taskFile) { 
        def tasksNeedingCredentials = taskRsrcNameTable // To eliminate a Groovy bug(?) Had to use a local method variable and copy from the class variable, otherwise HashMap datatype gets lost
        if (debugFlag)
            logger.debug("CredentialsDialogPrep-scanCommandStreamForCredentials: Debugging info has been requested")
        
        // Scan the task(s) to be executed to produce a list of tasks that may require authentication credentials
        def listTasksNeedingCredentials = []    // List of resources that may need credentials for which prompting will be necessary
        def pattern = /\S+/     // Pattern of non-whitespace words with whitespace(s) between the fields ("create Server SASServer1 SERVER")
        def fields = []         // Fields within the task line
        def taskToExec = ""     // String used to indentify the task by resourceType.operation
        def scope = ""          // scope from task line
        def resourceName = ""   // Resource name from task line
        def credentialsKey = "" // String used to isolate the unique key portion of the properties in the credentials properties file
        if (execType == "RUN") {    // Single-line task run mode
            fields = (singleTaskLine =~ pattern)    // Break out the fields from the task line using a Matcher
            boolean hasScope = (fields.size() == 4) // Scope not used for WebLogic
			if (debugFlag)  {
				//println "\nhasScope=${hasScope}"
				logger.debug("CredentialsDialogPrep-scanCommandStreamForCredentials: hasScope=${hasScope}")
			}
            // fields[0]=operation, fields[1]=resourceType, fields[2]=resourceName, fields[3]=scope
            taskToExec = fields[1]+"."+fields[0]    // e.g. "DataSource.create" or "LoginModule.create"
            //resourceName = fields[2]              // e.g. "SASServer1-SharedServices"
            if (hasScope)
                scope = fields[3] + "_"             // e.g. "SERVER"
			if (debugFlag)  {
				//println "\ntaskToExec=${taskToExec}"
				//println "\ntasksNeedingCredentials=${tasksNeedingCredentials}"
				logger.debug("CredentialsDialogPrep-scanCommandStreamForCredentials: taskToExec=${taskToExec}")
				logger.debug("CredentialsDialogPrep-scanCommandStreamForCredentials: tasksNeedingCredentials=${tasksNeedingCredentials}")
			}
            if (tasksNeedingCredentials.containsKey(taskToExec))  { // If this task is one of the ones needing credentials,
                resourceName = tasksNeedingCredentials.get(taskToExec)  // e.g. "*" or "SAS-Trusted-user"
                credentialsKey = taskToExec+"_"+scope+ resourceName // e.g. "DataSource.create_NODE_*"
                listTasksNeedingCredentials.add(credentialsKey) // Place single task into the empty list of tasks needing credentials
            }
        } else {    // execType="FILE", task file provided
            taskFile.eachLine { line ->
                if (!line.startsWith("#"))  {       // Process non-comment task lines
                    fields = (line =~ pattern)      // Break out the fields from the task line using a Matcher
                    boolean hasScope = (fields.size() == 4) // Scope not used for WebLogic
                    // fields[0]=operation, fields[1]=resourceType, fields[2]=resourceName, fields[3]=scope
                    // if (fields.size() == 4)  {
                        taskToExec = fields[1]+"."+fields[0]    // e.g. "DataSource.create" or "LoginModule.create"
                        //resourceName = fields[2]              // e.g. "SASServer1-SharedServices"
                        if ( hasScope )
                            scope = fields[3] + "_"                     // e.g. "SERVER"
                        if (tasksNeedingCredentials.containsKey(taskToExec))  { // If this task is one of the ones needing credentials,
                            resourceName = tasksNeedingCredentials.get(taskToExec)
                            credentialsKey = taskToExec+"_"+scope+resourceName
                            listTasksNeedingCredentials.add(credentialsKey) // Add task into the list of tasks needing credentials
                        }
                    // }
                }   // END process non-comment task line
            }   // END eachLine
        }
        listTasksNeedingCredentials = listTasksNeedingCredentials.unique()  // Make the list unique
        if (debugFlag)
            logger.debug("CredentialsDialogPrep-scanCommandStreamForCredentials: listTasksNeedingCredentials = ${listTasksNeedingCredentials}")
        return listTasksNeedingCredentials;
    }
	
	public scanCommandForCredential(String singleTaskLine, def listTasksNeedingCredentials, def otherTasksList)
	{
        def tasksNeedingCredentials = taskRsrcNameTable // To eliminate a Groovy bug(?) Had to use a local method variable and copy from the class variable, otherwise HashMap datatype gets lost
        def pattern = /\S+/     // Pattern of non-whitespace words with whitespace(s) between the fields ("create Server SASServer1 SERVER")
        def fields = []         // Fields within the task line
        def taskToExec = ""     // String used to indentify the task by resourceType.operation
        def scope = ""          // scope from task line
        def resourceName = ""   // Resource name from task line
        def credentialsKey = "" // String used to isolate the unique key portion of the properties in the credentials properties file
		fields = (singleTaskLine =~ pattern)    // Break out the fields from the task line using a Matcher
		boolean hasScope = (fields.size() == 4) // Scope not used for WebLogic
		taskToExec = fields[1]+"."+fields[0]    // e.g. "DataSource.create" or "LoginModule.create"
		if (otherTasksList != null) {
		    if (otherTasksList.containsKey(taskToExec))
				taskToExec = otherTasksList.getAt(taskToExec)
		}
		if (hasScope)
			scope = fields[3] + "_"             // e.g. "SERVER"
		if (tasksNeedingCredentials.containsKey(taskToExec))  { // If this task is one of the ones needing credentials,
			resourceName = tasksNeedingCredentials.get(taskToExec)  // e.g. "*" or "SAS-Trusted-user"
			credentialsKey = taskToExec+"_"+scope+ resourceName // e.g. "DataSource.create_NODE_*"
			listTasksNeedingCredentials.add(credentialsKey) // Place single task into the empty list of tasks needing credentials
		}
	}
}   // End CredentialsDialogPrep class
