//*************************************************************************************************************************************
// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved
//
// Class:	ResourceProperties
//
// Purpose: Encapsulate the behavior and attributes for the Resource Properties related to the resources that must be configured for
//			an appserver in support of the SAS webapps.
//
//*************************************************************************************************************************************
package com.sas.config.util
import java.util.Enumeration;

import org.apache.commons.io.FileUtils
import com.sas.util.SasPasswordString

public class ResourceProperties  extends Properties {
	//*********************************************************************************************************************************
	// Class variables:
	//*********************************************************************************************************************************
	int rp_rc = 0
	String propsFileName
	File propsFileObj = null
	FileInputStream fis = null	
	String backup_propsFileName
	File backup_propsFileObj
		
	//*********************************************************************************************************************************
	// CTOR - load properties from the file
	//*********************************************************************************************************************************
	public ResourceProperties(String fileName) {
		println "ResourceProperties: Load properties from file \"${fileName}\""
		propsFileName = fileName
		backup_propsFileName = propsFileName + ".bak"
		load()
	}	// End CTOR
	
	//**********************************************************************************************************************************
	// Override keys method by returning a sorted list of keys, which is how they get written to the associated file by the store method
	//**********************************************************************************************************************************
	public synchronized Enumeration keys() {
     Enumeration keysEnum = super.keys();
     Vector keyList = new Vector();
     while(keysEnum.hasMoreElements()){
       keyList.add(keysEnum.nextElement());
     }
     Collections.sort(keyList);
     return keyList.elements();
	}	// End keys method
	
	// Override getProperty method to obtain the value of a property
	public String getProperty(String name) {
		def propertyValue = null
		try {
			propertyValue = super.getProperty(name)
		}
		catch (groovy.lang.MissingPropertyException e) {
			println "ResourceProperties-getProperty: Property \"" + name + "\" does not exist in properties file \"" + propsFileName + "\"."
			propertyValue = null
			return null
		}
		return propertyValue
	}	// End getProperty method

	//*********************************************************************************************************************************
	// Override toString method to dump the contents of the properties object
	//*********************************************************************************************************************************
	public String toString() {
		StringBuffer sb = new StringBuffer()
		Enumeration keysEnum = super.keys()
		Vector keyList = new Vector()
		sb.append("Properties from \"${propsFileName}\":\n")
		while (keysEnum.hasMoreElements()) { 
			keyList.add keysEnum.nextElement()
		}
		Collections.sort(keyList)
		for (String key : keyList) {
			String value = getProperty(key)
			sb.append("$key=$value\n")
		}
		rp_rc = 1
		return sb.toString()
	}	// End toString method
	
	//*********************************************************************************************************************************
	// Override load method to load properties from the file
	//*********************************************************************************************************************************
	public int load()  {
		try {
			propsFileObj = new File(propsFileName);
    		fis = new FileInputStream(propsFileObj);
    	  	super.load(fis);
	      	fis.close();
       	} 
       	catch (FileNotFoundException e) {
       		rp_rc = -1
    		e.printStackTrace();
       	} 
       	catch (IOException e) {
       		rp_rc = -1
  			e.printStackTrace();
       	}
       	rp_rc = 1
       	return rp_rc
	}	// End load method
	
	//*********************************************************************************************************************************
	// Method: writeProps - Write out an updated properties file
	//*********************************************************************************************************************************
	public int writeProps() {
		println "ResourceProperties-writeProps: write properties to file \"${propsFileName}\""
		try {
			FileOutputStream fos = new FileOutputStream(propsFileObj);
			super.store(fos, "");
			fos.close();
			rp_rc = 1;
		}
		catch (IOException e) {
			rp_rc = -1;
			e.printStackTrace();
		}
		return rp_rc;
	}	// End writeProps method
	
	//*********************************************************************************************************************************
	// Method: getPropsFileName - Getter for the property file name of the instance
	//*********************************************************************************************************************************
	public String getPropsFileName() {
		return propsFileName
	}	// End getPropsFileName method
	
	//*********************************************************************************************************************************
	// Method: Return the set of keys from the Properties object
	//*********************************************************************************************************************************
	public Set getKeySet() {
		return keySet()
	}	// End getKeySet method
	
	//*********************************************************************************************************************************
	// Method: setPropertyValue - Set the value of a property
	//*********************************************************************************************************************************
	public void setPropertyValue(String name, Object value) {
		if (value != "not-set")
			setProperty(name, value)
	}	// End setProperty method
	
	//*********************************************************************************************************************************
	// Method: getBooleanProperty - Obtain boolean value of a property
	//*********************************************************************************************************************************
	public Boolean getBooleanProperty(String name)  {
		def booleanValue = null
		try {
			booleanValue = getProperty(name)
		}
		catch (groovy.lang.MissingPropertyException e) {
			println "ResourceProperties: Property \"" + name + "\" does not exist in properties file \"" + propsFileName + "\"."
			return false
		}
		def booleanTrue = booleanValue ==~ /^[Tt][Rr][Uu][Ee]/
		if (booleanTrue)
			return true
		def booleanFalse = booleanValue ==~ /^[Ff][Aa][Ll][Ss][Ee]/
		if (booleanFalse)
			return false
	}	// End getBooleanProperty method
	
	//*********************************************************************************************************************************
	// Method: backup - Make a backup copy within the same directory of the properties file associated with the declared 
	//		            Resource Properties object
	//*********************************************************************************************************************************
	public int backup() { 
		try {
			backup_propsFileObj = new File(backup_propsFileName);
			if (!backup_propsFileObj.exists())
				org.apache.commons.io.FileUtils.copyFile(propsFileObj,backup_propsFileObj);	// Returns nothing whether successful or not
			rp_rc = 1;
		}
       	catch (IOException e) {
       		rp_rc = -1;
  			e.printStackTrace();
       	}		
       	return rp_rc;	
	}	// End backup method
	
	//*********************************************************************************************************************************
	// Method: encodeAll - Encode all password properties ("_passwd") in the Properties object and write the sorted entries to the file
	//*********************************************************************************************************************************
	public int encodeAll()  {
		String [] keys = keySet().toArray()
		for (String key : keys)  {
			int lastUnderBar = key.lastIndexOf("_")
			if (lastUnderBar > 0)  {
				String type = key.substring(lastUnderBar)
				if (type == "_passwd")  {
					String unencodedPasswd = get(key)
					if ( !(unencodedPasswd.startsWith("{sas002}")) )  {
						String encodedPasswd = com.sas.util.SasPasswordString.encode(com.sas.util.SasPasswordString.SAS002_ENCODING, unencodedPasswd)
						put(key, encodedPasswd)
					}
				}
			}
		}
		return writeProps()
	}	// End encodeAll method
	
	//*********************************************************************************************************************************
	// Method: clearAll - Nullify the values for all "_userid" and "_passwd" properties entries in the Properties object and write the 
	//					  sorted entries to the file
	//*********************************************************************************************************************************
	public int clearAll()  {
		String [] keys = keySet().toArray()
		for (String key : keys)  {
			int lastUnderbar = key.lastIndexOf("_")
			if (lastUnderbar > 0) {
				String keyType = key.substring(lastUnderbar)
				if (keyType == "_userid" || keyType == "_passwd" )
					put(key, "")
			}
		}
		return writeProps()
	}	// End clearAll method
	
//*********************************************************************************************************************************
//  Load all property key-value pairs from the Properties object and create a hashmap entry for each 
//	public void loadHashMapFromProps() {
//		Set keys = keySet();
//		Iterator itr = keys.iterator();
//		while (itr.hasNext()) {
//			String key = (String) itr.next();
//			String value = getProperty(key);
//			if (value)
//				setProperty(key, value);
//		}
//	}	// End loadProperties method	

//*********************************************************************************************************************************
//  For testing:
//	public static void main(String[] args) {
//		ResourceProperties rp = new ResourceProperties("C:\\junk\\test.properties")
//		rp.setPropertyValue("jjj", "555")
//		rp.setPropertyValue("ddd", "777")
//		rp.setPropertyValue("ooo", "888")
//		rp.setPropertyValue("aaa", "111")
//		rp.setPropertyValue("mmm", "333")
//		rp.setPropertyValue("ccc", "222")
//		String x = rp.getProperty("zzz")
//		println "**DEBUG** main: x=${x}"
//		rp.toString()
//		println "**DEBUG** main: Write to the file..."
//		rp.writeProps()
//		println "**DEBUG** main: After store: " + rp.toString()
//	}
	
}	// End class ResourceProperties