// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: ActivationSpec
// Purpose: Encapsulate the attributes and methods used to manage and configure a Activation Specification
// Methods:
//		create - Create an Activation Specification definition
//		delete - Delete an Activation Specification definition
// ****************************************************************************************************************

class ActivationSpec extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc	// Object used to run an external command using Ant exec task
	def resourcePropsFileName
	def globalPropsFileName
	def servername
	def scope
	def scope_string
	def config_target_name				// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def SIBusname
	def jndiName
	def actSpecName
	def actSpecDestType	
	def actSpecDestJndiName
	String command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command 
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution	

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	// Global properties are fetched and initialized by the base Resource class
	public ActivationSpec(GlobalProperties parmGblProps) {
		super(parmGblProps)		// Load the global properties using the abstract base class CTOR
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name		
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands		
	}

	// *************************************************************************************
	// Method to create a JMS Activation Specification 
	// General flow:
	//	1. Run the createSIBJMSActivationSpec.jy Jython script
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 				= resourceProps.getProperty("servername")
		SIBusname 				= resourceProps.getProperty("SIBusname")
		jndiName				= resourceProps.getProperty("pjndiName")
		actSpecName 			= resourceProps.getProperty("pname")
		actSpecDestType 		= resourceProps.getProperty("pdestinationType")
		actSpecDestJndiName		= resourceProps.getProperty("pdestinationJndiName")
		scope 					= resourceProps.getProperty("scope")
		scope_string 			= resourceProps.getProperty("scope_string")
		config_target_name		= resourceProps.getProperty("config.target.name")
		if (DEBUG) {
			logger.debug("ActivationSpec-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("ActivationSpec-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("ActivationSpec-create: appsrvnodename=${appsrvnodename}")
			logger.debug("ActivationSpec-create: servername=${servername}")
			logger.debug("ActivationSpec-create: SIBusname=${SIBusname}")
			logger.debug("ActivationSpec-create: jndiName=${jndiName}")
			logger.debug("ActivationSpec-create: actSpecName=${actSpecName}")
			logger.debug("ActivationSpec-create: actSpecDestType=${actSpecDestType}")
			logger.debug("ActivationSpec-create: actSpecDestJndiName=${actSpecDestJndiName}")
			logger.debug("ActivationSpec-create: scope=${scope}")
			logger.debug("ActivationSpec-create: scope_string=${scope_string}")
			logger.debug("ActivationSpec-create: script_ext=${script_ext}")
			logger.debug("ActivationSpec-create: config_target_name=${config_target_name}")
			logger.debug("ActivationSpec-create: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("ActivationSpec-create: websphere_install_dir=${websphere_install_dir}")
			logger.debug("ActivationSpec-create: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("ActivationSpec-create: dmgrprotocol=${dmgrprotocol}")
			logger.debug("ActivationSpec-create: dmgrport=${dmgrport}")
			logger.debug("ActivationSpec-create: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("ActivationSpec-create: wsadminOptionsList=${wsadminOptionsList}")
		}		
		
		logger.info("ActivationSpec-create: Create the JMS ${actSpecDestType} Activation Specification \"${actSpecName}\" on \"${SIBusname}\" at scope \"${scope_string}\"...")  
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "createSIBJMSActivationSpec",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] ) 
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "ActSpecCreate_${actSpecName}_output", "ActSpecCreate_${actSpecName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("ActivationSpec-create: Command Return code: " + command_retcode)

		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the createSIBJMSActivationSpec.jy Jython proc completed successfully 
			if (  command_output.contains("createSIBJMSActivationSpec: Activation Spec \"${actSpecName}\" was successfully created.") ) { 
				method_MSG =  "ActivationSpec-create: ===> createSIBJMSActivationSpec.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** ActivationSpec-create: createSIBJMSActivationSpec.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** ActivationSpec-create: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END create method		

	// *************************************************************************************
	// Method to delete a JMS Activation Specification 
	// General flow:
	//	1. Run the deleteSIBJMSActivationSpec.jy Jython script
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties	
		servername				= resourceProps.getProperty("servername")
		actSpecName 			= resourceProps.getProperty("actSpecName")
		scope 					= resourceProps.getProperty("scope")
		scope_string 			= resourceProps.getProperty("scope_string")
		config_target_name		= resourceProps.getProperty("config.target.name")
		if (DEBUG) {
			logger.debug("ActivationSpec-delete: globalPropsFileName=${globalPropsFileName}")
			logger.debug("ActivationSpec-delete: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("ActivationSpec-delete: appsrvnodename=${appsrvnodename}")
			logger.debug("ActivationSpec-delete: servername=${servername}")
			logger.debug("ActivationSpec-delete: actSpecName=${actSpecName}")
			logger.debug("ActivationSpec-delete: scope=${scope}")
			logger.debug("ActivationSpec-delete: scope_string=${scope_string}")
			logger.debug("ActivationSpec-delete: config_target_name=${config_target_name}")
			logger.debug("ActivationSpec-delete: script_ext=${script_ext}")
			logger.debug("ActivationSpec-delete: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("ActivationSpec-delete: websphere_install_dir=${websphere_install_dir}")
			logger.debug("ActivationSpec-delete: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("ActivationSpec-delete: dmgrprotocol=${dmgrprotocol}")
			logger.debug("ActivationSpec-delete: dmgrport=${dmgrport}")
			logger.debug("ActivationSpec-delete: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("ActivationSpec-delete: wsadminOptionsList=${wsadminOptionsList}")			
		}		
		
		logger.info("ActivationSpec-delete: Delete the JMS ${actSpecDestType} Activation Specification \"${actSpecName}\" on \"${SIBusname}\" at scope \"${scope_string}\"...")  
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "deleteSIBJMSActivationSpec",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "ActSpecDelete_${actSpecName}_output", "ActSpecDelete_${actSpecName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 	
		logger.info("ActivationSpec-delete: Command Return code: " + command_retcode)

		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteSIBJMSActivationSpec.jy Jython proc completed successfully 
			if (  command_output.contains("deleteSIBJMSActivationSpec: Activation Spec \"${actSpecName}\" was successfully deleted.") ||
				  command_output.contains("deleteSIBJMSActivationSpec: An Activation Spec named \"${actSpecName}\" was not found; nothing to delete.") ) { 
				method_MSG =  "ActivationSpec-delete: ===> deleteSIBJMSActivationSpec.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** ActivationSpec-delete: deleteSIBJMSActivationSpec.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** ActivationSpec-delete: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END delete method
		
}	// END ActivationSpec class