// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: LoginModule
// Purpose: Encapsulate the attributes and methods used to manage and configure a JAAS Login Module definition
// Methods:
//		create 		- create a JAAS LoginModule entry
//		delete 		- delete a JAAS LoginModule entry
// ****************************************************************************************************************

class LoginModule extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc
	def globalPropsFileName
	def resourcePropsFileName	
	def loginModuleIdentifier
	def jaasAlias
	def jaasModuleClassName
	def jaasModuleFlag
	def jaasHost
	def jaasPort
	def jaasRepository
	def jaasDomain
	def jaasAliasDomain
	def jaasDebug
	def jaasHoldOpenConnection
	def jaasTrustedUserid
	def jaasTrustedPasswd
	def jaasCredentialsRequired
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command  
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution				

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	public LoginModule(GlobalProperties parmGblProps) {
		super(parmGblProps)
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands
	}	// END CTOR

	// *************************************************************************************
	// Method to create a JAAS Login Module entry
	// General flow:
	//	1. Add a JAAS Login Module entry using the addJaasLoginModule.jy script
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 		= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		loginModuleIdentifier 		= resourceProps.getProperty("loginModuleIdentifier")
		jaasAlias 					= resourceProps.getProperty("JaasAlias")
		jaasModuleClassName 		= resourceProps.getProperty("JaasModuleClassName")
		jaasModuleFlag 				= resourceProps.getProperty("JaasModuleFlag")
		jaasHost 					= resourceProps.getProperty("JaasHost")
		jaasPort 					= resourceProps.getProperty("JaasPort")
		jaasRepository 				= resourceProps.getProperty("JaasRepository")
		jaasDomain 					= resourceProps.getProperty("JaasDomain")
		jaasAliasDomain 			= resourceProps.getProperty("JaasAliasDomain")
		jaasDebug 					= resourceProps.getProperty("JaasDebug")
		jaasHoldOpenConnection 		= resourceProps.getProperty("JaasHoldOpenConnection")
		jaasCredentialsRequired		= resourceProps.getProperty("JaasCredentialsRequired")
		config_target_name			= resourceProps.getProperty("config.target.name")
		
		if (DEBUG) {
			logger.debug("LoginModule-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("LoginModule-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("LoginModule-create: loginModuleIdentifier=${loginModuleIdentifier}")
			logger.debug("LoginModule-create: jaasAlias=${jaasAlias}")
			logger.debug("LoginModule-create: jaasModuleClassName=${jaasModuleClassName}")
			logger.debug("LoginModule-create: jaasModuleFlag=${jaasModuleFlag}")
			logger.debug("LoginModule-create: jaasHost=${jaasHost}")
			logger.debug("LoginModule-create: jaasPort=${jaasPort}")
			logger.debug("LoginModule-create: jaasRepository=${jaasRepository}")
			logger.debug("LoginModule-create: jaasDomain=${jaasDomain}")
			logger.debug("LoginModule-create: jaasAliasDomain=${jaasAliasDomain}")
			logger.debug("LoginModule-create: jaasDebug=${jaasDebug}")
			logger.debug("LoginModule-create: jaasHoldOpenConnection=${jaasHoldOpenConnection}")
			logger.debug("LoginModule-create: jaasCredentialsRequired=${jaasCredentialsRequired}")
			logger.debug("LoginModule-create: script_ext=${script_ext}")
			logger.debug("LoginModule-create: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("LoginModule-create: websphere_install_dir=${websphere_install_dir}")
			logger.debug("LoginModule-create: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("LoginModule-create: dmgrprotocol=${dmgrprotocol}")
			logger.debug("LoginModule-create: dmgrport=${dmgrport}")
			logger.debug("LoginModule-create: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("LoginModule-create: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("LoginModule-create: config_target_name=${config_target_name}")			
		}
		
		// Obtain the SAS trusted userId and password from the credentials.properties file for all except "UsernamePassword" LoginModule
		if (jaasCredentialsRequired) {
			ResourceProperties credentialsProps = new ResourceProperties(websphere_scripting_credentials)	// Object used to read the websphere_scripting_credentials properties file
			String jaasTrustedUserid = credentialsProps.getProperty("LoginModule.create_CELL_SAS-Trusted-user_userid")
			String jaasTrustedPasswd = credentialsProps.getProperty("LoginModule.create_CELL_SAS-Trusted-user_passwd")
			// If userId and/or password are null in the credentials.properties file, can't continue because Login Module needs credentials
			def jaasAliasErrorType = ""
			if (jaasTrustedUserid == "")
				jaasAliasErrorType = "userId"
			if (jaasTrustedPasswd == "")
				jaasAliasErrorType += " password"
			if (jaasAliasErrorType) {
				method_MSG = "**SEVERE** LoginModule-create: JAAS Login Module \"${jaasAlias}\" requires SAS Trusted userId and password, but ${jaasAliasErrorType} not provided; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
		}
		
		// 1. Add a JAAS Login Module entry 
		logger.info("LoginModule-create: Add JAAS Login Module entry \"${loginModuleIdentifier}\" using the wsadmin${script_ext} command...")
		resourcePropsFileName = resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "addJaasLoginModule",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "LoginModuleCreate_${loginModuleIdentifier}_output", "LoginModuleCreate_${loginModuleIdentifier}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("LoginModule-create: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the addJaasLoginModule.jy Jython proc completed successfully
			if (  command_output.contains("addJaasLoginModule: JAAS Login Module modifications completed.") ) { 
				method_MSG =  "LoginModule-create: ===> addJaasLoginModule.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** LoginModule-create: addJaasLoginModule.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** LoginModule-create: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		logger.info("${method_MSG}")
		return method_RC	// Return to driver indicating failure
	}	// END create method

	// *************************************************************************************
	// Method to delete a JAAS Login Module entry
	// General flow:
	//	1. Remove a JAAS Login Module entry using the deleteJaasLoginModule.jy script
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		loginModuleIdentifier 	= resourceProps.getProperty("loginModuleIdentifier")
		jaasAlias 				= resourceProps.getProperty("JaasAlias")
		jaasModuleClassName 	= resourceProps.getProperty("JaasModuleClassName")
		config_target_name		= resourceProps.getProperty("config.target.name")
		
		if (DEBUG) {
			logger.debug("LoginModule-delete: globalPropsFileName=${globalPropsFileName}")
			logger.debug("LoginModule-delete: resourcePropsFileName=${resourcePropsFileName}")		
			logger.debug("LoginModule-delete: loginModuleIdentifier=${loginModuleIdentifier}")
			logger.debug("LoginModule-delete: jaasAlias=${jaasAlias}")
			logger.debug("LoginModule-delete: jaasModuleClassName=${jaasModuleClassName}")
			logger.debug("LoginModule-delete: script_ext=${script_ext}")
			logger.debug("LoginModule-delete: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("LoginModule-delete: websphere_install_dir=${websphere_install_dir}")
			logger.debug("LoginModule-delete: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("LoginModule-delete: dmgrprotocol=${dmgrprotocol}")
			logger.debug("LoginModule-delete: dmgrport=${dmgrport}")
			logger.debug("LoginModule-delete: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("LoginModule-delete: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("LoginModule-delete: config_target_name=${config_target_name}")
		}
		
		// 1. Remove a JAAS Login Module entry 
		logger.info("LoginModule-delete: Remove JAAS Login Module entry \"${loginModuleIdentifier}\" using the wsadmin${script_ext} command...")
		resourcePropsFileName = resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "deleteJaasLoginModule",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "LoginModuleDelete_${loginModuleIdentifier}_output", "LoginModuleDelete_${loginModuleIdentifier}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("LoginModule-delete: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteJaasLoginModule.jy Jython proc completed successfully
			if (  command_output.contains("deleteJaasLoginModule: JAAS Application Login Module \"${jaasModuleClassName}\" has been removed from JAAS Alias \"${jaasAlias}\"") ) { 
				method_MSG =  "LoginModule-delete: ===> deleteJaasLoginModule.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** LoginModule-delete: deleteJaasLoginModule.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** LoginModule-delete: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		logger.info("${method_MSG}")
		return method_RC	// Return to driver indicating failure
	}	// END delete method
			
}	// END LoginModule class