// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: NodeProfile
// Purpose: Encapsulate the attributes and methods used to manipulate a managed node profile for WAS
// Methods:
//		create 		- create the node profile
//		delete 		- delete the node profile
//		start  		- start the nodeagent server 
//		stop		- stop the nodeagent server
//		update		- update the nodeagent server Http ports and JVM generic arguments
//		updateProps	- update the node profile security and timeout properties
//		validate	- validate the profileRegistry
// ****************************************************************************************************************
public class NodeProfile extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc	// Object used to run an external command using Ant exec task
	def globalPropsFileName
	def resourcePropsFileName
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	String create_nodeprofile_response_file
	String command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command  
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution	
		
	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	public NodeProfile(GlobalProperties parmGblProps) {
		super(parmGblProps)
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands			
	}	// END CTOR
	
	// *************************************************************************************
	// Method to create the node profile
	// General flow:
	//	1. Validate profileRegistry
	//	2. Create node profile
	//  3. Update profile properties for security and timeout values	
	//	4. Stop nodeagent server, if running
	//  5. Federate the node into the cell via the addNode command	
	//	6. Update nodeagent server Http ports and JVM arguments
	//	7. Synchronize the dmgr and nodeagent
	//  8. Create Windows Service for nodeagent, if necessary
	//	9. Start nodeagent server
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("NodeProfile-create: Create the managed node profile \"${profilename}\"...")
		// Method variables
		create_nodeprofile_response_file = resourceProps.getProperty("create.nodeprofile.response.file")
		config_target_name 				 = resourceProps.getProperty("config.target.name")
		logger.info("NodeProfile-create: create_nodeprofile_response_file=${create_nodeprofile_response_file}")
		
		// 1. Validate profileRegistry
		def method_RC = validate(resourceName,resourceProps)	// Validate the profileRegistry and set profileStatus to "proceed", "reuse", "error", "warning", or "unknown"	
		if (method_RC != 1) {	// If status of profileRegistry indicates an error,
			return method_RC	// Return to driver indicating failure
		}
		
		// 2. Create node profile
		logger.info("=================================================")
		logger.info("NodeProfile-create: Issue manageprofiles command to create node profile...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}manageprofiles${script_ext}"
		optionsArray = ["-response", "${create_nodeprofile_response_file}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfleCreate_${profilename}_output", "NodeProfileCreate_${profilename}_result")	// Run the manageprofiles command to create the dmgr profile
		command_output  = cmdProc.process_output				// Get the output from the command
		logger.info("NodeProfile-create: Command Return code: " + command_retcode)
		
		// Evaluate results from manageprofiles command used to create the node profile:
		// Determine if we can look at text output from the manageprofiles command when locale == "en", or only at the return code when locale != "en"
		def profileStatus = ""
		if (os_localhost_default_locale == "en") {
			if (command_output.contains("INSTCONFSUCCESS: Success: Profile ${profilename} now exists."))
				profileStatus = "success"
			else if (command_output.contains("profileName: ${profilename} is already in use. Specify another name."))
				profileStatus = "reuse"
			else if (command_output.contains("profileName: ${profilename} is already in use."))
				profileStatus = "validateAndUpdateRegistryRequired" 
			else if (command_output.contains("INSTCONFFAILED: The profile could not be created."))
				profileStatus = "error"
			else
				profileStatus = "unknown" 
		} else {	// locale is non-English
			if (command_retcode == 0)  // Assume if the return code from manageprofiles -create was good, that this is success
				profileStatus = "success"
			else // Otherwise, assume a non-zero return code from manageprofiles -create is an error
				profileStatus = "error"
		}	
		// Prepare any error messages and return codes, if an severe error has occurred.
		logger.info("NodeProfile-create: manageprofiles create \"${profilename}\" profile status is: ${profileStatus}...")
		switch (profileStatus) {
			case ("success"):
				method_MSG = "NodeProfile-create: Creation of profile \"${profilename}\" was SUCCESSFUL; processing can continue."
				method_RC = 1
				break
				
			case ("reuse"):
				method_MSG = "NodeProfile-create: A valid profile named \"${profilename}\" already exists; it is being REUSED."
				method_RC = 1
				break

			case ("validateAndUpdateRegistryRequired"):
				method_MSG = "**SEVERE** NodeProfile-create: The \"${profilename}\" profile is invalid or possibly corrupted.\n" +
							 "           Please run the WebSphere \"${websphere_install_dir}${File.separator}bin${File.separator}manageprofiles${script_ext} -validateAndUpdateRegistry\" command and click \"Retry\" after the profileRegistry has been validated."
				method_RC = -99
				break

			case ("error"):
				method_MSG = "**SEVERE** NodeProfile-create: The \"manageprofiles${script_ext}\" command has generated a serious error.\n" +
							 "           Please check the \"${websphere_install_dir}${File.separator}logs${File.separator}manageprofiles\" log directory for the reason before attempting a \"Retry\"."
				method_RC = -99
				break

			default:	// "unknown"
				method_MSG = "**SEVERE** NodeProfile-create: The WebSphere \"manageprofiles${script_ext}\" command has generated an unknown error.\n" +
							 "           Please ensure the WebSphere \"profileRegistry.xml\" file and the \"profiles\" directory are valid and do not contain references to the \"${profilename}\" profile before attempting \"Retry\".\n" +
							 "           Also, ensure your userId has write privileges to the appropriate WebSphere directories."
				method_RC = -99
		}	// END switch
		if (method_RC < 1) { 	// If status of profileRegistry indicates error,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
			logger.info("NodeProfile-create: ===> manageprofiles command to create node profile COMPLETED SUCCESSFULLY.")
		}			
		
		// 3. Update properties file in the profile for security and timeout values
		method_RC = updateProps(resourceName, resourceProps)
		if (method_RC != 1) {	// If update of security or timeout values indicates an error,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}
			
		// 4. Stop nodeagent, if running
		method_RC = stop(resourceName, resourceProps) 
		if (method_RC != 1)	{ 	// If status of profileRegistry indicates an error,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}		
		
		// 5. Federate the node into the cell via the addNode command
		method_RC = addNode(resourceName, resourceProps)
		if (method_RC != 1)	 {	// If addNode failed, 
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}
	
		// 6. Update nodeagent server Http ports and JVM arguments
		method_RC = update(resourceName, resourceProps) 
		if (method_RC != 1)	{	// If update of HTTP Ports and JVM args had an error,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}

		// 7. Synchronize the dmgr and nodeagent to ensure new nodeagent configuration in master repository gets sent to node repository
		method_RC = syncNode(resourceName, resourceProps)
		if (method_RC != 1)	{	// If syncNode command failed,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}
		
		// 8. Create Windows Service for nodeagent, if necessary
		method_RC = createService(resourceName, resourceProps)
		if (method_RC != 1)	{	// If WASService command failed,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}
			
		// 9. Start nodeagent
		method_RC = start(resourceName, resourceProps)
		if (method_RC != 1)		// If startNode command failed,
			logger.error("${method_MSG}")
		else
			logger.info("${method_MSG}")	
		return method_RC
	}	// END create method

	// *************************************************************************************
	// Method to delete the node profile
	// General flow:
	//  1. Determine if the profile to be deleted actually exists
	//	2. Stop nodeagent, if running
	//	3. Delete node profile
	//	4. Remove node profile directory	
	public int delete(String resourceName, ResourceProperties resourceProps) {
		String nodeProfileDirectoryName = "${websphere_profile_dir}${File.separator}${profilename}"
		config_target_name = resourceProps.getProperty("config.target.name")
		logger.info("NodeProfile-delete: Delete profile \"${profilename}\"...")
		
		//  1. Determine if the profile to be deleted actually exists:
		if (!(new File(nodeProfileDirectoryName).exists())) {
			method_MSG = "**SEVERE** NodeProfile-delete: WAS profile directory \"${nodeprofileDirectoryName}\" does not exist;  Nothing to delete."
			method_RC = -99
 			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure      		
      	}

		//	2. Stop nodeagent, if running
		method_RC = stop(resourceName, resourceProps) 
		if (method_RC != 1) {	// If stopNode command failed,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}	
		
		//	3. Delete node profile
		logger.info("=================================================")
		logger.info("NodeProfile-delete: Issue manageprofiles command to delete profile \"${profilename}\"...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}manageprofiles${script_ext}" 
		optionsArray = ["-delete", "-profileName", "${profilename}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfileDelete_${profilename}_output", "NodeProfileDelete_${profilename}_result")	// Run the manageprofiles command to delete the dmgr profile
		command_output  = cmdProc.process_output				// Get the output from the command
		logger.info("NodeProfile-delete: Command Return code: " + command_retcode)	
		
		// Evaluate results from manageprofiles command used to delete the node profile:
		// Determine if we can look at text output from the manageprofiles command when locale == "en", or only at the return code when locale != "en"
		def profileStatus = ""
		if (os_localhost_default_locale == "en") {	// Locale is English, we can look at text in the output message from manageprofiles
			if (command_output.contains("SUCCESS: Success: The profile no longer exists."))
				profileStatus = "success"
			else 
				profileStatus = "error"
		} else {		// Locale is non-English, we have to base success on return codes only
			if (command_retcode == 0)  // Assume if the return code from manageprofiles -delete was good, that this is success
				profileStatus = "success"
			else 		// Otherwise, assume a non-zero return code from manageprofiles -delete is an error
				profileStatus = "error"			
		}
		
		//	4. Remove node profile directory
		logger.info("=================================================")
		if (profileStatus.equals("success")) {
			logger.info("NodeProfile-delete: Profile \"${profilename}\" has been deleted; now remove the associated profiles directory...")
			try {
				File nodeProfileDirectory = new File(nodeProfileDirectoryName);
				org.apache.commons.io.FileUtils.deleteDirectory(nodeProfileDirectory);	// Returns nothing whether successful or not
				logger.info("NodeProfile-delete: node profile directory \"${nodeProfileDirectoryName}\" has been removed.")
				method_RC = 1;
			}
       		catch (IOException e) {
       			method_RC = -99;
  				e.printStackTrace();
       		}		
			method_MSG = "NodeProfile-delete: ===> Delete of profile \"${profilename}\" COMPLETED SUCCESSFULLY" 
		} else {
			method_MSG = "NodeProfile-delete: ===> Delete of profile \"${profilename}\" FAILED; Check the \"${websphere_install_dir}${File.separator}logs${File.separator}manageprofiles\" log directory for the reason."
			method_RC = -99
		}
		logger.info("${method_MSG}")
		return method_RC
	}	// END delete method

	// ***********************************************************************************
	// Method to start the nodeagent server via startNode command
	public int start(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		logger.info("NodeProfile-start: Start the ${nodeagent} server using startNode command...")
		config_target_name = resourceProps.getProperty("config.target.name")
 		command = "${websphere_install_dir}${File.separator}bin${File.separator}startNode${script_ext}"
		optionsArray = ["-profileName", "${profilename}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfileStart_${appsrvnodename}_output", "NodeProfileStart_${appsrvnodename}_result")	// Run the manageprofiles command to start the dmgr server
		command_output  = cmdProc.process_output				// Get the output from the command
		logger.info("NodeProfile-start: Command Return code: " + command_retcode)						
		// Check if "ADMU3000I: Server {0} open for e-business; process id is {1}" or "ADMU3027E: An instance of the server may already be running: {0}"  
		if (command_output.contains("ADMU3000I") || command_output.contains("ADMU3027E")) { 
			method_MSG = "NodeProfile-start: ===> startNode${script_ext} COMPLETED SUCCESSFULLY"
			method_RC = 1
		} else { 
			method_MSG = "**SEVERE** NodeProfile-start: startNode${script_ext} command FAILED; Review WebSphere logs at \"${websphere_install_dir}${File.separator}profiles${File.separator}${profilename}${File.separator}logs\" for analysis."
			method_RC = -99
		}		
		return method_RC   		
	}	// END start method
	
	// ***********************************************************************************
	// Method to stop the nodeagent server via stopNode command
	public int stop(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		logger.info("NodeProfile-stop: Stop the ${nodeagent} server using stopNode command...")
		config_target_name = resourceProps.getProperty("config.target.name")
 		command = "${websphere_install_dir}${File.separator}bin${File.separator}stopNode${script_ext}"
		optionsArray = ["-profileName", "${profilename}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfileStop_${appsrvnodename}_output", "NodeProfileStop_${appsrvnodename}_result")	// Run the manageprofiles command to start the dmgr server
		command_output  = cmdProc.process_output				// Get the output from the command
		logger.info("NodeProfile-stop: Command Return code: " + command_retcode)
				
		// Check if "ADMU4000I: Server {0} stop completed." or 
		//          "ADMU0509I: The {0} "{1}" cannot be reached. It appears to be stopped." or
		//          "ADMU3522E: No server by this name in the configuration: {0}"    
		if ( command_output.contains("ADMU4000I") || 
		     command_output.contains("ADMU0509I") ||
		     command_output.contains("ADMU3522E") ) {
			   method_MSG = "NodeProfile-stop: ===> stopNode${script_ext} COMPLETED SUCCESSFULLY"
			   method_RC = 1
		} else {
			method_MSG = "**SEVERE** NodeProfile-stop: stopNode${script_ext} command FAILED; Review WebSphere logs at \"${websphere_install_dir}${File.separator}profiles${File.separator}${profilename}${File.separator}logs\" for analysis."
			method_RC = -99
		}	
		return method_RC 	
	}	// END stop method

	// ***********************************************************************************
	// Method to validate the profileRegistry, return 1: success, others: failed
	//		proceed - node profile name is valid
	//		error	- node profile appears in the list of profiles that are not valid
	//		warning	- node profile seems to be ok, but list of profiles that are not valid returned from the manageprofiles -validateRegistry command
	//		unknown - an unknown error has been returned by the manageprofiles -validateRegistry command
	public int validate(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")	
		logger.info("NodeProfile-validate: Validate the profileRegistry...") 
		config_target_name = resourceProps.getProperty("config.target.name")
		// Issue manageprofiles command to validate the profileRegistry...
		command = "${websphere_install_dir}${File.separator}bin${File.separator}manageprofiles${script_ext}" 
		optionsArray = ["-validateRegistry"]
		command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfileValidate_${profilename}_output", "NodeProfileValidate_${profilename}_result")	// Run the manageprofiles command to validate the profile registry		
		command_output  = cmdProc.process_output	// Get output from the command
		logger.info("NodeProfile-validate: Command Return code: " + command_retcode)
		if (command_retcode == 0)
			logger.info("NodeProfile-validate: ===> manageprofiles${script_ext} -validateRegistry command COMPLETED SUCCESSFULLY")

		// Determine if we can look at text output from the manageprofiles command when locale == "en", or only the return code when locale != "en"
		String profileStatus = ""
		if (os_localhost_default_locale == "en")  {
			if (command_output.contains("All of the profiles in the registry are valid."))  {
		        profileStatus = "proceed"
			} else {
			    if (command_output.contains("A list of profiles that are not valid follows:"))  {
			        if (command_output.contains(profilename))
			            profileStatus = "error"
			        else
			            profileStatus = "warning"
			    } else {
			        profileStatus = "unknown"
				}
			}
		} else { 	// locale is non-English
			if (command_retcode == 0) 
				profileStatus = "proceed"
			else 
				profileStatus = "error"
		}
		def val_RC=0
		switch (profileStatus) {
			case ("proceed"):
				method_MSG = "NodeProfile-create: WebSphere profile entries, if any, have been validated; An attempt to create \"${profilename}\" will proceed..."
				val_RC = 1
				break

			case ("warning"):
        		method_MSG = "**WARNING** NodeProfile-create The WebSphere profiles directory contains one or more possibly invalid profiles.\n" + 
 		                     "            This will not affect continued configuration of WebSphere for SAS, but this problem warrants further investigation." 
				val_RC = 1
				break
				
			case ("error"):
				method_MSG = "**SEVERE** NodeProfile-create \"${profilename}\" is invalid or possibly corrupted.\n " +
							 "           Please run the \"${websphere_install_dir}"+File.separator+"bin"+File.separator+"manageprofiles${script_ext} -validateAndUpdateRegistry\" to attempt correction of the WAS profileRegistry.xml file."
				val_RC = -99
				break

			default:	// "unknown"
				method_MSG = "**SEVERE** NodeProfile-create An unknown error has occurred attempting to validate the WAS profileRegistry.xml file.\n" +
							 "           Please run the \"${websphere_install_dir}"+File.separator+"bin"+File.separator+"manageprofiles${script_ext} -validateAndUpdateRegistry\" to attempt correction of the WAS profileRegistry.xml file.\n" +
							 "          Also, ensure your userId has write privileges to the appropriate WebSphere directories."
				val_RC = -99
		}	// End switch
		if (val_RC != 1) {	// If status of profileRegistry indicates an error,
			logger.error("${method_MSG}")
		} else {
			logger.info("${method_MSG}")
		}
		return val_RC
	}	// END validate method
		
	// ******************************************************************************************************
	// Method updateProps to update the node profile properties for SOAP and/or RMI security & timeout values
	public int updateProps(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		logger.info("NodeProfile-updateProps: Update the SOAP and/or RMI security and timeout properties for the node profile ...")
		config_target_name = resourceProps.getProperty("config.target.name")
		ResourceProperties credentialsProps		// Object used to read the websphere_scripting_credentials properties file
		String webappsrv_admin_userid = ""		// WAS Admin userId, required if security is enabled
		String webappsrv_admin_passwd = ""		// WAS Admin password, required if security is enabled
		logger.info("NodeProfile-updateProps: WAS Admin Security is enabled? ${webappsrv_admin_security_is_enabled}")
		if (webappsrv_admin_security_is_enabled) {
			credentialsProps = new ResourceProperties(websphere_scripting_credentials)
			webappsrv_admin_userid = credentialsProps.getProperty("DmgrProfile.create_CELL_${dmgrprofilename}_userid")
			webappsrv_admin_passwd = credentialsProps.getProperty("DmgrProfile.create_CELL_${dmgrprofilename}_passwd")
		}
		
		if (DEBUG) {
			logger.debug("NodeProfile-updateProps: webappsrv_admin_security_is_enabled=${webappsrv_admin_security_is_enabled}")
			logger.debug("NodeProfile-updateProps: dmgrprotocol=${dmgrprotocol}")
			logger.debug("NodeProfile-updateProps: nodesoapprops=${nodesoapprops}")
			logger.debug("NodeProfile-updateProps: nodesoaptimeout=${nodesoaptimeout}")
			logger.debug("NodeProfile-updateProps: nodermiprops=${nodermiprops}")
			logger.debug("NodeProfile-updateProps: nodermitimeout=${nodermitimeout}")
			logger.debug("NodeProfile-updateProps: nodeipcprops=${nodermiprops}")
			logger.debug("NodeProfile-updateProps: nodeipctimeout=${nodermitimeout}")
			logger.debug("NodeProfile-updateProps: config_target_name=${config_target_name}")
		}
	
		// Update the soap.client.props file (I found you have to update this, even if RMI protocol was selected because the stopManager,
		// stopServer, addNode, etc. go looking for the credentials in soap.client.props, although the wsadmin with -conntype RMI only looks in 
		// sas.client.props for the -user, -password credentials.)
		ResourceProperties soapProps = new ResourceProperties(nodesoapprops)	// Use the ResourceProperties object to manipulate the WAS soap.client.props file
		// Make a backup of the existing soap.client.props file before alteration:
		logger.info("NodeProfile-updateProps: Create backup of \"soap.client.props\" file before alteration.")
		soapProps.backup()
		
		// Update the timeout value for SOAP communications
		logger.info("NodeProfile-updateProps: Update SOAP timeout property in \"${nodesoapprops}\" file...")
		soapProps.setPropertyValue("com.ibm.SOAP.requestTimeout", nodesoaptimeout)
		
		// If WAS Admin Security is enabled, update the soap.client.props security properties to avoid anoying userID/password prompts
		if (webappsrv_admin_security_is_enabled) {
			logger.info("NodeProfile-updateProps: Set the SOAP security properties in \"soap.client.props\" file...")
			soapProps.setPropertyValue("com.ibm.SOAP.securityEnabled", "true")
			soapProps.setPropertyValue("com.ibm.SOAP.loginSource", "none")
			soapProps.setPropertyValue("com.ibm.SOAP.loginUserid", webappsrv_admin_userid)
			
			// Decode the admin password using the SAS decoder utility
			String decodedPassword = com.sas.util.SasPasswordString.decode(webappsrv_admin_passwd);
			soapProps.setPropertyValue("com.ibm.SOAP.loginPassword", decodedPassword)
		}
				
		// Write the property updates to the soap.client.props file
		logger.info("NodeProfile-updateProps: Update the SOAP security and/or timeout properties file \"soap.client.props\"...")
		if (soapProps.writeProps() == 1) {
			method_RC = 1
			method_MSG = "NodeProfile-updateProps: Update of SOAP \"soap.client.props\" file succeeded."
			// Run the WAS PropFilePasswordEncoder utility to ensure no clear-text password appears
			if (webappsrv_admin_security_is_enabled) {
				logger.info("${method_MSG}")
				command = "${websphere_install_dir}${File.separator}bin${File.separator}PropFilePasswordEncoder${script_ext}"
				optionsArray =["${nodesoapprops}", "com.ibm.SOAP.loginPassword"]
				command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfileUpdatePropsSoap_${profilename}_output", "NodeProfileUpdatePropsSoap_${profilename}_result", "n")	// Run PropFilePasswordEncoder utility to encode password ("n" input string required for response to question on exchange of keystores) 
				command_output  = cmdProc.process_output	// Get output from the command
				logger.info("NodeProfile-updateProps: Command Return code = " + command_retcode)
				if (command_retcode == 0) {
					method_RC = 1
					method_MSG = "NodeProfile-updateProps: PropFilePasswordEncoder utility has successfully encoded WAS Admin password."
				} else {
					method_RC = -1
					method_MSG = "**ERROR** NodeProfile-updateProps: PropFilePasswordEncoder utility produced non-zero return code."
				}					
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** NodeProfile-updateProps: Update of SOAP \"soap.client.props\" file failed."
		}	// END SOAP protocol properties updated
		
		if (method_RC == 1) {
			logger.info("${method_MSG}")
			// Update the ipc.client.props file
			ResourceProperties ipcProps = new ResourceProperties(nodeipcprops)
			logger.info("NodeProfile-updateProps: Create backup of \"ipc.client.props\" file before alteration.")
			ipcProps.backup()
			
			// Update the timeout value for IPC communications
			logger.info("NodeProfile-updateProps: Update IPC timeout property in \"${nodeipcprops}\" file...")
			ipcProps.setPropertyValue("com.ibm.IPC.requestTimeout", nodeipctimeout)
	
			if (webappsrv_admin_security_is_enabled) {
				logger.info("NodeProfile-updateProps: Set the IPC security properties in \"ipc.client.props\" file...")
				ipcProps.setPropertyValue("com.ibm.IPC.securityEnabled", "true")
				ipcProps.setPropertyValue("com.ibm.IPC.loginSource", "none")
				ipcProps.setPropertyValue("com.ibm.IPC.loginUserid", webappsrv_admin_userid)
				
				String decodedPassword = com.sas.util.SasPasswordString.decode(webappsrv_admin_passwd);
				ipcProps.setPropertyValue("com.ibm.IPC.loginPassword", decodedPassword)
			}
					
			logger.info("NodeProfile-updateProps: Update the IPC security and/or timeout properties file \"ipc.client.props\"...")
			if (ipcProps.writeProps() == 1) {
				method_RC = 1
				method_MSG = "NodeProfile-updateProps: Update of IPC \"ipc.client.props\" file succeeded."
				if (webappsrv_admin_security_is_enabled) {
					logger.info("${method_MSG}")
					command = "${websphere_install_dir}${File.separator}bin${File.separator}PropFilePasswordEncoder${script_ext}"
					optionsArray =["${nodesoapprops}", "com.ibm.IPC.loginPassword"]
					command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfileUpdatePropsIPC_${profilename}_output", "NodeProfileUpdatePropsIPC_${profilename}_result", "n")
					command_output  = cmdProc.process_output
					logger.info("NodeProfile-updateProps: Command Return code = " + command_retcode)
					if (command_retcode == 0) {
						method_RC = 1
						method_MSG = "NodeProfile-updateProps: PropFilePasswordEncoder utility has successfully encoded WAS Admin password."
					} else {
						method_RC = -1
						method_MSG = "**ERROR** NodeProfile-updateProps: PropFilePasswordEncoder utility produced non-zero return code."
					}
				}
			} else {
				method_RC = -1
				method_MSG = "**ERROR** NodeProfile-updateProps: Update of IPC \"ipc.client.props\" file failed."
			}
		}   // END Update the ipc.client.props file
		
		// Determine if the sas.client.props properties file has to be updated when using the RMI protocol:
		if (method_RC == 1 && dmgrprotocol == "RMI")  {	// If protocol is RMI, then also update the sas.client.props file
			logger.info("${method_MSG}")
			ResourceProperties rmiProps = new ResourceProperties(nodermiprops)	// Use the ResourceProperties object to manipulate the WAS sas.client.props file
			// Make a backup of the existing sas.client.props file before alteration:
			logger.info("NodeProfile-updateProps: Create backup of \"sas.client.props\" file before alteration.")
			rmiProps.backup()
			
			// Update the timeout value for RMI communications
			logger.info("NodeProfile-updateProps: Update RMI timeout property in \"${nodermiprops}\" file...")
			rmiProps.setPropertyValue("com.ibm.CORBA.requestTimeout", nodermitimeout)
			
			// If WAS Admin Security is enabled, update the sas.client.props security properties to avoid anoying userID/password prompts
			if (webappsrv_admin_security_is_enabled) {
				logger.info("NodeProfile-updateProps: Set the RMI security properties in \"sas.client.props\" file...")
				rmiProps.setPropertyValue("com.ibm.CORBA.securityEnabled", "true")
				rmiProps.setPropertyValue("com.ibm.CORBA.loginSource", "none")
				rmiProps.setPropertyValue("com.ibm.CORBA.loginUserid", webappsrv_admin_userid)
			
				// Decode the admin password using the SAS decoder utility
				String decodedPassword = com.sas.util.SasPasswordString.decode(webappsrv_admin_passwd)
				rmiProps.setPropertyValue("com.ibm.CORBA.loginPassword", decodedPassword)
			}
			
			// Write the property updates to the sas.client.props file
			logger.info("NodeProfile-updateProps: Update the RMI security and/or timeout properties file \"sas.client.props\"...")
			if (rmiProps.writeProps() == 1) {
				method_RC = 1
				method_MSG = "NodeProfile-updateProps: Update of RMI \"sas.client.props\" file succeeded."
				
				// If security is enabled, run the WAS PropFilePasswordEncoder utility to ensure no clear-text password appears
				if (webappsrv_admin_security_is_enabled) {
					logger.info("${method_MSG}")
					command = "${websphere_install_dir}${File.separator}bin${File.separator}PropFilePasswordEncoder${script_ext}"
					optionsArray = ["${nodermiprops}", "com.ibm.CORBA.loginPassword"]
					command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfileUpdatePropsRmi_${profilename}_output", "NodeProfileUpdatePropsRmi_${profilename}__result", "n")	// Run PropFilePasswordEncoder utility to encode password ("n" input string required for response to question on exchange of keystores) 
					command_output  = cmdProc.process_output	// Get output from the command
					logger.info("NodeProfile-updateProps: Command Return code: " + command_retcode)
					if (command_retcode == 0) {
						method_MSG = "NodeProfile-updateProps: PropFilePasswordEncoder utility has successfully encoded password."
						method_RC = 1
					} else {
						method_MSG = "**ERROR** NodeProfile-updateProps: PropFilePasswordEncoder utility produced non-zero return code."
						method_RC = -1
					}	
				}				
			} else {
				method_RC = -1
				method_MSG = "NodeProfile-updateProps: **ERROR** Update of RMI \"sas.client.props\" file failed."
			}
		}	// End RMI protocol
		if (method_RC == 1) {
			logger.info("${method_MSG}")
			method_MSG = "NodeProfile-updateProps: ===> Update of timeout and security properties COMPLETED SUCCESSFULLY."
			logger.info("${method_MSG}")
		} 
		return method_RC	
	}	// END updateProps method
	
	// ***********************************************************************************
	// Method to federate the node into the cell via the addNode command
	public int addNode(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		logger.info("NodeProfile-addNode: Federate the new node into the existing cell using the addNode command...")
		logger.info("NodeProfile-addNode: addNode${script_ext} command connects to ${dmgr} server in host \"${webappsrv_admin_host}\" using port \"${dmgrport}\".")
		config_target_name = resourceProps.getProperty("config.target.name")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}addNode${script_ext}"
		optionsArray = ["${webappsrv_admin_host}", "${dmgrport}", "-conntype", "${dmgrprotocol}", "-profileName", "${profilename}", "-includeapps", "-noagent"]
		command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfileAddNode_${appsrvnodename}_output", "NodeProfileAddNode_${appsrvnodename}__result")	// Run PropFilePasswordEncoder utility to encode password ("n" input string required for response to question on exchange of keystores) 
		command_output  = cmdProc.process_output	// Get output from the command
		logger.info("NodeProfile-addNode: Command Return code: " + command_retcode)
				
		// Check if "ADMU0003I: Node {0} has been successfully federated." or 
		//          "ADMU0033E: The node {0} already has been added to the cell." or 
		//          "ADMU0039E: ADMU0039E: There is already a node named {0} in the cell." messages indicate addNode was successful  
		if (command_output.contains("ADMU0003I") || command_output.contains("ADMU0033E") || command_output.contains("ADMU0039E")) {
			method_MSG = "NodeProfile-addNode: ===> addNode${script_ext} COMPLETED SUCCESSFULLY"
			method_RC = 1
		} else {
			method_MSG = "**SEVERE** NodeProfile-addNode: addNode${script_ext} command failed; Check WebSphere logs at \"${websphere_install_dir}${File.separator}logs${File.separator}manageprofiles\" for analysis."
			method_RC = -99
		}	
		return method_RC 	
	}	// END addNode method

	// ******************************************************************************************
	// Method to update the nodeagent server Http ports and JVM generic arguments
	public int update(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		logger.info("NodeProfile-update: Update the ${nodeagent} server Http ports and JVM generic arguments using the wsadmin command...")
		resourcePropsFileName = resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		config_target_name    = resourceProps.getProperty("config.target.name")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		optionsArray = [ "-conntype", "NONE", 
						 "-lang", "jython", 
						 "-f", "${websphere_wsadmin_jython}${File.separator}callMultJythonFunction.jy", 
						 "${websphere_wsadmin_jython}", 
						 "${resourcePropsFileName}",
						 "${globalPropsFileName}"  ] 
		command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfileUpdate_${appsrvnodename}_output", "NodeProfileUpdate_${appsrvnodename}_result")	// Run wsadmin client to execute Jython procs to update Http ports and JVM Generic Arguments
		command_output  = cmdProc.process_output	// Get output from the command
		logger.info("NodeProfile-update: Command Return code: " + command_retcode)			
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the setContainerHttpPorts.jy Jython proc completed successfully
			if (  command_output.contains("setHttpPort: Processing complete.") &&
                  command_output.contains("setContainerHttpPorts: Processing complete.")  )  { 
				method_MSG =  "NodeProfile-update: ===> setContainerHttpPorts.jy COMPLETED SUCCESSFULLY; Http ports for the ${nodeagent} server are updated."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** NodeProfile-update: setContainerHttpPorts.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				return method_RC
			}
			
			// Check if the setJvmArguments.jy Jython proc completed successfully
			if (command_output.contains("setJvmArguments: Modifications to JVM Arguments completed.")) {
				method_MSG = "NodeProfile-update: ===> setJvmArguments.jy COMPLETED SUCCESSFULLY; JVM generic arguments for the ${nodeagent} server are updated."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** NodeProfile-update: setJvmArguments.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				return method_RC
			}
			
			// Check if the setJMXClientTimeout.jy Jython proc completed successfully
			if (command_output.contains("setJMXClientTimeout: Set requestTimeout for JMX Clients completed.")) {
				method_MSG = "NodeProfile-update: ===> setJMXClientTimeout.jy COMPLETED SUCCESSFULLY; SOAP or RMI, IPC requestTimeout values for the ${nodeagent} server are updated."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** NodeProfile-update: setJMXClientTimeout.jy Jython processing FAILED; unable to continue."
				method_RC = -99
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** NodeProfile-update: wsadmin${script_ext} command failed; RC=" + command_retcode
		}
		return method_RC	
	}	// END update method
	
	// ***********************************************************************************
	// Method to synchronize the node and dmgr repositories via the syncNode command
	public int syncNode(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		logger.info("NodeProfile-syncNode: Synchronize the dmgr and node repositories using the syncNode${script_ext} command...")
		config_target_name = resourceProps.getProperty("config.target.name")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}syncNode${script_ext}"
		optionsArray = ["${webappsrv_admin_host}", "${dmgrport}", "-conntype", "${dmgrprotocol}", "-profileName", "${profilename}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfilesSyncNode_${appsrvnodename}_output", "NodeProfileSyncNode_${appsrvnodename}__result")	// Run PropFilePasswordEncoder utility to encode password ("n" input string required for response to question on exchange of keystores) 
		command_output  = cmdProc.process_output	// Get output from the command
		logger.info("NodeProfile-syncNode: Command Return code: " + command_retcode)
		// Check for "ADMU0402I: The configuration for node {0} has been synchronized with Deployment Manager {1}: {2}" message - indicates syncNode was successful  
		if (command_output.contains("ADMU0402I")) {
			method_MSG = "NodeProfile-syncNode: ===> syncNode${script_ext} COMPLETED SUCCESSFULLY"
			method_RC = 1
		} else {
			method_MSG = "**SEVERE** NodeProfile-syncNode: syncNode${script_ext} command FAILED; Check WebSphere logs at \"${websphere_profile_dir}${File.separator}${profilename}${File.separator}logs${File.separator}syncNode.log\" for analysis."
			method_RC = -99
		}	
		return method_RC 	
	}	// END syncNode method
	
	// ***********************************************************************************
	// Method to create a Windows service for the nodeagent via the WASService command
	public int createService(String resourceName, ResourceProperties resourceProps) {
		if (os_localhost_type == "win") {
			logger.info("=================================================")
			logger.info("NodeProfile-createService: Create a Windows service for the ${nodeagent} using the WASService command...")
			config_target_name = resourceProps.getProperty("config.target.name")
			command = "${websphere_install_dir}${File.separator}bin${File.separator}WASService"
			optionsArray = [ "-add", "${appsrvnodename}", 
							 "-servername", "${nodeagent}", 
							 "-restart", "true", 
							 "-startType", "automatic", 
							 "-profilePath", "${websphere_profile_dir}${File.separator}${profilename}"]
			command_retcode = cmdProc.run(".", command, optionsArray, "NodeProfilesCreateService_${appsrvnodename}_output", "NodeProfileCreateService_${appsrvnodename}__result")	// Run PropFilePasswordEncoder utility to encode password ("n" input string required for response to question on exchange of keystores) 
			command_output  = cmdProc.process_output	// Get output from the command
			logger.info("NodeProfile-createService: Command Return code: " + command_retcode)
			// Determine if the WASService output can be examined if locale is "en"
			if (os_localhost_default_locale == "en") {
				// Check for messages from WASService which indicate service was created successfully  
				if (command_output.contains("IBM WebSphere Application Server V6.1 - ${appsrvnodename} service successfully added.") ||
					command_output.contains("IBM WebSphere Application Server V7.0 - ${appsrvnodename} service successfully added.") ||
					command_output.contains("IBM WebSphere Application Server V8.0 - ${appsrvnodename} service successfully added.") ||
					command_output.contains("Service already exists, updating parameters...")) {
					  method_MSG = "NodeProfile-createService: ===> WASService COMPLETED SUCCESSFULLY"
					  method_RC = 1
				} else {
					method_MSG = "**SEVERE** NodeProfile-createService: WASService command FAILED; Check Windows Event Log and WebSphere logs at \"${websphere_install_dir}${File.separator}logs${File.separator}\" for analysis."
					method_RC = -99
				}
			} else {	// Non-English locale means we can only check the return code from WASService 
				if (command_retcode == 0) {
					method_MSG = "NodeProfile-createService: ===> WASService COMPLETED SUCCESSFULLY"
					method_RC = 1					
				} else {
					method_MSG = "**SEVERE** NodeProfile-createService: WASService command FAILED; Check Windows Event Log and WebSphere logs at \"${websphere_install_dir}${File.separator}logs${File.separator}\" for analysis."
					method_RC = -99	
				}
			}
		} else {	// Not running on a Windows system, so no need to create a service
			method_MSG = "NodeProfile-createServer: Service for ${nodeagent} server is only required for Windows systems."
			method_RC = 1
		}			
		return method_RC 	
	}	// END createService method	
	
}	// END NodeProfile class