// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: SIBus
// Purpose: Encapsulate the attributes and methods used to manage and configure a Service Integration Bus
// Methods:
//		create 		- create a Service Integration Bus
//		delete 		- delete a Service Integration Bus
// ****************************************************************************************************************

class SIBus extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc	// Object used to run an external command using Ant exec task
	def globalPropsFileName
	def resourcePropsFileName	
	def	servername
	def SIBusname
	Boolean reuseSIBusMember
	def ssl_signer_exchange_prompt_response
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	String command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command  
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution	
	
	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	public SIBus(GlobalProperties parmGblProps) {
		super(parmGblProps)
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands	
	}	// END CTOR
	
	// *************************************************************************************
	// Method to create a Service Integration Bus 
	// General flow:
	//	1. Create the Service Integration Bus and Add the server as a member on that bus
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 				= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 							= resourceProps.getProperty("servername")
		SIBusname 							= resourceProps.getProperty("SIBusname")
		reuseSIBusMember 					= resourceProps.getProperty("reuseSIBusMember")
		ssl_signer_exchange_prompt_response = resourceProps.getProperty("ssl.signer.exchange.prompt_response")
		config_target_name					= resourceProps.getProperty("config.target.name")
		logger.info("SIBus-create: Create  \"${SIBusname}\"...")		
		if (DEBUG) {
			logger.debug("SIBus-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("SIBus-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("SIBus-create: servername=${servername}")
			logger.debug("SIBus-create: appsrvnodename=${appsrvnodename}")
			logger.debug("SIBus-create: SIBusname=${SIBusname}")
			logger.debug("SIBus-create: reuseSIBusMember=${reuseSIBusMember}")
			logger.debug("SIBus-create: ssl_signer_exchange_prompt_response=${ssl_signer_exchange_prompt_response}")
			logger.debug("SIBus-create: webappsrv_admin_security_is_enabled=${webappsrv_admin_security_is_enabled}")
			logger.debug("SIBus-create: script_ext=${script_ext}")
			logger.debug("SIBus-create: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("SIBus-create: websphere_install_dir=${websphere_install_dir}")
			logger.debug("SIBus-create: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("SIBus-create: dmgrprotocol=${dmgrprotocol}")
			logger.debug("SIBus-create: dmgrport=${dmgrport}")
			logger.debug("SIBus-create: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("SIBus-create: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("SIBus-create: config_target_name=${config_target_name}")
		}
		
		// 1. Create the Service Integration Bus and add the server to the bus (createSIBus.jy and addSIBusMember.jy)
		logger.info("SIBus-create: Create SIBus \"${SIBusname}\" and add \"${servername}\" as a member to the bus using the wsadmin${script_ext} command...")
		logger.info("SIBus-create: Run the createSIBus.jy and addSIBusMember.jy Jython scripts to accomplish this task.") 
		resourcePropsFileName = resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		// If WAS Admin Security is enabled, an interactive response for SSL SIGNER EXCHANGE is requested during creation of an SIBus - a reply of 'y' satisfies it 
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callMultJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "SIBusCreate_${SIBusname}_output", "SIBusCreate_${SIBusname}_result", "${ssl_signer_exchange_prompt_response}")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("SIBus-create: Command Return code: " + command_retcode)					
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the createSIbus.jy Jython proc completed successfully
			if (  command_output.contains("createSIBus: SIBus \"${SIBusname}\" successfully created.") || 
                  command_output.contains("createSIBus: SIBus \"${SIBusname}\" definition already exists.")  )  { 
				method_MSG =  "SIBus-create: ===> createSIbus.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** SIBus-create: createSIBus.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
			
			// Check if the addSIBusMember.jy Jython proc completed successfully
			if (  command_output.contains("addSIBusMember: SIBus member \"${servername}\" on node \"${appsrvnodename}\" successfully added to SIBus \"${SIBusname}\".")  ||
				  command_output.contains("addSIBusMember: SIBus member \"${servername}\" on node \"${appsrvnodename}\" already exists on SIBus \"${SIBusname}\"; it will be reused.") ) {
				method_MSG = "SIBus-create: ===> addSIBusMember.jy COMPLETED SUCCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** SIBus-create: addSIBusMember.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}	  
		} else {
			method_MSG = "**SEVERE** SIBus-create: wsadmin${script_ext} command failed; RC=" + command_retcode
			method_RC = -99
			logger.error("${method_MSG}")		
		}
		return method_RC	// Return to driver with return code
	}	// END create method

	// *************************************************************************************
	// Method to delete a Service Integration Bus 
	// General flow:
	//	1. Delete the Service Integration Bus 
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		SIBusname 				= resourceProps.getProperty("SIBusname")
		config_target_name		= resourceProps.getProperty("config.target.name")
		logger.info("SIBus-delete: Delete  \"${SIBusname}\"...")
		if (DEBUG) {
			logger.debug("SIBus-delete: globalPropsFileName=${globalPropsFileName}")
			logger.debug("SIBus-delete: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("SIBus-delete: appsrvnodename=${appsrvnodename}")
			logger.debug("SIBus-delete: SIBusname=${SIBusname}")
			logger.debug("SIBus-delete: ssl_signer_exchange_prompt_response=${ssl_signer_exchange_prompt_response}")
			logger.debug("SIBus-delete: webappsrv_admin_security_is_enabled=${webappsrv_admin_security_is_enabled}")
			logger.debug("SIBus-delete: script_ext=${script_ext}")
			logger.debug("SIBus-delete: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("SIBus-delete: websphere_install_dir=${websphere_install_dir}")
			logger.debug("SIBus-delete: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("SIBus-delete: dmgrprotocol=${dmgrprotocol}")
			logger.debug("SIBus-delete: dmgrport=${dmgrport}")
			logger.debug("SIBus-delete: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("SIBus-delete: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("SIBus-delete: config_target_name=${config_target_name}")
		}
		
		// 1. Delete the Service Integration Bus via deleteSIBus.jy proc 
		logger.info("SIBus-delete: Delete SIBus \"${SIBusname}\" using the wsadmin${script_ext} command...")
		logger.info("SIBus-delete: Run the deleteSIBus.jy Jython script to accomplish this task.")
		resourcePropsFileName = resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "deleteSIBus",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "SIBusDelete_${SIBusname}_output", "SIBusDelete_${SIBusname}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("SIBus-delete: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteSIbus.jy Jython proc completed successfully
			if (  command_output.contains("deleteSIBus: SIBus \"${SIBusname}\" successfully removed.") || 
                  command_output.contains("deleteSIBus: No SIBus entries to delete.")  )  { 
				method_MSG =  "SIBus-delete: ===> deleteSIbus.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** SIBus-delete: deleteSIBus.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** SIBus-delete: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END delete method		
			
}	// END SIBus class