// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: Server
// Purpose: Encapsulate the attributes and methods used to manage and configure an application server for WAS
// Methods:
//		create 		- create an appserver
//		delete 		- delete an appserver
//		start  		- start the appserver 
//		stop		- stop the appserver
//		update		- update the appserver Http ports, JVM generic arguments, Web Container custom properties, and Java2 security (if required)
//		getPort		- get port number for given WAS port symbollic name (e.g. "BOOTSTRAP_ADDRESS" or "SIB_ENDPOINT_ADDRESS")
//		getStatus	- get server status ("STARTED" or "STOPPED") 
// ****************************************************************************************************************
class Server extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc	// Object used to run an external command using Ant exec task
	def globalPropsFileName
	def resourcePropsFileName
	def servername
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	String jvmOptions
	Boolean enforceJava2Security
	String ssl_signer_exchange_prompt_response
    String websphere_port_endpointName_WC_defaulthost
    String websphere_port_endPointName_WC_defaulthost_secure
    String websphere_port_endPointName_WC_adminhost
	String websphere_port_endPointName_WC_adminhost_secure
    String websphere_port_endPointName_SOAP_CONNECTOR_ADDRESS
	String websphere_port_endPointName_BOOTSTRAP_ADDRESS
	String command 			// Used to hold the command string that will be executed via the Process class
	def options				// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command  
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution				
	
	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	public Server(GlobalProperties parmGblProps) {
		super(parmGblProps)
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands
		options = []
	}	// END CTOR
	
	// *************************************************************************************
	// Method to create an appserver 
	// General flow:
	//	1. Create the appserver
	//	2. Stop the appserver, if running
	//	3. Update the appserver Http ports, JVM arguments, Web Container custom properties, and Java2 security (if required)
	//	Note: Creation of the default SIBus and adding the new appserver to it as a bus member will be performed by the "create sibus" task
	//	Note: Synchronization of the dmgr and nodeagent will be performed by the "syncNode node" task 
	public int create(String resourceName, ResourceProperties resourceProps) {
		// Method variables
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 				= resourceProps.getProperty("servername")
		config_target_name		= resourceProps.getProperty("config.target.name")
		logger.info("Server-create: Create appserver \"${servername}\" under node \"${appsrvnodename}\"...")
		logger.info("Server-create: DEBUG=${DEBUG}")
		if (DEBUG) {
			logger.debug("Server-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("Server-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("Server-create: dmgrprofilename=${dmgrprofilename}")
			logger.debug("Server-create: profilename=${profilename}")
			logger.debug("Server-create: cellname=${cellname}") 
			logger.debug("Server-create: appsrvnodename=${appsrvnodename}")
			logger.debug("Server-create: servername=${servername}")
			logger.debug("Server-create: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("Server-create: config_target_name=${config_target_name}")
		}

		// 1. Create the appserver
		logger.info("=================================================")
		logger.info("Server-create: Create server \"${servername}\" using the wsadmin${script_ext} command...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll([ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						 "${websphere_scripting_jython}",
						 "createServer",
						 "${resourcePropsFileName}", 
						 "${globalPropsFileName}"  ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "ServerCreate_${servername}_output", "ServerCreate_${servername}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("Server-create: Command Return code: " + command_retcode)			
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the createServer.jy Jython proc completed successfully
			if (  command_output.contains("createServer: Server \"${servername}\" created successfully.") || 
				  command_output.contains("createServer: for z/OS, contact your WebSphere Administrator to create the Application Server(s).") ||
                  command_output.contains("createServer: Server \"${servername}\" already exists;")  )  { 
				method_MSG =  "Server-create: ===> createServer.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** Server-create: createServer.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				return method_RC	// Return to driver indicating failure
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** Server-create: wsadmin${script_ext} command failed; RC=" + command_retcode
		}
		if (method_RC != 1)	{	// If createServer.jy Jython proc failed,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}
		
		//	2. Stop the appserver, if running
		method_RC = stop(resourceName, resourceProps) 
		if (method_RC != 1)	{	// If stopServer command failed,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}	
			
		//	3. Update the appserver Http ports, JVM arguments, Web Container custom properties, and Java2 security (if required) 
		method_RC = update(resourceName, resourceProps) 
		if (method_RC != 1)	{	// If update of HTTP Ports, JVM args, Web Container custom properties, and Java2 security (if required) had an error,
			logger.error("**SEVERE** Update of server \"${servername}\" FAILED; unable to continue.")
		} else {
			logger.info("Server-create: Update of server \"${servername}\" COMPLETED SUCCESSFULLY.")
		}
		return method_RC	// Return to driver

	}	// END create method
	
	// *************************************************************************************
	// Method to delete an appserver 
	// General flow:
	//	1. Stop the appserver, if running
	//	2. delete the appserver
	//	Note: Synchronization of the dmgr and nodeagent will be performed by the "syncNode node" task 
	public int delete(String resourceName, ResourceProperties resourceProps) {
		// Obtain values for required method variables from the provided ResourceProperties object
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 				= resourceProps.getProperty("servername")
		config_target_name		= resourceProps.getProperty("config.target.name")
		logger.info("Server-delete: Delete appserver \"${servername}\" under node \"${appsrvnodename}\"...")
		if (DEBUG) {
			logger.debug("Server-delete: globalPropsFileName=${globalPropsFileName}")
			logger.debug("Server-delete: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("Server-delete: profilename=${profilename}")
			logger.debug("Server-delete: appsrvnodename=${appsrvnodename}") 
			logger.debug("Server-delete: servername=${servername}")
			logger.debug("Server-delete: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("Server-delete: config_target_name=${config_target_name}")
		}
		
		//	1. Stop the appserver, if running
		method_RC = stop(resourceName, resourceProps) 
		if (method_RC != 1)	{	// If stopServer command failed,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}
		
		// 2. Delete the appserver
		logger.info("=================================================")
		logger.info("Server-delete: Delete server \"${servername}\" using the wsadmin${script_ext} command...")
		resourcePropsFileName = resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "deleteServer",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()  
		command_retcode = cmdProc.run(".", command, optionsArray, "ServerDelete_${servername}_output", "ServerDelete_${servername}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 			logger.info("Server-delete: Command Return code: " + command_retcode)			
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteServer.jy Jython proc completed successfully
			if (  command_output.contains("deleteServer: Server \"${servername}\" has been removed.") ) { 
				method_MSG =  "Server-delete: ===> deleteServer.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** Server-delete: deleteServer.jy Jython processing FAILED; unable to continue."
				method_RC = -99
			}
		} else {
			method_MSG = "**SEVERE** Server-delete: wsadmin${script_ext} command failed; RC=" + command_retcode
			method_RC = -99
		}
		if (method_RC != 1)	{	// If deleteServer.jy Jython proc failed,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}		
		return method_RC	// Return to driver
		
	}	// END delete method 
	
	// ***********************************************************************************
	// Method to start the appserver via startServer command
	public int start(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		servername 			= resourceProps.getProperty("servername")
		config_target_name	= resourceProps.getProperty("config.target.name")
		logger.info("Server-start: Start server \"${servername}\" via the startServer${script_ext} command...")
 		command = "${websphere_install_dir}${File.separator}bin${File.separator}startServer${script_ext}"
		optionsArray = ["${servername}", "-profileName", "${profilename}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "ServerStart_${servername}_output", "ServerStart_${servername}_result")	// Run the manageprofiles command to start the dmgr server
		command_output  = cmdProc.process_output				// Get the output from the command
		logger.info("Server-start: Command Return code: " + command_retcode)
				
		// Check if "ADMU3000I: Server {0} open for e-business; process id is {1}" or 
		//			"ADMU3027E: An instance of the server may already be running: {0}" or
		//			"ADMU3028I: Conflict detected on port {0}. Likely causes: a) An instance of the server {1} is already running b) some other process is using port {0}"  
		if 	( 
			  (  command_output.contains("ADMU3000I")  ) || 
			  ( (command_output.contains("ADMU3027E"))   && 
			   !(command_output.contains("ADMU3028I")) ) 
			)  { 
				method_MSG = "Server-start: ===> startServer${script_ext} COMPLETED SUCCESSFULLY"
				method_RC = 1
		} else { 
			method_MSG = "**SEVERE** Server-start: startServer${script_ext} command FAILED; Check WebSphere logs at \"${websphere_install_dir}${File.separator}profiles${File.separator}${profilename}${File.separator}logs${File.separator}${servername}\" for analysis."
			method_RC = -99
		}		
		return method_RC   		
	}	// END start method
	
	// ***********************************************************************************
	// Method to stop the appserver server via stopServer command
	public int stop(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		servername = resourceProps.getProperty("servername")
		config_target_name	= resourceProps.getProperty("config.target.name")
		logger.info("Server-stop: Stop server \"${servername}\" via the stopServer command...")	
 		command = "${websphere_install_dir}${File.separator}bin${File.separator}stopServer${script_ext}"
		optionsArray = ["${servername}", "-profileName", "${profilename}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "ServerStop_${servername}_output", "ServerStop_${servername}_result")	// Run the manageprofiles command to start the dmgr server
		command_output  = cmdProc.process_output				// Get the output from the command
		logger.info("Server-stop: Command Return code: " + command_retcode)
				
		// Check if "ADMU4000I: Server {0} stop completed." or 
		//          "ADMU0509I: The {0} "{1}" cannot be reached. It appears to be stopped." or
		//			"ADMU0512I: Server {0} cannot be reached. It appears to be stopped." or		
		//          "ADMU3522E: No server by this name in the configuration: {0}"    
		if ( command_output.contains("ADMU4000I") || 
		     command_output.contains("ADMU0509I") ||
		     command_output.contains("ADMU0512I") ||
		     command_output.contains("ADMU3522E") ) {
			   method_MSG = "Server-stop: ===> stopServer${script_ext} COMPLETED SUCCESSFULLY"
			   method_RC = 1
		} else {
			method_MSG = "**SEVERE** Server-stop: stopNode${script_ext} command FAILED; Check WebSphere logs at \"${websphere_install_dir}${File.separator}profiles${File.separator}${profilename}${File.separator}logs${File.separator}${servername}\" for analysis."
			method_RC = -99
		}	
		return method_RC 	
	}	// END stop method	
	
	// ***********************************************************************************************************************************
	// Method to update the appserver Http ports, JVM generic arguments, Web Container custom properties, and Java2 security (if required)  
	public int update(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Resource properties required by this method:
		resourcePropsFileName 								= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 											= resourceProps.getProperty("servername")
		jvmOptions 											= resourceProps.getProperty("jvmOptions")
		enforceJava2Security 								= resourceProps.getBooleanProperty("enforceJava2Security")
		ssl_signer_exchange_prompt_response 				= resourceProps.getProperty("ssl.signer.exchange.prompt_response")
        websphere_port_endpointName_WC_defaulthost 			= resourceProps.getProperty("websphere.port.endPointName.WC_defaulthost")
        websphere_port_endPointName_WC_defaulthost_secure 	= resourceProps.getProperty("websphere.port.endPointName.WC_defaulthost_secure")
        websphere_port_endPointName_WC_adminhost 			= resourceProps.getProperty("websphere.port.endPointName.WC_adminhost")
		websphere_port_endPointName_WC_adminhost_secure 	= resourceProps.getProperty("websphere.port.endPointName.WC_adminhost_secure")
        websphere_port_endPointName_SOAP_CONNECTOR_ADDRESS 	= resourceProps.getProperty("websphere.port.endPointName.SOAP_CONNECTOR_ADDRESS")
		websphere_port_endPointName_BOOTSTRAP_ADDRESS 		= resourceProps.getProperty("websphere.port.endPointName.BOOTSTRAP_ADDRESS")
		config_target_name									= resourceProps.getProperty("config.target.name")
		if (DEBUG) {
			logger.debug("Server-update: globalPropsFileName=${globalPropsFileName}")
			logger.debug("Server-update: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("Server-update: dmgrnodename=${dmgrnodename}") 
			logger.debug("Server-update: appsrvnodename=${appsrvnodename}")			
			logger.debug("Server-update: servername=${servername}")
			logger.debug("Server-update: jvmOptions=${jvmOptions}")
			logger.debug("Server-update: enforceJava2Security=${enforceJava2Security}")
			logger.debug("Server-update: ssl_signer_exchange_prompt_response=${ssl_signer_exchange_prompt_response}")
			logger.debug("Server-update: websphere_port_endpointName_WC_defaulthost=${websphere_port_endpointName_WC_defaulthost}")
			logger.debug("Server-update: websphere_port_endPointName_WC_defaulthost_secure=${websphere_port_endPointName_WC_defaulthost_secure}")
			logger.debug("Server-update: websphere_port_endPointName_WC_adminhost=${websphere_port_endPointName_WC_adminhost}")
			logger.debug("Server-update: websphere_port_endPointName_WC_adminhost_secure=${websphere_port_endPointName_WC_adminhost_secure}")
			logger.debug("Server-update: websphere_port_endPointName_SOAP_CONNECTOR_ADDRESS=${websphere_port_endPointName_SOAP_CONNECTOR_ADDRESS}")
			logger.debug("Server-update: websphere_port_endPointName_BOOTSTRAP_ADDRESS=${websphere_port_endPointName_BOOTSTRAP_ADDRESS}")
			logger.debug("Server-update: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("Server-update: config_target_name=${config_target_name}")
		}
						
		logger.info("Server-update: Update server \"${servername}\" Http ports, JVM generic arguments, Web Container custom properties, and Java2 security (if required) using the wsadmin${script_ext} command...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_wsadmin_jython}${File.separator}callMultJythonFunction.jy", 
						 "${websphere_wsadmin_jython}", 
						 "${resourcePropsFileName}",
						 "${globalPropsFileName}"  ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "ServerUpdate_${servername}_output", "ServerUpdate_${servername}_result")	// Run wsadmin client to execute Jython procs to update Http ports and JVM Generic Arguments
		command_output  = cmdProc.process_output	// Get output from the command
		logger.info("Server-update: Command Return code: " + command_retcode)			
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the setContainerHttpPorts.jy Jython proc completed successfully
			if (  command_output.contains("setHttpPort: Processing complete.") &&
				  command_output.contains("setVirtualHostPort: Processing complete.") && 
                  command_output.contains("setContainerHttpPorts: Processing complete.")  )  { 
				method_MSG =  "Server-update: ===> setContainerHttpPorts.jy COMPLETED SUCCESSFULLY; Http ports for server \"${servername}\" have been updated."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** Server-update: setContainerHttpPorts.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
			
			// Check if the setJvmArguments.jy Jython proc completed successfully
			if (command_output.contains("setJvmArguments: Modifications to JVM Arguments completed.")) {
				method_MSG = "Server-update: ===> setJvmArguments.jy COMPLETED SUCCESSFULLY; JVM generic arguments for server \"${servername}\" have been updated."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** Server-update: setJvmArguments.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
			
			// Check if setWCProperties.jy Jython proc completed succesfully
			if (command_output.contains("setWCProperties: Webcontainer Custom Properties successfully set for server \"${servername}\"."))  {
				method_MSG = "Server-update: ===> setWCProperties.jy COMPLETED SUCCESSFULLY; Web Container properties have been updated." 
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** Server-update: setWCProperties.jy processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
			
			// Check if setJava2Security.jy Jython proc needs to be run and if it completed successfully
			if (enforceJava2Security) {
				if (command_output.contains("setJava2Security: Java2 Security successfully set for server \"${servername}\".")) {
					method_MSG = "Server-update: ===> setJava2Security.jy COMPLETED SUCCESSFULLY; Java2 security has been enabled for server \"${servername}\"."
					method_RC = 1
					logger.info("${method_MSG}")
				} else {
					method_MSG = "**SEVERE** Server-update: setJava2Security.jy processing FAILED; unable to continue."
					method_RC = -99
					logger.error("${method_MSG}")
					return method_RC 
				}
			}
			
			// Check if on z/OS then setServerVariables.jy Jython proc needs to run; check if it completed successfully
			if (os_localhost_type == "zos") {
				if (command_output.contains("setServerVariables: WebSphere Variables successfully set for z/OS server \"${servername}\".")) {
					method_MSG = "Server-update: ===> setServerVariables.jy COMPLETED SUCCESSFULLY; WebSphere Variables set for z/OS server \"${servername}\"."
					method_RC = 1
					logger.info("${method_MSG}")
				} else {
					method_MSG = "**SEVERE** Server-update: setServerVariables.jy for z/OS server processing FAILED; unable to continue."
					method_RC = -99
					logger.error("${method_MSG}")
					return method_RC
				}
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** Server-update: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	
	}	// END update method
		
	// ***********************************************************************************************************************************
	// Method to obtain the port number for the provided WAS symbollic name port name via Jython proc "getServerPortViaProps" 
	public int getPort(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 				= resourceProps.getProperty("servername")
		String portName			= resourceProps.getProperty("portName")
		String portNumber		= "Unknown"		// Initialize output port number value
		config_target_name		= resourceProps.getProperty("config.target.name")
		method_RC = 0			// Initialize method return code
		if (DEBUG) {
			logger.debug("Server-getPort: globalPropsFileName=${globalPropsFileName}")
			logger.debug("Server-getPort: resourcePropsFileName=${resourcePropsFileName}")		
			logger.debug("Server-getPort: profilename=${profilename}") 
			logger.debug("Server-getPort: appsrvnodename=${appsrvnodename}")
			logger.debug("Server-getPort: servername=${servername}")
			logger.debug("Server-getPort: portName=${portName}") 
			logger.debug("Server-getPort: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("Server-getPort: config_target_name=${config_target_name}")
		}
			
		logger.info("Server-getPort: Get port number for symbollic name \"${portName}\" for server \"${servername}\" using the wsadmin${script_ext} command...")
 		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_wsadmin_jython}${File.separator}callJythonFunction.jy", 
						  "${websphere_wsadmin_jython}",
						  "getServerPortViaProps", 
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "ServerGetPort_${servername}_output", "ServerGetPort_${servername}_result")	// Run wsadmin client to execute Jython procs to update Http ports and JVM Generic Arguments
		logger.info("Server-getPort: Command Return code: " + command_retcode)		
		command_output  = cmdProc.process_output	// Get output from the command

		// Assume getServerPort.jy ended successfully
		String text = "getServerPort: portName \"${portName}\" = #"		// Text to look for in command_output indicating success
		int startText = command_output.indexOf(text)	// Find start of successful message text
		if (startText >= 0) {			// If successful message exists in the command output,
			int startPortNum = startText + text.size()	// Find the beginning of the port number
			int endPortNum = command_output.indexOf('#',startPortNum)	// Find the end of the port number
			if (endPortNum >= 0)  {		// If the ending delimiter for the port number was found,
				portNumber = command_output.substring(startPortNum, endPortNum)		// Pick off the port number
				method_MSG = "Server-getPort: ${servername} port name \"${portName}\" port number is: \"${portNumber}\""
				//if (!portNumber.equals("Unknown"))		// If real port number found, 
				//	resourceProps.setPropertyValue(portName, portNumber)	// Save it in the Server...properties file
				method_RC = 1
			} else {					// Ending delimiter for the port number was not found,
				method_MSG = "**SEVERE** Server-getPort: getServerPort.jy proc unable to obtain port number for \"${portName}\" from ${servername}."
				method_RC = -99
			}
		} else {						// Text indicating success not found in command output
			if ( command_output.contains("**JYTHON EXCEPTION**") ) {	// If a Jython exception occurred, this is a serious error
				method_MSG = "**SEVERE** Server-getPort: getServerPort.jy Jython proc failed with Jython exception; unable to continue."
			} else {					// Failed to find port number for the symbollic port name
				method_MSG = "**SEVERE** Server-getPort: getServerPort.jy proc unable to obtain port number for \"${portName}\" from ${servername}."
			}
			method_RC = -99    			// Serious error
		}
		logger.info("${method_MSG}")
		return method_RC					
	}	// END getPort method
		
	// ***********************************************************************************************************************************
	// Method to obtain the status for the requested server via the WAS "serverStatus.bat/sh" command 
	// Required properties:
	//		servername	- from ResourceProperties file
	//		profilename - from GlobalProperties file
	public int getStatus(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 				= resourceProps.getProperty("servername")
		config_target_name		= resourceProps.getProperty("config.target.name")
		
 		command = "${websphere_install_dir}${File.separator}bin${File.separator}serverStatus${script_ext}"
		optionsArray = ["${servername}", "-profileName", "${profilename}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "ServerGetStatus_${servername}_output", "ServerGetStatus_${servername}_result")	// Run the manageprofiles command to start the dmgr server
		command_output  = cmdProc.process_output				// Get the output from the command
		logger.info("Server-getStatus: Command Return code: " + command_retcode)
		if (command_retcode == 0) {
			// Check if "ADMU0508I - The state of the specified server is given" or 
			//          "ADMU0509I - The server "${servername}" cannot be reached. It appears to be stopped." 
			method_VALUE = "UNKNOWN"
			if ( command_output.contains("ADMU0508I") ) {
			   	method_MSG = "Server-getStatus: ===> serverStatus${script_ext} COMPLETED SUCCESSFULLY"
			   	method_RC = 1
				method_VALUE = "STARTED"
				logger.info("${method_MSG}")
			} else {
				if ( command_output.contains("ADMU0509I") ) {
					method_MSG = "Server-getStatus: ===> serverStatus${script_ext} COMPLETED SUCCESSFULLY"
					method_RC = 1
					method_VALUE = "STOPPED"
					logger.info("${method_MSG}")
				} else {
					// This could be the "ADMU0522E: No server by this name in the configuration: {0}" case, therefore, assume the server is STOPPED 
					method_MSG = "Server-getStatus: ==> Server \"${servername}\" NOT FOUND, assumed to be STOPPED"
					method_RC = 1
					method_VALUE = "STOPPED"
					logger.info("${method_MSG}")
				}
			}
		} else {
			method_MSG = "**ERROR** Server-getStatus: serverStatus${script_ext} command FAILED; Check WebSphere logs at \"${websphere_install_dir}${File.separator}profiles${File.separator}${profilename}${File.separator}logs${File.separator}${servername}\" for analysis."
			method_RC = -1
			method_VALUE = "UNKNOWN" 
			logger.error("${method_MSG}")
		}	
		logger.info("Server-getStatus: The application server \"${servername}\" is: ${method_VALUE}")
		return method_RC 	
	}	// END getStatus method			

	// ***********************************************************************************************************************************
	// Method to obtain the status for all deployed applications on the requested server via the checkAppStatus Jython proc
	// Required properties:
	//		servername	- from ResourceProperties file
	public int checkAppStatus(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 				= resourceProps.getProperty("servername")
		config_target_name		= resourceProps.getProperty("config.target.name")
		if (DEBUG) {
			logger.debug("Server-checkAppStatus: globalPropsFileName=${globalPropsFileName}")
			logger.debug("Server-checkAppStatus: resourcePropsFileName=${resourcePropsFileName}")		
			logger.debug("Server-checkAppStatus: servername=${servername}")
			logger.debug("Server-checkAppStatus: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("Server-checkAppStatus: config_target_name=${config_target_name}")
		}
			
		logger.info("Server-checkAppStatus: Check status for all webapps deployed on server \"${servername}\" using the wsadmin${script_ext} command...")
 		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_wsadmin_jython}${File.separator}callJythonFunction.jy", 
						  "${websphere_wsadmin_jython}",
						  "checkAppStatus", 
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "CheckAppStatus_${servername}_output", "CheckAppStatus_${servername}_result")	// Run wsadmin client to execute Jython procs to update Http ports and JVM Generic Arguments
		logger.info("Server-checkAppStatus: Command Return code: " + command_retcode)		
		command_output  = cmdProc.process_output	// Get output from the command
		// Check if the checkAppStatus.jy Jython proc completed successfully
		method_RC = 1
		method_MSG = "Server-checkAppStatus: All applications are running on server \"${servername}\"."
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion
			def outputLines = command_output.split("\n")
			outputLines.each {line -> 
				if (line.contains("checkAppStatus: **ERROR** Application ")) {
				  logger.info("Server-checkAppStatus: Unstarted application --> ${line}")
				  method_RC = -1
				}
			}	  	
		} else {
			method_MSG = "**ERROR** Server-checkAppStatus: checkAppStatus.jy processing FAILED; unable to continue."
			logger.error("${method_MSG}")
			method_RC = -99
		}
		if (method_RC == -1) {
			method_MSG = "Server-checkAppStatus: At least one deployed application is not running on server \"${servername}\""
		}
		logger.info("${method_MSG}")
		return method_RC					
	}	// END checkAppStatus method
	
}	// END Server class
