// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import java.lang.System
import java.util.regex.Pattern
import org.apache.log4j.Logger
import org.apache.log4j.PropertyConfigurator
import com.sas.config.util.CredentialsDialog
import com.sas.config.util.CredentialsDialogPrep
import com.sas.config.util.ResourceProperties
import com.sas.util.SasPasswordString

// ##########################################################################################
// WAS driver class 
// ##########################################################################################
public class WASDriver {
	
	// Class static fields:
	final static Logger logger = Logger.getLogger("com.sas.config");	// Instantiate singleton of log4j object for logging
	static WASDriver driver					// This class
	static String globalPropsFileName 		// Global properties file name					
	static String scriptingDirectory		// Fully qualified scripting directory path
	static String websphere_scripting_credentials	// Credentials.file name
	static String execType					// Execution type RUN or FILE
	static String execMode					// Execution mode AUTO or NOAUTO
	static String promptMode				// Prompting mode GRAPHIC or TEXT
	static String fullTaskFileName			// Full path task file name 
	static String singleTaskLine			// Single task to be executed if execType is "RUN"
	static final String bs = '\\\\'			// Declare variable used to find backslashes in the filename
	static final String fs = '/'			// Declare variable used to replace backslash matches with forward slashes
	static final Pattern pattern = ~/$bs/	// Declare the pattern used to find all matches for backslashes in the filename

	// Class mutable objects
	Boolean DEBUG_WASDriver				// Debug flag
	GlobalProperties gblProps			// Global properties object
	Boolean webappsrv_auto_config		// Auto-config (true) or Manual config (false)
	Boolean webappsrv_admin_security_is_enabled	// Whether WAS Admin Security is enabled (true) or disabled (false)
	File taskFile						// File object to read task file
	
	// Closure to set a readable return code value
	def getResult = {
		def result = ""
		switch (it) {
    		case 1:
        		result = "SUCCESS"
        		break
    		case -1:
        		result = "ERROR"
        		break
    		case -99:
        		result = "SEVERE-ERROR"
        		break
    		default: 
        		result = "WARNING"
		}    // END switch
		return result     
	}	
			
	//**************************************
	// Main method
	//**************************************
	public static void main(String[] args) {
		println "     "
		println ">>>>>  Begin SAS WebSphere Application Server Configuration Driver Program <<<<<"
		// Run the CTOR to process args (-h, -e, -d, -o, -r, -n, -s, and/or -t):
		driver = new WASDriver(args)
		
	    if (driver.prepareTask() == 1) {
	    	if (execType == "FILE")
	      		logger.info("WASDriver-main: All tasks requested in \"" + fullTaskFileName + "\" have been completed.")
	      	else 
	      		logger.info("WASDriver-main: Single task request has been completed.")	 
	 	} else {
	      	logger.error("**SEVERE** WASDriver-main: Severe error has occurred; requested tasks have not been completed.")
	      	System.exit(-99)
	    }
		println ">>>>>  End   SAS WebSphere Application Server Configuration Driver Program <<<<<"
		println "     "
	}	// End main
	
	//**************************************
	// CTOR for WASDriver class
	//**************************************
	public WASDriver(String[] args) {
		int ctr = 0
		// Process the input command-line arguments:
		println "WASDriver: Number of input command-line arguments is: " + args.size()
		validateArgs(args)	// Validate the command-line arguments
		
		// Configure log4j:
		PropertyConfigurator.configure(scriptingDirectory + "/props/log4j.properties")	// Properties for log4j configuration

    	if (!(new File(scriptingDirectory).exists())) {
      		logger.info("**SEVERE** WASDriver: Provided scripting directory \"" + scriptingDirectory + "\" does not exist.")
			usage()		      	
    	}
		
		// Process the scripting directory command-line argument:	    	
    	globalPropsFileName = scriptingDirectory + File.separator + "props" + File.separator + "websphere.global.properties"
		globalPropsFileName = (globalPropsFileName =~ pattern).replaceAll("$fs")	// Convert all backslash matches to forward slashes in the filename

    	logger.info("WASDriver: Scripting directory is: \"${scriptingDirectory}\"")
    	logger.info("WASDriver: Global properties file is: \"" + globalPropsFileName + "\"")
				
		// Create GlobalProperties object used to process global properties file from the scripting directory
		gblProps = new GlobalProperties(globalPropsFileName)
		gblProps.setPropertyValue("scriptingDirectory", scriptingDirectory)
		gblProps.setPropertyValue("globalPropsFileName", globalPropsFileName)
		websphere_scripting_credentials = gblProps.websphere_scripting_credentials
		def write_RC = gblProps.writeProps()
		DEBUG_WASDriver = gblProps.DEBUG_WASDriver
		if (DEBUG_WASDriver == null)
			DEBUG_WASDriver = false
		logger.info("WASDriver: DEBUG_WASDriver=${DEBUG_WASDriver}")
		if (DEBUG_WASDriver) {
			logger.debug("**DEBUG** WASDriver: Save all global properties after update of scriptingDirectory property, RC=${write_RC}")
			logger.debug("**DEBUG** WASDriver: WAS is installed at \"" + gblProps.websphere_install_dir + "\"")
			gblProps.printAll()		// Dump all global properties
		}

		// If execution is FILE mode, open the provided task file:
		if (execType == "FILE") {
			logger.info("WASDriver: Prepare task file \"${fullTaskFileName}\" for processing...")
			try {
	  			taskFile = new File(fullTaskFileName)
			} catch (FileNotFoundException e) {
	  			logger.error("**SEVERE** WASDriver: Tasks file \"" + fullTaskFileName + "\" does not exist; unable to run WASDriver.")
	  			e.printStackTrace();
			} catch (IOException e) {
	  			logger.error("**SEVERE** WASDriver: I/O Error occurred in processing Tasks file \"" + fullTaskFileName + "\"; unable to run WASDriver.")
	  			e.printStackTrace();
			}  
		} else {	// Execution mode is RUN, so use default task file name
			fullTaskFileName = gblProps.websphere_scripting_tasks_file
		}         
  		return 
	} // End ctor

	//********************************************
	// Method: 	validateArgs
	//********************************************
	public validateArgs(String [] args) {
		println "WASDriver-validateArgs: Size of args is: " + args.size()
		for (int i=0; i<args.size(); i++)
    		println "WASDriver-validateArgs: args["+i+"] = " + args[i]		
        def cli = new CliBuilder(usage:'WASDriver command-line options')

		// Define the options and their information:
        cli.h(                             longOpt:'help',                                               'Show help information'                 )
        cli.e(argName:'type-of-execution', longOpt:'execType',     args:1, required:true,  type:GString, '(required) \"RUN\" | \"FILE\"'         )
        cli.m(argName:'mode-of-execution', longOpt:'execMode',     args:1, required:true,  type:GString, '(required) \"AUTO\" | \"NOAUTO\"'      )  
        cli.d(argName:'scripting-dir',     longOpt:'directory',    args:1, required:true,  type:GString, '(required) Scripting directory path'   )
		cli.t(argName:'task-file-name',    longOpt:'taskFile',     args:1, required:false, type:GString, 'Name of task file'                     )
		cli.o(argName:'operation',         longOpt:'operation',    args:1, required:false, type:GString, '(e.g. \"create\")'                     )
		cli.r(argName:'resource-type',     longOpt:'resourceType', args:1, required:false, type:GString, 'Type of resource (e.g. \"Server\")'    )
		cli.n(argName:'resource-name',     longOpt:'resourceName', args:1, required:false, type:GString, 'Name of resource (e.g. \"SASServer\")' )
		cli.s(argName:'scope',             longOpt:'scope',        args:1, required:false, type:GString, 'Scope (e.g. \"SERVER\")'               )
		cli.p(argName:'prompt-mode',       longOpt:'promptMode',   args:1, required:false, type:GString, '\"GRAPHIC\" | \"TEXT\"(default)'       )
		def opt = cli.parse(args)
		
		// Check if help is requested
        if (opt == null || opt.h)  { 
            usage()
		    return 
		}
		
		// Get the required scripting directory path
		scriptingDirectory = opt.d.trim()
		println "WASDriver-validateArgs: Scripting directory: \"${scriptingDirectory}\""
		
		// Validate type of execution (execType option) requested
		execType = ''
		if (opt.e)
		  	execType = opt.e.trim()
		if (!execType.equals("RUN") && !execType.equals("FILE")) {  // If the execType was not defined
		    println "**ERROR** WASDriver-validateArgs: Specified execution type \"${execType}\" is invalid; Use: \"-e RUN\" to run a single task, or \"-e FILE\" to execute task(s) from a file."
		    System.exit(-99)
		}
		
		// Validate mode of execution (execMode option); AUTO=run all tasks automatically; NOAUTO=do not run tasks, simply syntax check them
		execMode = ''
		if (opt.m)
		  execMode = opt.m.trim()
		if (!execMode.equals("AUTO") && !execMode.equals("NOAUTO"))  {  // If execMode is not provided
			println "**ERROR** WASDriver-validateArgs: Specified execution mode \"${execMode}\" is invalid; Use: \"-m AUTO\" to automatically run all tasks (auto-configure), or \"-m NOAUTO\" not to run any tasks, just syntax check them."
			System.exit(-99)
		}
		
		// Validate prompting mode (promptMode option); "GRAPHIC" or "TEXT", default to "TEXT" if option not provided
		promptMode = ''
		if (opt.p) {
		  	promptMode = opt.p.trim()
		} else {
			promptMode = "TEXT"
		}
		if (!promptMode.equals("GRAPHIC") && !promptMode.equals("TEXT"))  {
			println "**WARNING** WASDriver-validateArgs: --promptMode(-p) option value must be \"GRAPHIC\" or \"TEXT\"; defaulting to \"TEXT\"."
			promptMode = "TEXT"
		}	  
		println "WASDriver-validateArgs: Prompting mode is \"${promptMode}\"."
		
		if (execType.equals("RUN")) {
			// Validate required options when execType is "RUN" 
			def operation = ''
			def resourceType = ''
			def resourceName = ''
			def scope = ''

		    if (opt.o) {
		        operation = opt.o.trim()
		        println "WASDriver-validateArgs: CLI argument: operation=${operation}"
		    } else {
		    	println "**ERROR** WASDriver-validateArgs: Exec type \"RUN\" requires -o <operation> option."
		    	System.exit(-99)
		    }
		    if (opt.r) {
		    	resourceType = opt.r.trim()
		    	println "WASDriver-validateArgs: CLI argument: resourceType=${resourceType}"
		    } else {
		      	println "**ERROR** WASDriver-validateArgs: Exec type \"RUN\" requires -r <resource-type> option."
		       	System.exit(-99)
		    }
		    if (opt.n) {
		    	resourceName = opt.n.trim()
		    	println "WASDriver-validateArgs: CLI argument: resourceName=${resourceName}"
		    } else {
		    	println "**ERROR** WASDriver-validateArgs: Exec type \"RUN\" requires -n <resource-name> option."
		    	System.exit(-99)
		    }
		    if (opt.s) {
		    	scope = opt.s.trim()
		    	println "WASDriver-validateArgs: CLI argument: scope=${scope}" 
		    } else {
		    	println "**ERROR** WASDriver-validateArgs: Exec type \"RUN\" requires -s <scope> option."
		    	System.exit(-99)
		    }
		    println "WASDriver-validateArgs: Execution type: \"RUN\" - execute single task."
		    singleTaskLine = "${operation} ${resourceType} ${resourceName} ${scope}"
		    println "WASDriver-validateArgs: Task: \"${singleTaskLine}\""
		} else {
			// Validate required taskFile option when execType is "FILE" and build fully-qualified task file name from -taskFile option 
			def taskFileName = opt.t.trim()
			if (execType.equals("FILE")) {
			    if (!(opt.t)) {
			        println "**ERROR** WASDriver-validateArgs: For execution type \"FILE\", a task file name must be provided."
			        System.exit(-99)
			    }
			    println "WASDriver-validateArgs: Execution type: \"FILE\" - execute task(s) from a file."
			    println "WASDriver-validateArgs: Task file name: \"${taskFileName}\"."
				def prefix = scriptingDirectory + File.separator + "tasks" + File.separator 
				if (taskFileName.startsWith(prefix)) {
					fullTaskFileName = taskFileName
				} else {
			    	fullTaskFileName = prefix + taskFileName
				}	
			}
		}
	}	// END validateArgs method

	//********************************************
	// Method: 	prepareTask
	//********************************************
	public int prepareTask() {
		// Table of ResourceTypes.operations and related resourceNames ("*" means any name) that require authentication credentials:
		def taskRsrcNameTable = [ "DmgrProfile.create":"*",
								  "NodeProfile.create":"*",
								  "LoginModule.create":"SAS-Trusted-user",
								  "MailSession.create":"SMTP-Mail-Server",
								  "DataSource.create":"*",
								  "JMSConnectionFactory.create":"*",
								  "MultiCast.reset":"*"  ]
	
		webappsrv_auto_config 				= gblProps.webappsrv_auto_config
		webappsrv_admin_security_is_enabled = gblProps.webappsrv_admin_security_is_enabled
		int task_RC = 0
		
		// Print a message about type of WASDriver execution:
		if (webappsrv_auto_config || execMode == "AUTO")
			logger.info("WASDriver-prepareTask: Auto configuration or WASDriver -execMode=\"AUTO\" option has been requested")
		else
			logger.info("WASDriver-prepareTask: Manual configuration has been requested")
					
		if (DEBUG_WASDriver)
			logger.info("WASDriver-prepareTask: Debugging info has been requested")
		
		// Scan the task(s) to be executed to produce a list of tasks that may require authentication credentials 
		def listTasksNeedingCredentials = []	// List of resources that may need credentials for which prompting will be necessary 
		// Construct CredentialsDialogPrep object is used to prepare for the dialog to prompt for credentials as required:
		CredentialsDialogPrep credPrep = new CredentialsDialogPrep(taskRsrcNameTable, websphere_scripting_credentials, DEBUG_WASDriver, promptMode);
		listTasksNeedingCredentials = credPrep.scanCommandsForCredentials(execType, singleTaskLine, taskFile)
		
		// At this point, we have a list of tasks from the tasks to-be-executed that require credentials and will require prompting
		// Here is the sequence to perform prompting for credentials:
		// 1.  The processCredentials method of the CredentialsDialogPrep class should produce a list of "contents" maps whose entries
		//		will be used to prompt for userId and password for the resource
		// 2.  Instantiate the CredentialsDialog object using the "contents" list-of-maps constructed by the CredentialsDialogPrep.processCredentials method
		// 		along with a "results" map that will be created by the CredentialsDialog constructor.
		// 3.  Process the "results" map by calling the processDialogResults method of the the CredentialsDialogPrep class
		def contents = []		// List where each entry is a map containing contents for CredentialsDialog prompting.
		if (listTasksNeedingCredentials)  {
			logger.info("WASDriver-prepareTask: Determine credentials requirements...")
			contents = credPrep.processCredentials(listTasksNeedingCredentials)	// Process the credentials.properties file for the tasks that require it
			if (contents.size() > 0)  {	// There should be some prompting contents available
				logger.info("WASDriver-prepareTask: Task(s) to be executed indicate operations on resourceType(s) require userIds and passwords for authentication...")
				def resultsMap = [:]
				new CredentialsDialog(contents, resultsMap, promptMode)
				if (resultsMap == null || resultsMap.size() == 0) {
					logger.info("**ERROR** CredentialsDialogPrep-processCredentials: Dialog to solicit credentials for resources requiring userIds and passwords has failed.")
					System.exit(-99)
				}
				if (DEBUG_WASDriver) 
					logger.debug("**DEBUG** WASDriver-prepareTask: CredentialsDialog resultsMap=${resultsMap}")
				if (credPrep.processDialogResults(resultsMap) != 1) {
					logger.info("**SEVERE** WASDriver-prepareTask: Prompting for Credentials FAILED; Unable to continue.")
					System.exit(-99)
				}	 
			} 	// End contents returned from CredentialsDialogPrep.processCredentials method is not empty
		}	// End listTasksNeedingCredentials is not null
		
		// Parse the task in preparation to perform the requested operation on the Resource		
		if (execType == "RUN") {	// Parse and execute a single task
			logger.info("WASDriver-prepareTask: Process single task: \"${singleTaskLine}\"...") 
			task_RC = parseTask(singleTaskLine,1)
			logger.info("WASDriver-prepareTask: RC=${task_RC} from parseTask()")
		} else {					// Parse and execute the tasks from the tasks file:
			logger.info("WASDriver-prepareTask: Process task file: \"${fullTaskFileName}\"...")		
			int linecnt = 0		// Line count in task file
			// Loop through each line in the task file:				
			taskFile.eachLine { line ->
				linecnt++
				if (line.startsWith("#")) {
					if (DEBUG_WASDriver) 
			  			logger.debug("**DEBUG** WASDriver-prepareTask: Task line ${linecnt}: \"" + line + "\" [Comment - ignored]") 
			  	} else {	// Not a comment
					task_RC = parseTask(line,linecnt)
					logger.info("WASDriver-prepareTask: RC=${task_RC} from parseTask(${line},${linecnt})")
					if (task_RC < -1)
						System.exit(-99)
			  	}	// End not a comment
			}	// End eachLine closure
		}
		return task_RC   
	}	// End prepareTask method
	
	//**************************************************************************************************
	// Method: 	parseTask
	//**************************************************************************************************
	public int parseTask(String tline, int linecnt) {
		// List of valid operations (methods):
		def opList = [	"create", "delete", 
						"start", "stop",
						"deploy", "undeploy",
						"update", "updateProps", "validate",
						"updateProviderEndpoints",
						"reset", "syncNode",
						"getPort", "getStatus", "checkAppStatus", 
						"clear", "encodeAll", 
						"removeDuplicates"  ]
		int m_rc = 0	// Overall method return code
  		
		def pattern = /\S+/				// Pattern of non-whitespace words with whitespace(s) between the fields ("create Server SASServer1 SERVER") 
		def fields = (tline =~ pattern)	// Break out the fields from the task line using a Matcher
		
  		if (DEBUG_WASDriver) {
			logger.debug("**DEBUG** WASDriver-parseTask: Task line ${linecnt}: \"" + tline + "\" [Task to be propcessed]")
  			int cnt=0
  			fields.each { fld -> 
  				logger.debug("**DEBUG** WASDriver-parseTask: field[${cnt++}] = ${fld}")
  			}	// End each field closure
  		}
  		if (fields.size() == 4) {	// A valid task line must have 4 fields, then process the task request
	  		def op = fields[0]
	  		// Validate the requested operation for the task:
			if ( (opList.find { it == op }) == null ) {
  				String error_msg = "WASDriver-parseTask: Operation \"${op}\" in task line " + linecnt + " is invalid: \"${tline}\"."
	  			logger.error("**SEVERE** ${error_msg}")
	  			return -99	
	  		}
	  		def resourceType = fields[1]
	  		def resourceName = fields[2]
	  		def resourceScope = fields[3]
	  		logger.info("**********  Begin \"${op} ${resourceType}\" Task Processing  **********")		  		
	  		logger.info("WASDriver-parseTask: Operation=${op}; ResourceType=${resourceType}; ResourceName=${resourceName}; ResourceScope=${resourceScope}")
	  		// Execute the task, if in auto-config mode or running in batch mode and must execute all tasks automatically:
	  		if (execMode == "AUTO") {
	  			m_rc = executeTask(op, resourceType, resourceName, resourceScope)
	  		} else {	// Execution mode is not AUTO, syntax check of task is all that is needed
	  			m_rc = 1
	  			logger.info("**INFO** WASDriver-parseTask: execMode=${execMode}, validate task syntax only; task not executed")
	  		}		
			logger.info("WASDriver-parseTask: RC=${m_rc} from executeTask(${op},${resourceType},${resourceName},${resourceScope})")
	  		if (m_rc == 1) { 
	    		logger.info("WASDriver-parseTask: Task \"${op} ${resourceType}\" COMPLETED SUCCESSFULLY; (Result is: \"${getResult(m_rc)}\")")
	    	} else {
	    		String error_msg = "WASDriver-parseTask: Task \"${op} ${resourceType}\" execution FAILED; (Result is: \"${getResult(m_rc)}\")"
	    		if (m_rc < -1) {
	    			logger.error("**SEVERE** ${error_msg}")
	    		} else {
	    			logger.error("**ERROR** ${error_msg}")
	    		} 
	  		}
	  		logger.info("**********  End   \"${op} ${resourceType}\" Task Processing  **********")
		} else {
  			logger.error("**ERROR** WASDriver-parseTask: Task line ${linecnt} is invalid, it is ignored")
  			logger.error("          WASDriver-parseTask: Ignored task line is: \"${tline}\"")
  			m_rc = -1		  		
		}
		return m_rc			  		
	}	// END parseTask
	
	//**************************************************************************************************
	// Method: 	executeTask
	//**************************************************************************************************
	public int executeTask(String op, String resourceType, String resourceName, String resourceScope) {
		
		// *********************************************************************************************
		// If this is a task to run a local WASDriver method, run it now.
		//**********************************************************************************************
		if (resourceType == "WASDriver")  {
			return (driver.invokeMethod(op,[fullTaskFileName]))	
		}
		
		// *********************************************************************************************
		// Otherwise, run the requested task by instantiating the associated Resource class and running
		// the requested method (operation) using the provided ResourceProperties file for the task. 
		// ********************************************************************************************* 
		ResourceProperties taskPropFile = null		// Each task has an associated properties file		
		String websphere_scripting_props = gblProps.websphere_scripting_props
		String taskPropFileName = websphere_scripting_props+File.separator+resourceType+'.'+resourceScope+'.'+resourceName+".properties"	// Obtain the file name of the associated task properties file
		
		// ************************************************************************************************
		// Convert the properties filename for the task to an acceptable form for the Java Properties class
		// The following code converts all backslashes to forward slashes in the properties filename, since the Java Properties class doesn't
		// like names with backslashes (e.g. a filename containing a string like this: "...\nodeProfile..." has the "\n" interpreted as a 
		// newline and you'll get a FileNotFound exception from the Properties class.
		// Groovy has some pecularities about the pattern.  It doesn't like using the escaped backslash '\\' in the pattern, so you have to declare
		// it using '\\\\' and you have to use a similar declaration for the forward slash '/' to be used as a variable in the replacement string.
		// ************************************************************************************************ 
		taskPropFileName = (taskPropFileName =~ pattern).replaceAll("$fs")	// Convert all backslash matches to forward slashes in the filename
		
		logger.info("WASDriver-executeTask: Properties file associated with the \"${op} ${resourceType}\" task is: \"${taskPropFileName}\"") 
		taskPropFile = new ResourceProperties(taskPropFileName)		// Create the ResourceProperties object to process the associated task properties file
		def taskPropFile_RC = taskPropFile.getRp_rc()		// Use Autogenerated getter method to obtain "rp_rc" method attribute value
		if (DEBUG_WASDriver)
			logger.debug("**DEBUG** WASDriver-executeTask: Return code from loading \"${taskPropFileName}\" = ${taskPropFile_RC}")
		if (taskPropFile_RC != 1) {
			logger.error("**SEVERE** WASDriver-executeTask: Unable to process properties file \"" + taskPropFileName + "\" associated with resource \"" + resourceName + "\".")
			return -1 				
		}

		// Perform the requested task using the global properties and the associated task properties file	
		if (DEBUG_WASDriver) 
			logger.debug("**DEBUG** WASDriver-executeTask: Resource type is: ${resourceType}")
		logger.info("WASDriver-executeTask: Dispatch ${resourceType}.${op}(${resourceName},${taskPropFileName}) to perform the task...")				
		switch (resourceType) {
			case "ActivationSpec":
				def activationSpec = new ActivationSpec(gblProps)
				return (activationSpec.invokeMethod(op,[resourceName,taskPropFile]))		// Methods: create, delete
 			case "Application":
				def application = new Application(gblProps)
				return (application.invokeMethod(op,[resourceName,taskPropFile]))			// Methods: deploy, undeploy, start, stop
			case "Credentials":
				def credentials = new Credentials(gblProps)
				return (credentials.invokeMethod(op,[resourceName,taskPropFile]))			// Methods: clear
			case "DataSource":
				def dataSource = new DataSource(gblProps)
				return (dataSource.invokeMethod(op,[resourceName,taskPropFile]))			// Methods: create, delete				
			case "DmgrProfile":
				def dmgrProfile = new DmgrProfile(gblProps)
				return (dmgrProfile.invokeMethod(op,[resourceName,taskPropFile]))			// Methods: create, delete, start, stop, update, updateProps, validate
			case "JDBCProvider":
				def jdbcProvider = new JDBCProvider(gblProps)
				return (jdbcProvider.invokeMethod(op,[resourceName,taskPropFile]))			// Methods: create, delete
			case "JMSConnectionFactory":
				def jmsConnectionFactory = new JMSConnectionFactory(gblProps)
				return (jmsConnectionFactory.invokeMethod(op,[resourceName,taskPropFile]))	// Methods: create, delete, updateProviderEndpoints
			case "JMSQueue":
				def jmsQueue = new JMSQueue(gblProps)
				return (jmsQueue.invokeMethod(op,[resourceName,taskPropFile]))				// Methods: create, delete
			case "JMSTopic":
				def jmsTopic = new JMSTopic(gblProps)
				return (jmsTopic.invokeMethod(op,[resourceName,taskPropFile]))				// Methods: create, delete
			case "LoginModule":
				def loginModule = new LoginModule(gblProps)
				return (loginModule.invokeMethod(op,[resourceName,taskPropFile]))			// Methods: create, delete
			case "MailSession":
				def mailSession = new MailSession(gblProps)
				return (mailSession.invokeMethod(op,[resourceName,taskPropFile]))			// Methods: create, delete
			case "MultiCast":
				def multiCast = new MultiCast(gblProps)
				return (multiCast.invokeMethod(op,[resourceName,taskPropFile]))				// Methods: reset																			
			case "Node":
				def node = new Node(gblProps)
				return (node.invokeMethod(op,[resourceName,taskPropFile]))					// Methods: syncNode
			case "NodeProfile":
				def nodeProfile = new NodeProfile(gblProps)
				return (nodeProfile.invokeMethod(op,[resourceName,taskPropFile]))			// Methods: create, delete, start, stop, update, updateProps, validate
			case "ObjectCache":
				def objectCache = new ObjectCache(gblProps)
				return (objectCache.invokeMethod(op,[resourceName,taskPropFile]))			// Methods: create, delete
			case "Server":
				def server = new Server(gblProps)
				return (server.invokeMethod(op,[resourceName,taskPropFile]))				// Methods: create, delete, start, stop, update, getPort, getStatus
			case "SIBus":
				def sibus = new SIBus(gblProps)
				return (sibus.invokeMethod(op,[resourceName,taskPropFile]))					// Methods: create, delete
			default:
				logger.error("**ERROR** WASDriver-executeTask: Invalid or unknown resourceType \"" + resourceType + "\"; cannot execute requested task.")
				return -99
		}	// End switch
	}	// End executeTask method

	//*********************************************************************************************
	// Method:  removeDuplicates
	// Purpose: Optimize the provided task file by removing immediately contiguous duplicate tasks.
	//			This removes the effects of an SDW "Retry" operation. 
	//*********************************************************************************************
	public int removeDuplicates(String _fullTaskFileName) {
		logger.info("WASDriver-removeDuplicates: Optimize \"${_fullTaskFileName}\" tasks file by removing duplicate tasks...")
		try {
			taskFile = new File(_fullTaskFileName)
		} catch (FileNotFoundException e) {
			logger.error("**SEVERE** WASDriver-removeDuplicates: Tasks file \"${_fullTaskFileName}\" does not exist; unable to run WASDriver.")
			e.printStackTrace();
			return -99
		} catch (IOException e) {
			logger.error("**SEVERE** WASDriver-removeDuplicates: I/O Error occurred in processing Tasks file \"${_fullTaskFileName}\"; unable to run WASDriver.")
			e.printStackTrace();
			return -99
		}
		def lines = taskFile.readLines()
		logger.info("WASDriver-removeDuplicates: Number of lines in task file \"${_fullTaskFileName}\" before optimization is: ${lines.size()}")		
		def newLines = []
		String priorLine = ""
		lines.each { line->
			if (!line.equals(priorLine))  {
				newLines.add(line)
				priorLine = line
			}
		}
		
		// Now clear the existing task file and write the optimized tasks
		logger.info("WASDriver-removeDuplicates: Writing optimized task file \"${_fullTaskFileName}\"...")
		taskFile.write("")		// Clear the existing file
		// Write out the new optimized task file:
		newLines.each { line->	
			taskFile.append(line+"\n")
		}
		logger.info("WASDriver-removeDuplicates: Wrote ${newLines.size()} lines to task file.")
		return 1 
	}
			
	//********************************
	// Method:	usage
	//********************************
	public void usage() {
        String osName = System.getProperty("os.name")
        println "WASDriver-usage: How to run the WASDriver scripting program"
        if (osName.contains("Windows")) {
            println "WASDriver-usage: To run a single task: "
            println "  <scripting-directory>\\scripts\\WASDriver.bat -e RUN  -m AUTO -p TEXT -d <scripting-directory> -o <operation> -r <resource-type> -n <resource-name> -s <scope> "
            println "WASDriver-usage: To run a series of tasks from a task file: "
            println "  <scripting-directory>\\scripts\\WASDriver.bat -e FILE -m AUTO -p TEXT -d <scripting-directory> -t <task-file-name> "
        } else {
            println "WASDriver-usage: To run a single task: " 
            println "  <scripting-directory>/scripts/WASDriver.sh -e RUN  -m AUTO -p TEXT -d <scripting-directory> -o <operation> -r <resource-type> -n <resource-name> -s <scope> "
            println "WASDriver-usage: To run a series of tasks from a task file: "
            println "  <scripting-directory>/scripts/WASDriver.sh -e FILE -m AUTO -p TEXT -d <scripting-directory> -t <task-file-name> "
        }
        System.exit(1)
	}	// End usage method	
}	// End class WASDriver