# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys

#**************************************************************************************************
# Name: SAScreateJmsServer
# Purpose: Create or update a JMS Server for Weblogic.
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation:
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#@@ jmsserver resourceDescription: Resources are stored in properties files that are named in the pattern <tt>jmsserver.<em>resource</em>.properties</tt>.  For information about how these properties control WebLogic Server configuration, see the online help in the WebLogic Administration Console.  More information is available in the WebLogic Server documentation found on the Oracle Web site.  The WebLogic Server 10.3.3 documentation for this resource can be found at <a href="http://download.oracle.com/docs/cd/E14571_01/web.1111/e13738/toc.htm">http://download.oracle.com/docs/cd/E14571_01/web.1111/e13738/toc.htm</a>.
#@@ jmsserver dataSourceName:  The data source to use for a JDBC persistent store.  If a data source is not configured, the value is <tt>not-set</tt>. SAS applications might depend on particular settings.  Do not change this property.
#@@ jmsserver jdbcStoreName:  The file or database in which this JMS server stores persistent messages. If a data store is not configured, then the value of <tt>dataSourceName</tt> is <tt>not-set</tt>. SAS applications might depend on particular settings.  Do not change this property.
#@@ jmsserver jmsServerName: The name of the JMS Server.
#@@ jmsserver tablePrefix: A prefix to prepend to the table name in this JDBC store. If a table prefix is not configured, the value is <tt>not-set</tt>.
#@@ jmsserver targetServerName: The name of the managed server that will execute this JMS server.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output:
#   None
#**************************************************************************************************

def SAScreateJmsServer(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    # Debug Statements
    logger.debug('adminUrl:' + adminUrl)
    logger.debug('jmsServerName:' + jmsServerName)
    logger.debug('targetServerName:' + targetServerName)
    logger.debug('tablePrefix:' + tablePrefix)
    logger.debug('dataSourceName:' + dataSourceName)

    editing = None

    try:
        # Connect to administration server
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
     
        # Check if Target Server MBean exists.
        targetServerMBean = getMBean('/Servers/' + targetServerName)
        if targetServerMBean is None:        
            logger.error('Target Server ' + targetServerName + ' not found.')
            writeRC(RC_TARGET_DOES_NOT_EXIST)
            disconnect()
            return RC_TARGET_DOES_NOT_EXIST  
        else:
            logger.debug('Creating new JMS Server named ' + jmsServerName + '.')
            edit()
            startEdit()
            editing = true
            cd('/')
    
            # Check if JMS Server already exists.
            try:
                cd('/JMSServers/' + jmsServerName)
                logger.debug('The JMS Server with name ' + jmsServerName + ' already exists.')
                jmsServerMBean = getMBean('/JMSServers/' + jmsServerName)
                try:
                    cd('Targets/' + targetServerName)
                except WLSTException:
                    logger.error('JMSServer exists, but is targeted to a different managed server.')
                    writeRC(RC_JMSSERVER_TARGETTED)
                    cancelEdit('y')    
                    disconnect()
                    return RC_JMSSERVER_TARGETTED  
            except WLSTException:
                logger.debug('JMSServer does not exist. Creating...')
                jmsServerMBean = create(jmsServerName,'JMSServer')    	      
                jmsServerMBean.addTarget(targetServerMBean)      
    
            if dataSourceName <> "not-set" and tablePrefix <> "not-set":
                # jdbcStoreName = jmsServerName + '_JDBCStore'
                jdbcStore = getMBean('/JDBCStores/' + jdbcStoreName)
                if jdbcStore is None:
                    jdbcStore = create(jdbcStoreName, "JDBCStore")      
                jdbcSystemResource = getMBean('/JDBCSystemResources/' + dataSourceName)
                if jdbcSystemResource is None:
                    logger.error("Datasource does not exist: " + dataSourceName + '.')
                    writeRC(RC_RESOURCE_NOT_CONFIGURED)
                    return RC_RESOURCE_NOT_CONFIGURED
                jdbcStore.setDataSource(jdbcSystemResource)
                jdbcStore.setPrefixName(tablePrefix)
                jdbcStore.addTarget(targetServerMBean)
                jmsServerMBean.setPersistentStore(jdbcStore) 
    
            save()
            activate(block='true') 	# Activate changes
            disconnect()
		 	
            logger.debug('JMS Server ' + jmsServerName + " has been successfully created and configured.")
            writeRC(RC_OK_NEW)
            return RC_OK_NEW
    except Exception,e:
        logger.error('Failed to create JMS server ' + jmsServerName + '.')
        logger.error(e)

        writeRC(RC_FAILURE)
    
        # Cancel edit.
        try:
            if editing is not None:
                logger.debug('Canceling edit...')
                cancelEdit('y')    
        except WLSTException:
            pass
        return RC_FAILURE
  
