
# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

from com.sas.config import TemplateEngine,VelocityTemplateEngine,PropertyContextBuilder
from org.apache.commons.io import FileUtils
import sys
import string
import os
import os.path
from com.sas.util import SasPasswordString

#**************************************************************************************
# Subroutine: generateServiceScript
#   Purpose:  Generates the scripts used to install services to start managed servers
#             on Windows systems. 
#
# global.properties used in this function:
#!! backupBinDir
#!! domainDir
#!! templateDirectoryName
#         
#   Output:   returns a string containing the listen address
#***************************************************************************************

def generateServiceScript(serverName, resourcePropertiesFile):
    templateDirectory = File(templateDirectoryName)
    outputDirectory = File(domainDir + '/bldbin')
    binDirectory = File(domainDir + '/bin')
    backupDirectory = File(backupBinDir)
    encoding = 'UTF-8'
        
    if outputDirectory.exists() != true:
        outputDirectory.mkdir()

    templateEngine = VelocityTemplateEngine(templateDirectory,outputDirectory)
    pcb = PropertyContextBuilder()
    
    globalProperties = File(domainPropertiesFile)
    serverProperties = File(resourcePropertiesFile)
    pcb.load(globalProperties)
    pcb.load(serverProperties)
    
    templateFile = File(templateDirectory, '/installSvc.cmd')
    outputFile = File(outputDirectory, '/' + '/install' + serverName + 'Svc.cmd')
    targetFile = File(binDirectory,'/installSvc.cmd')
    templateEngine.merge(pcb.build(templateFile), templateFile, encoding, outputFile)
    if targetFile.exists() == true:
        FileUtils.copyFileToDirectory(targetFile, backupDirectory)
    FileUtils.copyFileToDirectory(outputFile, binDirectory)

    templateFile = File(templateDirectory, '/uninstallSvc.cmd')
    outputFile = File(outputDirectory, '/' + '/uninstall' + serverName + 'Svc.cmd')
    targetFile = File(binDirectory,'/uninstallSvc.cmd')
    templateEngine.merge(pcb.build(templateFile), templateFile, encoding, outputFile)
    if targetFile.exists() == true:
        FileUtils.copyFileToDirectory(targetFile, backupDirectory)
    FileUtils.copyFileToDirectory(outputFile, binDirectory)

#**************************************************************************************************
# Name: createServer.py
# Purpose: Create or update a Weblogic Server.
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials used in this module:
#                        domain.create_WeblogicAdmin_userid - Weblogic Admin Username.
#                        domain.create_WeblogicAdmin_passwd - Encoded Weblogic Admin Password.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!!   adminUrl
#!!   autoConfigure  
#!!   domainDir
#!!   host
#!!   JavaHome
#!!   nodeManagerPort
#!!   os_type
#!!   startScript
#!!   weblogicHome
#
#   server.servername.properties:
#@@ server  resourceDescription: Server resources are stored in properties files that are named in the pattern <tt>server.<em>servername</em>.properties</tt>.  For information about how these properties control WebLogic Server configuration, see the online help in the WebLogic Administration Console.  More information is available in the WebLogic Server documentation found on the Oracle Web site.  The WebLogic Server 10.3.3 documentation for this resource can be found at <a href="http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/domainconfig/CreateManagedServers.html">http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/domainconfig/CreateManagedServers.html</a>.
#@@ server  cmdLineChanges: This property is used by SAS Deployment Wizard to produce documentation.  It is not used by the SAS Configuration Scripting Tools.
#@@ server  cmdLineScriptChanges: This property is used by SAS Deployment Wizard to produce documentation.  It is not used by the SAS Configuration Scripting Tools.
#@@ server  javaPolicySettings: This property contains JVM options for use when a Java security manager is used.  This option is not currently supported with WebLogic Server and this property is ignored. 
#@@ server  JDKOptions: The JVM options for this server.
#@@ server  listenPort: The port number that this server uses for HTTP connections.
#@@ server  managedServerName: The name of the managed server.
#@@ server  scriptJDKOptions: The JVM options for this server.  This property is used by velocity templates to create the setDomainEnv.cmd and setDomainEnv.sh scripts.  The value is usually identical to JDKOptions.
#@@ server  serverChanges: This property is used by SAS Deployment Wizard to produce documentation.  It is not used by the SAS Configuration Scripting Tools.
#@@ server  serviceName: The Windows service name.  Used to create a Windows service for the server.
#@@ server  sslListenPort: The port number that this server uses for SSL connections.  The SAS Configuration Scripting Tools do not currently support SSL configuration.
#@@ server  startScript: The location of the domain setup script, setDomainEnv.cmd.
#
# Output:
#   None
#**************************************************************************************************

# Input Parameters.
def SAScreateServer(credentialsFile, resourcePropertiesFile):
    logger.debug('resourcePropertiesFile:' + resourcePropertiesFile)
    loadProperties(resourcePropertiesFile)
    # Debug Statements
    logger.debug('adminUrl:' + adminUrl)
    logger.debug('domainDir:' + domainDir)
    logger.debug('javaHome:' + javaHome)
    logger.debug('host:' + host)
    logger.debug('nodeManagerPort:' + nodeManagerPort)
    logger.debug('weblogicHome:' + weblogicHome)
    
    logger.debug('listenPort:' + listenPort)
    logger.debug('sslListenPort:' + sslListenPort)
    logger.debug('osType:' + osType)
    logger.debug('weblogicHome:' + weblogicHome)
    logger.debug('startScript:' + startScript)
    logger.debug('autoConfigure:' + autoConfigure)
    logger.debug("javaVendor=" + javaVendor)
    logger.debug('JDKOptions:' + JDKOptions)
    logger.debug('managedServerName:' + managedServerName)

    rc = 0

    editing = None

    wlstAdminUsername=""
    wlstAdminPassword=""
    try:
        if autoConfigure == 'true':
            if osType == 'unx':
                global minimumUlimit
                ulimit = getulimit()
                if ulimit == 0:
                    logger.error("Could not find tool to check File descriptor ulimit.")
                    writeReturnProperty("ulimitChecked", "false")
                else:
                    writeReturnProperty("ulimitChecked", "true")
                    if ulimit < int(minimumUlimit):
                        logger.error("File descriptor ulimit setting is " + `ulimit` + ".  It should be at least " + minimumUlimit + ".")
                        writeRC(RC_LOW_ULIMIT)
                        return RC_LOW_ULIMIT

            if not os.path.exists(nmUserConfigFile) or not os.path.exists(nmUserKeyFile):
                logger.error("No UserConfig file is found")
                writeRC(RC_MISSING_CREDENTIALS)
                return RC_MISSING_CREDENTIALS

            wlstAdminUsername = getAdminUser(nmUserConfigFile, nmUserKeyFile)
            wlstAdminPassword = getAdminPassword(nmUserConfigFile, nmUserKeyFile)
            
            try:
                connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
            except WLSTException:
                logger.error('Failed to connect to the Admin Server.')
                writeRC(RC_FAIL_ADM_CONNECT)
                return RC_FAIL_ADM_CONNECT

            serverResourcePath = '/Servers/'
            edit()
            startEdit()
            editing = true
        else:
            wlstAdminUsername = getCredentialUser(credentialsFile, "domain.create_WeblogicAdmin")
            wlstAdminPassword = getCredentialPassword(credentialsFile, "domain.create_WeblogicAdmin")
            try:
                from com.sas.util import SasPasswordString
                wlstAdminPassword = SasPasswordString.decode(wlstAdminPassword)
            except ImportError, e:
                logger.error("Can't find sas.core.jar")
                writeRC(RC_NO_SAS_CORE_JAR)
                return RC_NO_SAS_CORE_JAR
            readDomain(domainDir)  # additional structures are setup
            serverResourcePath = '/Server/'

        try:
            cd (serverResourcePath + managedServerName)
            isNewServer = false
        except Exception, e:
            isNewServer = true
  
        logger.debug('isNewServer:' + `isNewServer`)
        HostListenAddress = getHostListenAddress(weblogicHome, javaHome)
        machineChangesList = ['']
        machine = createMachine(host, HostListenAddress, nodeManagerPort, machineChangesList)
        serverChangesList = ['']
        cd('/')
        mergedCmdLineArguments = [''] 
        mergedCmdLineChanges = ['']
        mergedScriptCmdLineArguments = [''] 
        mergedScriptCmdLineChanges = ['']
        createServer(managedServerName, machine, listenPort, sslListenPort, host, host, 'false', JDKOptions, 
                     mergedCmdLineArguments, mergedScriptCmdLineArguments, mergedCmdLineChanges, 
                     mergedScriptCmdLineChanges, serverChangesList, weblogicHome, javaHome, wlstAdminUsername, 
                     wlstAdminPassword, domainDir, startScript, isNewServer)  
    
        if autoConfigure == 'false':
            updateDomain()
        else:  
            save()
            activate(block='true')
            disconnect()    

        logger.debug('isNewServer:' + `isNewServer`)
        writeReturnProperty('weblogic.' + managedServerName + '.merged.cmdLine', mergedCmdLineArguments[0])
        writeReturnProperty('weblogic.' + managedServerName + '.merged.script.cmdLine', mergedScriptCmdLineArguments[0])
        if osType == 'win':
            generateServiceScript(managedServerName, resourcePropertiesFile)
        if isNewServer:
            logger.debug(managedServerName + " has been successfully created and configured")
            writeRC(RC_OK_NEW)
            rc = RC_OK_NEW
        else:
            logger.debug(managedServerName + " has been successfully reconfigured")
            if machineChangesList[0] != '':
                writeReturnProperty('weblogic.machine.changes=', machineChangesList[0])
            if len(serverChangesList[0].split("=")) == 1:
                writeReturnProperty(serverChangesList[0].split("=")[0], "")
            else:
                writeReturnProperty(serverChangesList[0].split("=")[0], string.strip(serverChangesList[0].split("=")[1]))
            if mergedCmdLineChanges[0] != '':
                writeReturnProperty('weblogic.server.' + managedServerName + '.cmdline.changes', mergedCmdLineChanges[0])
            if mergedScriptCmdLineChanges[0] != '':
                writeReturnProperty('weblogic.server.' + managedServerName + '.cmdline.script.changes', mergedScriptCmdLineChanges[0])
                
            writeRC(RC_OK_RECONFIGURE)
            rc = RC_OK_RECONFIGURE
        return  rc
    except Exception, e:
        logger.error('Failed to create server ' + managedServerName + '.')
        logger.error(e)
        writeRC(RC_FAILURE)
        return RC_FAILURE
    
        # Cancel edit.
        try:
            if editing is not None:
                logger.debug('Canceling edit...')
                cancelEdit('y')
        except WLSTException:
            pass

        dumpStack()   
        return  rc

