# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

#**************************************************************************************
# Module: sasutils.py
#   Purpose:  Provide reusable functions for the WebLogic Jython configuration scripts 
#**************************************************************************************
from java.util import Date, Properties
from java.io import FileInputStream,FileOutputStream

#**************************************************************************************
# Subroutine: getHostListenAddress
#   Purpose:  Compute an appropriate listen address for node manager and AdminServer.  
#             For Unix platforms it examines the DemoIdentity certificate installed with 
#             WebLogic so that host name verification can be turned on. 
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! osType
#         
#   Output:   returns a string containing the listen address
#***************************************************************************************
def getHostListenAddress(
       wl_home,       # the weblogic server installation location
       java_home):    # The JDK installation location
       
     # Calculate ListenAddress
    demoKeyStore = wl_home + "/server/lib/DemoIdentity.jks"
    keytool_opts = "-J-Duser.language=en -list -v -alias demoidentity -keystore " + demoKeyStore + " -storepass DemoIdentityKeyStorePassPhrase"
    logger.debug("wl_home=" + wl_home)
    logger.debug("demoKeyStore=" + demoKeyStore)
    outfile = File.createTempFile("keytool", ".out")
    if osType == 'win':
        demoKeyStore = '"' + demoKeyStore + '"'
        keytool_opts = "-J-Duser.language=en -list -v -alias demoidentity -keystore " + demoKeyStore + " -storepass DemoIdentityKeyStorePassPhrase"
        cmdfile = File.createTempFile("keytool", ".cmd")
        cmdShell = "cmd /c "
        keytool_cmd = '"' + java_home + "/bin/keytool" + '"'
        cmdScriptLine = keytool_cmd + " " + keytool_opts + ">" + `outfile`
        logger.debug(cmdScriptLine)
        f = open(`cmdfile`, "w")
        f.writelines(cmdScriptLine)
        cmdline = cmdShell + `cmdfile`
        f.close()
    else:
        # cmdShell = "sh -c "
        keytool_cmd = java_home + "/bin/keytool"
        # cmdline = cmdShell + '"' + keytool_cmd + " " + keytool_opts + ">" + `outfile` + ' 2>&1"'
        cmdline = keytool_cmd + " " + keytool_opts + ">" + `outfile`
    logger.debug("cmdline=" + cmdline)
    if osType == 'win':
        proc = Runtime.getRuntime().exec(cmdline)
        proc.waitFor()
        rc = proc.exitValue()
        logger.debug("return code=" + `rc`)
        os.remove(`cmdfile`)
    else:
        os.system(cmdline)
    printFile(`outfile`)
    f = open(`outfile`, "r") 
    HostListenAddress=''
    try:
      line = f.readline()
      while line != "":
        if line.find("Owner: CN") != -1:
            logger.debug(line)
            end = line.find(",")
            HostListenAddress = line[10:end]
        line = f.readline()
    finally:
      f.close()
      os.remove(`outfile`)
    return HostListenAddress
  
#**************************************************************************************
# Subroutine: createMachine
#   Purpose:  create the definition of a new machine if it does not already exists.  
#             If it exists, return a change log containing any differences from the 
#             original configuration. 
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! host
#
#   Output:
#        return value - the machine mbean object
#        output parameter - changeList - a string containing the log of changes  
#                           The format is:
#             weblogic.machine.changes=[hostName,NodeManagerListenAddress,oldvalue,newvalue;]
#                                      [hostName,NodeManagerListenPort,oldvalue,newvalue;]
#**************************************************************************************
def createMachine(
      hostName,        # the name used for this machine
      listenAddress,   # the DNS name or numeric address for this machine
      nodeManagerPort, # node manager listen port
      changesList):    # (output parameter) a list that can hold one return value - the change log
  changes = ''
  if isAdminServer:
    machineResourcePath = '/Machines/'
  else:
    machineResourcePath = '/Machine/'
  try:
    # test to see if the machine exists
    cd (machineResourcePath + hostName)
    machine = cmo
    isNewMachine = false
    logger.debug('The machine ' + hostName + ' is previously configured')
  except Exception, e:
    logger.debug('Creating the Machine ' + hostName)
    logger.debug('host=' + host)
    isNewMachine = true
    machine = create(host, 'Machine')
    cd (machineResourcePath + hostName)
  
  try:
    cd ('NodeManager/' + hostName)
  except Exception, e:
    cd (machineResourcePath + hostName)
    create(hostName, 'NodeManager')
    cd ('NodeManager/' + hostName)
  
  if not isNewMachine:
     originalListenPort = get('ListenPort')
     originalListenAddress = get('ListenAddress')
     if originalListenPort != int(nodeManagerPort):
        logger.debug('Changed: NodeManager Listen Port: from=' + `originalListenPort` + ' to=' + nodeManagerPort)
        changes = changes + 'NodeManagerListenPort,' + `originalListenPort` + ',' + nodeManagerPort + ';' 
     if originalListenAddress != listenAddress:
        logger.debug('Changed: NodeManager Listen Address: from=' + originalListenAddress + ' to=' + listenAddress)
        changes = changes + 'NodeManagerListenAddress,' + originalListenAddress + ',' + listenAddress + ';' 
     
  set('ListenPort', int(nodeManagerPort))
  set('ListenAddress', listenAddress)
  changesList[0] = changes
  return machine

#**************************************************************************************
# Subroutine: createServer
#   Purpose:  create the definition of a new server if it does not already exists.  
#             If it exists, return a change log containing any differences from the 
#             original configuration. 
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! javaVendor
#!! osType
#
#   Output:
#        returns the Server mbean.  
#             The formats for change log output is:
#             weblogic.server.serverName.changes=[hostName,NodeManagerListenAddress,oldvalue,newvalue;]
#                                      [hostName,NodeManagerListenPort,oldvalue,newvalue;]
#             weblogic.server.ServerName.cmdline.changes='changes list'
#**************************************************************************************
def createServer(
       serverName,                    # the name of this server
       machineObject,                 # the mbean object for the machine
       port,                          # the listen port for this server
       sslPort,                       # the ssl listen port for this server
       hostName,                      # the name used for this machine
       hostListenAddress,             # the DNS name or numeric address for this machine
       tunnelingEnabled,              # boolean - is tunneling enabled for this server?
       cmdLineArguments,              # the proposed command line arguments for this server
       mergedCmdLineArguments,        # the resulting command line for config.xml after merging in pre-existing options
       mergedScriptCmdLineArguments,  # the resulting command line options for setDomainEnv.sh after merging in pre-existing options
       serverCmdLineChanges,          # (returned in a list) - the change log for the config.xml command line options. 
       mergedScriptCmdLineChanges,    # (returned in a list) - the change log for the script command line options. 
       changesList,                   # (returned in a list) - the changes to other server configuration attributes.
       BEAHome,                       # actually the WL_HOME value.  For the StartServers bean.
       JavaHome,                      # JAVA_HOME.  For the StartServers bean.
       adminUser,                     # name of the admin user.
       adminPasswd,                   # the admin password.
       domainDir,                     # the domain home directory.
       startScript,                   # the file path to setDomainEnv.cmd/sh
       configureCredentials):         # boolean - should credentials be updated?
  import string
  changes = ''
  if connected:
    serverResourcePath = '/Servers/'
  else:
    serverResourcePath = '/Server/'
  try:
     cd (serverResourcePath + serverName)
     logger.debug('The server ' + serverName + ' exists.')
     isNewServer = false
  except Exception, e:
     logger.debug('Creating the server ' + serverName)
     create(serverName, 'Server')
     isNewServer = true
     cd (serverResourcePath + serverName)
  
  serverObject = cmo
  
  global javaVendor
  if javaVendor == "Oracle":
      if string.find(version, '10.3 SP0') > 0:
          logger.debug('Change vendor from Oracle to BEA for 10.3 SP0')
          javaVendor = "BEA"
  
  if isNewServer:
     # For a new server merge command line to catch and mark duplicates from additional options list. 
     logger.trace('New: ' + serverName + ' Command Line Arguments:' + cmdLineArguments)
     serverCmdLineChanges[0] = processJVMCmdLine('', cmdLineArguments, mergedCmdLineArguments)
     logger.trace('mergedCmdLineArguments:' + mergedCmdLineArguments[0])
  else:
     # Track all changes to existing servers
     originalPort = get ('ListenPort')
     try:
        cd('SSL/' + serverName)
        originalSSLPort = get('ListenPort')
     except Exception, e:
        originalSSLPort = ''
     cd(serverResourcePath + serverName)
     try:
        if isAdminServer:
           originalMachine = cmo.getMachine().getName()
        else:
           originalMachine = get ('Machine').getName()
     except AttributeError, e:
        originalMachine = ''
     originalListenAddress = get ('ListenAddress')
     originalTunnelingEnabled = get ('TunnelingEnabled')
     if originalTunnelingEnabled == 0:  originalTunnelingEnabled = 'false'
     if originalTunnelingEnabled == 1:  originalTunnelingEnabled = 'true'
     try:
        cd('ServerStart/' + serverName)
        originalCmdLineArguments = get('Arguments')
        originalBEAHome = get('BeaHome')
        originalJavaHome = get('JavaHome')
        originalJavaVendor = get('JavaVendor')
        originalUsername = get('Username')
     except Exception, e:
        # no ServerStart defined yet
        originalCmdLineArguments = ''
        originalBEAHome = ''
        originalJavaHome = ''
        originalJavaVendor = ''
        originalUsername = ''
        originalPasswordEncrypted = ''
     if originalCmdLineArguments == None:
        originalCmdLineArguments = ''
     cd (serverResourcePath + serverName)
     changes = changes + logChange(serverName, 'ListenPort', `originalPort`, port)
    # Remove the change track for SSLListenPort as we never changed in the middle tier
    # changes = changes + logChange(serverName, 'SSLListenPort', `originalSSLPort`, sslPort)  
     changes = changes + logChange(serverName, 'Machine', originalMachine, hostName)  
     changes = changes + logChange(serverName, 'ListenAddress', originalListenAddress, hostListenAddress)  
     changes = changes + logChange(serverName, 'TunnelingEnabled', originalTunnelingEnabled, tunnelingEnabled)  
     logger.trace('Changed: ' + serverName + ' Command Line Arguments: from=' + originalCmdLineArguments + ' to=' + cmdLineArguments)
     serverCmdLineChanges[0] = processJVMCmdLine(originalCmdLineArguments, cmdLineArguments, mergedCmdLineArguments)
     logger.trace('mergedCmdLineArguments:' + mergedCmdLineArguments[0])
     changes = changes + logChange(serverName, 'BeaHome', originalBEAHome, BEAHome)  
     changes = changes + logChange(serverName, 'JavaHome', originalJavaHome, JavaHome)  
     changes = changes + logChange(serverName, 'JavaVendor', originalJavaVendor, javaVendor)
     if configureCredentials:  
        changes = changes + logChange(serverName, 'Username', originalUsername, adminUser)   
  
  # Set the server properties     
  set ('ListenPort', int(port))
  if connected == true:
     try:
        cd(serverResourcePath + serverName + '/SSL/' + serverName)
        logger.debug('SSL mbean already exists.')
        set('ListenPort', int(sslPort))
     except Exception, e:
        logger.debug('SSL mbean does not exist, creating it.')
        cd(serverResourcePath + serverName)
        create(serverName, 'SSL')
        cd(serverResourcePath + serverName + '/SSL/' + serverName)
        set('ListenPort', int(sslPort))
     cd('../..')
  set ('Machine', machineObject)
  set('ListenAddress', hostListenAddress)
  set ('TunnelingEnabled', tunnelingEnabled)
  try:
     create(serverName, 'ServerStart')
  except Exception, e:
     logger.debug('ServerStart already existed')
  cd('ServerStart/' + serverName)
  set('Arguments', mergedCmdLineArguments[0])
  set('BeaHome', BEAHome)
  set('JavaHome', JavaHome)
  set('JavaVendor', javaVendor)
  if configureCredentials:
     set('Username', adminUser)
     set('PasswordEncrypted', encrypt(adminPasswd, domainDir))
  
  # Handle scripts
  scriptCmdLine = getStartScriptCmdLine(serverName, startScript, osType, cmdLineArguments)
  if scriptCmdLine == '':
     logger.debug("Can't find server definition in script. Using configuration file command line.")
     mergedScriptCmdLineArguments[0] = mergedCmdLineArguments[0]
     mergedScriptCmdLineChanges[0] = ''
  else:
     logger.debug('Script CommandLine=' + scriptCmdLine)
     mergedScriptCmdLineChanges[0] = processJVMCmdLine(scriptCmdLine, cmdLineArguments, mergedScriptCmdLineArguments)
  
  logger.debug('Script merged command line=' + mergedScriptCmdLineArguments[0])
  logger.debug('Script Command Line Changes:' + mergedScriptCmdLineChanges[0])
  # Backslashes need to be escaped when saved in properties files for ant to read
  # The changes properties are handled in processJVMCmdLine
  mergedScriptCmdLineArguments[0] = string.replace(mergedScriptCmdLineArguments[0], '\\', '\\\\')
  mergedCmdLineArguments[0] = string.replace(mergedCmdLineArguments[0], '\\', '\\\\')
  changes = string.replace(changes, '\\', '\\\\')
  
  if changes != '':
     changesList[0] = 'weblogic.server.' + serverName + '.changes=' + changes + '\n'
  return serverObject

#**************************************************************************************
# Subroutine: logChange()
#   Purpose:  create a changeLog entry for a server configuration update 
#         
#   Output:
#        returns a string containing the log of changes.  The format is:
#             weblogic.server.ServerName.cmdline.changes='changes list'
#***************************************************************************************
def logChange(
  serverName,     # the name of this server
  propName,       # the server attribute
  originalValue,  # the original attribute value
  newValue):      # the new attribute value
  if originalValue == None:
     originalValue = ''
  if originalValue != newValue:
     logger.debug('Changed: ' + serverName + ' ' + propName + ': from=' + originalValue + ' to=' + newValue)
     return propName + ',' + originalValue + ',' + newValue + ';'
  else:
     logger.debug('Unchanged: ' + serverName + ' ' + propName + ': from=' + originalValue + ' to=' + newValue)
     return '' 

#**************************************************************************************
# Subroutine: processJVMCmdLine
#   Purpose:  Rebuild the JVM CommandLine by merging previously existing options with 
#             new arguments.  Some basic rules:
#             1.  No duplicate options are allowed.  The last setting on the command 
#                 line is honored.
#             2.  Options set on the existing command line that are not set on the new 
#                 command line are preserved.
#             3.  A changeLog is returned as a string.
#             4.  The order of the original arguments is preserved.  
#                 Changes will be in the position of the original setting.
#    Output:
#        newJvmArgList - A list is passed in containing one element - a string that will be returned with the new merged argument list
#        returns the changeLog.  See setJvmArgUnique for the format of these entries
#**************************************************************************************
def processJVMCmdLine(
    currJvmArgs,     # the existing JVM arguments
    inputJvmOptions, # the new JVM arguments
    newJvmArgList):  # (output parameter) the new merged argument list 
  from java.util import TreeMap
  from java.util import HashSet
  import string
  changes = ''
  # Insure spaces are handled properly for 'file:// URL' JVM options
  inputJvmOptions = ' ' + inputJvmOptions
  jvmOptions = ''
  jvmOptionsList = inputJvmOptions.split(' -')
  for option in jvmOptionsList:
    option = option.strip()
    if (option == '-'):
      continue
    if (option.find(' ') >= 0):
      if (option.find("file://") >= 0):
        option = option.replace(" ","%20")
    if (len(option) > 0):
      jvmOptions = jvmOptions + " -" + option
  #endFor
  jvmOptions = jvmOptions.strip()
  currJvmArgs = currJvmArgs.strip()
  jvmArgsList = jvmOptions.split(" ")  # Assume that the input JVM arguments are blank-delimited  
   
  logger.trace("Current JVM Arguments \"" + currJvmArgs + "\"")
  logger.trace("Input JVM Arguments \"" + jvmOptions + "\"")
  
  # Isolate each JVM generic argument and insure it appears only once in the resulting merged set
  newJvmArgString = ""    # String to hold updated list of generic JVM arguments   
  if (currJvmArgs == None or len(currJvmArgs) == 0):     # If there are no existing JVM arguments then,
    newJvmArgString = jvmOptions        # The new list will be equal to the provided input JVM arguments. 
  else:    # Generic JVM arguments already exist
    if (jvmOptions == None or len(jvmOptions) == 0):    # If no input JVM arguments were provided then, 
      newJvmArgString = currJvmArgs     # The new list will be whatever existing JVM arguments are present.
    else:   # Input JVM arguments were provided, so process them. 
      t = TreeMap()     # Used to hold all unique JVM args with "key=value" pairs 
      s = HashSet()     # Used to hold all unique single JVM args which are not "key=value" pairs
      currJvmArgList = currJvmArgs.split()     # Generic JVM args are blank delimited
      l = []
      #  Process the current JVM arguments by extracting the unique key-value pairs and single args 
      setJvmArgUnique(currJvmArgList,s,t,l,false)   # Process the current JVM arguments to obtain the unique single and key-value options 
        
      # Process input JVM arguments by extracting unique key-value pairs and merging them into the current JVM args, overriding any existing ones that match
      t1 = TreeMap()     # Used to hold all unique JVM args with "key=value" pairs 
      s1 = HashSet()     # Used to hold all unique single JVM args which are not "key=value" pairs
      l1 = []
      overrides = setJvmArgUnique(jvmArgsList,s1,t1,l1,false) # Process the input JVM arguments to obtain the unique single and key-value options
      logger.trace('overrides:' + overrides)
      uniqueNewJvmArgs = ''
      for i in l1:
        uniqueNewJvmArgs = uniqueNewJvmArgs + i + " " 
      logger.trace('uniqueNewJvmArgs:' + uniqueNewJvmArgs)
      
      changes = setJvmArgUnique(l1,s,t,l,true) # Process the input JVM arguments to obtain the unique single and key-value options
       
      for i in l:
        newJvmArgString = newJvmArgString + i + " " 
      logger.trace('newJvmArgString:' + newJvmArgString)
      # Backslashes need to be escaped when saved in properties files for ant to read
      # The command line lists are handled in createServer (we do not want to escape them before setting them in wlst)
      changes = string.replace(changes, '\\', '\\\\')
           
    #endIf there were no input JVM arguments provided
  #endIf there are no existing JVM arguments
  newJvmArgList[0] = newJvmArgString
  return changes
  
#**********************************************************************************************************************************
# Subroutine: setJvmArgUnique
#   Purpose:  Insure the JVM arguments are unique and the last or latest occurrence becomes the final value.
#
#    Output:
#        returns a string containing a list of changes to the properties.  Entries are separated by semicolons and take one of these forms:
#                REPLACE, property, old-value, new-value;
#                ADD, option;
#                ADD, property, value;
#                DROP, option;
#        The most recent or last-appearing options are added to the treeMap or hashMap as unique JVM options. 
#**********************************************************************************************************************************
def setJvmArgUnique(
    argList,     # Jython list of strings where each string is a JVM option
    hashSet,     # A Java HashSet Collection object used to store the unique single value options (i.e. those that aren't key=value pairs)
    treeMap,     # A Java TreeMap Collection object used to store the unique key-value pair options
    mergedList,  # (returned) The ordered list of options
    logChanges): # Boolean: Should a string containing a log of change be generated?

  from java.util import TreeMap
  from java.util import HashSet
  from java.util import ArrayList

  # make deep copies of hashSet and treeMap.  We'll use these to track which options are not in replacement set
  trackingHashSet = HashSet()
  for hashSetEntry in hashSet.toArray():
     trackingHashSet.add(hashSetEntry)
  treeMapCollection = treeMap.keySet()
  trackingTreeMapCollection = ArrayList()
  for treeMapEntry in treeMapCollection.toArray():
     trackingTreeMapCollection.add(treeMapEntry)
  changeLog = ''

  for arg in argList:           # Process each of the JVM arguments
    entryLogged = false
    logger.trace('arg="' + arg + '"')
    if (arg.find('=') < 0):     # If not a key-value pair, then
      for i in range(len(arg)):     # Find the first non-alphabetic character after the "-X..." prefix
        if (arg[i]>='0' and arg[i]<='9'):
          break
        #endIf
      #endFor
      prefix = arg[0:i]         # Obtain the prefix substring
      if (arg[0:len("-Xgcpolicy:")] == "-Xgcpolicy:"):
        prefix = "-Xgcpolicy:"
      if (hashSet.contains(arg)):   # If an occurrence of this single option already was found,
        hashSet.remove(arg)     # Remove it in preparation for adding the one most recently found.
        trackingHashSet.remove(arg)
        entryLogged = true      # We want to silently delete when there is no change
      else:             # This is the first occurrence for this single option.
        for hashSetEntry in hashSet.toArray():  # Check if it has already been seen before (i.e. a duplicate) 
          for i in range(len(hashSetEntry)):    # Find the first non-alphabetic character after the "-X..." prefix
            if (hashSetEntry[i]>='0' and hashSetEntry[i]<='9'):
              break
            #endIf
          #endFor
          if (hashSetEntry[0:i] == prefix): # Does an option with the same prefix as the current argument already exist?
            if logChanges:  
               logger.debug('Replacing ' + hashSetEntry + ' with ' + arg)
               changeLog = changeLog + 'REPLACE,' + hashSetEntry + ',' + arg + ';'
            entryLogged = true
            hashSet.remove(hashSetEntry)    # Remove it in preparation for adding the current option as the new unique value
            try:
               mergedList[mergedList.index(hashSetEntry)] = arg
            except ValueError:
               logger.debug("Can't find " + hashSetEntry + ' in command line list')
               raise Exception('setJvmArgUnique failed.')
            trackingHashSet.remove(hashSetEntry)
          if (hashSetEntry[0:len("-Xgcpolicy:")] == prefix):
            if logChanges:  
              logger.debug('Replacing ' + hashSetEntry + ' with ' + arg)
              changeLog = changeLog + 'REPLACE,' + hashSetEntry + ',' + arg + ';'
            entryLogged = true
            hashSet.remove(hashSetEntry)    # Remove GC Policy, since we need to replace it with the new one
            try:
               mergedList[mergedList.index(hashSetEntry)] = arg
            except ValueError:
               logger.debug("Can't find " + hashSetEntry + ' in command line list')
               raise Exception('setJvmArgUnique failed.')
            trackingHashSet.remove(hashSetEntry)
          #endIf
        #endFor 
      hashSet.add(arg)          # Add this current option to the unique set of single options
      if logChanges and not entryLogged: 
        logger.debug('Adding ' + arg)
        changeLog = changeLog + 'ADD,' + arg + ';'
      if not entryLogged:
        mergedList.append(arg)       # Add this to the ordered list of command line options
    else:               # Process the key-value pair options
      keyValuePair = arg.split("=") # Extract the key-value string 
      key = keyValuePair[0]     # Isolate the key string
      if (len(keyValuePair) > 1):   # Does this entry have a value?
        value = keyValuePair[1] # Isolate the value string
      else:             # No value, just a key
        value = ""          # Empty value
      oldValue = treeMap.get(key)
      if oldValue:
         entryLogged = true    # don't report on non-changes
         if oldValue != value and logChanges:
            logger.debug('Replacing ' + key + ' old=' + oldValue + ' new=' + value)
            changeLog = changeLog + 'REPLACE,' + key + '=' + oldValue + ',' + key + '=' + value + ';'        
         if not trackingTreeMapCollection.remove(key):
            logger.debug('failed to delete ' + key + ' from trackingTreeMapCollection')
         try:
            mergedList[mergedList.index(key + '=' + oldValue)] = arg
         except ValueError:
            logger.debug("Can't find " + key + '=' + oldValue + ' in command line list')
            raise Exception('setJvmArgUnique failed.')
      treeMap.put(key, value)       # Add each unique current JVM argument to the dictionary
      if logChanges and not entryLogged: 
        logger.debug('Adding ' + key + '=' + value)
        changeLog = changeLog + 'ADD,' + key + '=' + value + ';'        
      if not entryLogged:
        mergedList.append(arg)       # Add this to the ordered list of command line options
  #endFor
  for hashSetEntry in trackingHashSet.toArray():
      if logChanges: 
        logger.debug('In old list, but not new: ' + hashSetEntry)
        changeLog = changeLog + 'DROP,' + hashSetEntry + ';'        
        
  for treeMapEntry in trackingTreeMapCollection.toArray():
      if logChanges: 
        logger.debug('In old list, but not new: ' + treeMapEntry)
        changeLog = changeLog + 'DROP,' + treeMapEntry + '=' + treeMap.get(treeMapEntry) + ';'        
  return changeLog
#endDef setJvmArgUnique =======

#***************************************************************************************
# Subroutine: getScriptCmdLine
#   Purpose:  Extract the managed server command line options from the batch script.  
#             We want to compare this against the config.xml version of the command line 
#             options and preserve any customer changes.
#
#    Output:
#        returns The command line 
#**************************************************************************************

def getStartScriptCmdLine(
    servername,  # the name of the server - we need to check for 'AdminServer'
    script,      # file pathname for script
    osType,      # 'win' or 'unx'
    newCmdLine): # new command line, so we can pick out our server property to match
  import string
  import re
  
  if servername == 'AdminServer':
    return getAdminStartScriptCmdLine(script, osType)
    
  SAS_SERVER_PROP_NAME='-Dsas.server.name='
  oldScriptRegexp = re.compile('SERVER[0-9]*_MEM_ARGS=(?P<arglist>.*)')  # locate command line in old style
  nameIndx = string.find(newCmdLine, SAS_SERVER_PROP_NAME) + len(SAS_SERVER_PROP_NAME)
  logger.debug(newCmdLine[nameIndx:])
  serverIDList = string.split(newCmdLine[nameIndx:])
  serverid =serverIDList[0]
  logger.debug('Server id:' + serverid)
 
  try:
     scriptFD = open(script, "r")
  except IOError, e:
     return ''
  line = scriptFD.readline()
  while  line != "":
    if string.count(line, SAS_SERVER_PROP_NAME + serverid) > 0:
       results = oldScriptRegexp.search(line)  # is this an old style assignment?
       if results == None:
           # New style script
           if osType == 'win':
              logger.debug(line)
              line = string.replace(line, 'set USER_MEM_ARGS=', '')
           else:
              line = string.replace(line, 'USER_MEM_ARGS=', '')
              line = string.replace(line, '"', '')  # strip surrounding quotes
           line = string.strip(line)
       else:
          # Old style script
          line = results.group('arglist')
          if osType == 'win':
              line = line + ' -Djava.security.auth.login.config=%SAS_LOGINMODULE% -Dcom.sas.log.config.url=%SAS_LOGFILE% -Dcom.sun.xml.namespace.QName.useCompatibleSerialVersionUID=1.0'
          else:
              line = string.replace(line, '"', '')  # strip surrounding quotes
              line = line + ' -Djava.security.auth.login.config=${SAS_LOGINMODULE} -Dcom.sas.log.config.url=${SAS_LOGFILE} -Dcom.sun.xml.namespace.QName.useCompatibleSerialVersionUID=1.0'
       
       scriptFD.close()
       return line
    line = scriptFD.readline()
  scriptFD.close()
  return ''

#***************************************************************************************
# Subroutine: getAdminStartScriptCmdLine
#   Purpose:  Extract the Admin Server command line options from the batch script.  
#             We want to compare this against the config.xml version of the command line 
#             options and preserve any customer changes.
#
#    Output:
#        returns The command line 
#**************************************************************************************

def getAdminStartScriptCmdLine(
    script,      # file pathname for script
    osType):     # 'win' or 'unx'

  import string
  
  try:
     scriptFD = open(script, "r")
  except IOError, e:
     logger.debug('Failed to open script:' + script)
     return ''
  line = scriptFD.readline()
  while  line != "":
    if string.count(line, "ADMIN_SERVER_MEM_ARGS") > 0:
       if osType == 'win':
          line = string.replace(line, 'set ADMIN_SERVER_MEM_ARGS=', '')
       else:
          line = string.replace(line, 'ADMIN_SERVER_MEM_ARGS=', '')
          line = string.replace(line, '"', '')  # strip surrounding quotes
       line = string.strip(line)
       line = line + ' -Dcom.sun.xml.namespace.QName.useCompatibleSerialVersionUID=1.0' # This was set in a different place in old scripts
       scriptFD.close()
       return line
    line = scriptFD.readline()
  scriptFD.close()
  return ''
  
#**************************************************************************************
# Subroutine: moreJMSResources
#   Purpose:  Determine if a given module has any JMS Resources remaining.  
#             It is used after deleting a
#             resource, to determine if the module should also be removed.
#
#    Output:
#        returns boolean.  true if there are more resources for this module. 
#**************************************************************************************
 
def moreJMSResources(
    module):  # JMSModule mbean
    
    if len(module.getConnectionFactories()) > 0:
       return true;
    if len(module.getDestinationKeys()) > 0:
       return true;
    if len(module.getDistributedQueues()) > 0:
       return true;
    if len(module.getDistributedTopics()) > 0:
       return true;
    if len(module.getForeignServers()) > 0:
       return true;
    if len(module.getQueues()) > 0:
       return true;
    if len(module.getQuotas()) > 0:
       return true;
    if len(module.getSAFErrorHandlings()) > 0:
       return true;
    if len(module.getSAFImportedDestinations()) > 0:
       return true;
    if len(module.getSAFRemoteContexts()) > 0:
       return true;
    if len(module.getTemplates()) > 0:
       return true;
    if len(module.getTopics()) > 0:
       return true;
    if len(module.getUniformDistributedQueues()) > 0:
       return true;
    if len(module.getUniformDistributedTopics()) > 0:
       return true;
    return false;

#***************************************************************************************
# Subroutine: isWebAuthConfigured
#   Purpose:  Determine if WebAuthentication has been configured.  This is the case if  
#             the line that sets WEBAUTH_CLASSPATH in commEnvSAS.cmd or commEnvSAS.sh 
#             is not commented out.
#
#    Output:
#             returns true or false 
#**************************************************************************************

def isWebAuthConfigured(
    domain,      # file pathname for script
    osType):     # 'win' or 'unx'

  import string
  
  if osType == 'win':
     script = domain + '\\bin\\commEnvSAS.cmd'
  else:
     script = domain + '/bin/commEnvSAS.sh'
  
  logger.debug('isWebAuthConfigured: script=' + script)
  try:
     scriptFD = open(script, "r")
  except IOError, e:
     logger.debug('Failed to open script:' + script)
     return false
  line = scriptFD.readline()
  while  line != "":
    if string.count(line, "WEBAUTH_CLASSPATH=") > 0:
       logger.debug(line)
       scriptFD.close()
       if osType == 'win':
          if string.find(line, "@rem") == -1:
              return true
          else:
              return false 
       else:
          logger.debug('string.find returns ' + `string.find(line, "#")`)
          if string.find(line, "#") == -1:
              return true
          else:
              return false 
       return line
    line = scriptFD.readline()
  scriptFD.close()
  return false

#***************************************************************************************
# Subroutine: waitUntilServerIsAvailable
#   Purpose:  monitor a server's listening port.  Poll every several seconds until the
#             server is available or until several minutes have passed.  This function
#             was implemented because we've seen nodeManager take well over 1 minute to
#             bind on its listen address on some machines.
#
#    Output:
#             returns true or false 
#**************************************************************************************

def waitUntilServerIsAvailable(
    address,   # server listen address
    port):     # server listen port

    import time
    startTime = time.time()
    from java.net import Socket
    fd = -1
    tries = 0
    while fd == -1 and tries < 60:
       try:
           fd = Socket(address, int(port))
           endTime = time.time()
           print time.ctime()
           print fd
           tries = tries + 1
           logger.debug('opened socket successfully after ' + `tries` + ' tries')
           logger.debug('elapsed time to open was ' + `endTime - startTime` + ' seconds')
           fd.close()
           return true
       except Exception, e:
           tries = tries + 1
           logger.debug('still not open at ' + time.ctime())
           time.sleep(2)
        
    endTime = time.time()
    logger.warn('failed to open after ' + `tries` + ' tries')
    logger.warn('timed out after ' + `endTime - startTime` + ' seconds')
    return false

#***************************************************************************************
# Subroutine: isServerAvailable
#   Purpose:  is there a server on a listening port?  
#
#    Output:
#             returns true or false 
#**************************************************************************************

def isServerAvailable(
    address,   # server listen address
    port):     # server listen port

    from java.net import Socket
    try:
       fd = Socket(address, int(port))
       fd.close()
       return true
    except Exception, e:
       return false
  
#***************************************************************************************
# Subroutine: writeRC
#   Purpose:  Write a return code value to the output property file.
#
#   Global Properties:
#!!           rcFileName:  Name of return code file
#    Output:
#             Writes return code to property wlst.rc
#**************************************************************************************
def writeRC(rcode):
    rcFile = open(rcFileName, "a");
    rcFile.writelines("wlst.rc=" + rcode + "\n");
    rcFile.close()
    return
    
#***************************************************************************************
# Subroutine: writeReturnProperty
#   Purpose:  Write a property to the output property file
#
#    Output:
#             Writes property
#**************************************************************************************
def writeReturnProperty(name, value):
    logger.debug("writeReturnProperty: name=" + name + ", value=" + value)
    rcFile = open(rcFileName, "a");
    rcFile.writelines(name + "=" + value + "\n");
    rcFile.close()
    return
    
#***************************************************************************************
# Subroutine: deleteReturnProperties
#   Purpose:  deletes the return properties file. Used to cleanup old copies from previous
#
#    Output:
#             Writes return code
#**************************************************************************************
def deleteReturnProperties():
    if os.path.exists(rcFileName) == true:
    	os.remove(rcFileName)
    return
    
#***************************************************************************************
# Subroutine: updatePropertyFile
#   Purpose:  Updates a property in a specified property file
#
#    Output:
#             Writes return code
#**************************************************************************************
def updatePropertyFile(propFileName, prop, value):
    props = Properties()
    propsFile = File(propFileName)
    fh = FileInputStream(propsFile)
    props.load(fh)
    fh.close()
    props.setProperty(prop, value)
    fh = FileOutputStream(propsFile)
    props.store(fh, "")
    fh.close()

#***************************************************************************************
# Subroutine: setWLSCredentials
#   Purpose:  Sets the location of the secure credentials files
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! domainDir
#!! osType
#
#    Output:
#             Writes return code
#**************************************************************************************
def setWLSCredentials():
    global nmUserConfigFile, nmUserKeyFile
    nmUserConfigFile = domainDir + "/nodemanager/nmUserConfigFile.secure" 
    nmUserKeyFile = domainDir + "/nodemanager/nmUserKeyFile.secure"
    logger.debug("nmUserConfigFile=" + nmUserConfigFile)
    logger.debug("nmUserKeyFile=" + nmUserKeyFile)


import os

def getulimit():
    outfile = File.createTempFile("checkulimit", ".out")
    os.system('uname -s>' + `outfile`)
    f = open(`outfile`)
    try:
       line = f.readline()
       print 'line="' + line + '"'
    finally:
       f.close()
       os.remove(`outfile`)

    ostype = line.strip()

    rc = 0;
    if ostype == 'HP-UX':
        print '/usr/sbin/kctune maxfiles_lim >' + `outfile`
        rc = os.system('/usr/sbin/kctune maxfiles_lim >' + `outfile`)
    else:
        print 'ulimit -S -n >' + `outfile`
        rc = os.system('ulimit -S -n >' + `outfile`)
   
    if rc != 0:
        return 0
   
    f = open(`outfile`)
    try:
       line = f.readline()
       if ( ostype == 'HP-UX' ):
           line = f.readline()
    finally:
       f.close()
       os.remove(`outfile`)

    print "line=" + line
    
    if ostype == 'HP-UX':
       t = line.split()
       ulimit = t[1]
    else:
       ulimit = line.strip()

    if ulimit == "unlimited":
        ulimit = java.lang.Long.MAX_VALUE
    print "ulimit=" + ulimit

    return long(ulimit)

def getCredential(credentialsFile, credentialName, type):
    props = Properties()
    propsFile = File(credentialsFile)
    fh = FileInputStream(propsFile)
    props.load(fh)
    fh.close()
    return props.getProperty(credentialName + '_' + type)
    
def getCredentialUser(credentialsFile, credentialName):
    return getCredential(credentialsFile, credentialName, 'userid')
    
def getCredentialPassword(credentialsFile, credentialName):
    return getCredential(credentialsFile, credentialName, 'passwd')

from weblogic.security import UserConfigFileManager
def getAdminUser(userConfigFile, userKeyFile):
    ucfg = UserConfigFileManager.getUsernameAndPassword(userConfigFile, userKeyFile, 'weblogic.management')
    return ucfg.getUsername()

def getAdminPassword(userConfigFile, userKeyFile):
    ucfg = UserConfigFileManager.getUsernameAndPassword(userConfigFile, userKeyFile, 'weblogic.management')
    pw = ucfg.getPassword()
    passwd = ''
    for c in pw:
        passwd = passwd + c
    return passwd

def setNodeManagerDomain():
    nmDomainProperties = Properties()
    nmDomainPropsFile = File(domainDir + '/nodemanager/nodemanager.domains')
    if nmDomainPropsFile.exists():
        fh = FileInputStream(nmDomainPropsFile)
        nmDomainProperties.load(fh)
        fh.close()
    nmDomainProperties.setProperty(os.path.basename(domainDir), domainDir)
    fh = FileOutputStream(nmDomainPropsFile)
    nmDomainProperties.store(fh, "Domains supported by this nodemanager")
    fh.close()

def setNodeManagerProperty(key, value):
    nmProperties = Properties()
    nmPropsFile = File(domainDir + '/nodemanager/nodemanager.properties')
    if nmPropsFile.exists():
        fh = FileInputStream(nmPropsFile)
        nmProperties.load(fh)
        fh.close()
    nmProperties.setProperty(key, value)
    fh = FileOutputStream(nmPropsFile)
    nmProperties.store(fh, "Nodemanager Properties")
    fh.close()

def checkForURandomUseOnLinux():
    if osLinuxVendor == 'UNKNOWN':
        return false;
    securityFile = javaHome + '/jre/lib/security/java.security'
    f = open(securityFile, "r") 
    try:
      line = f.readline()
      while line != "":
        if line.find("securerandom.source=file:/dev/urandom") != -1:
            logger.error("The JDK is configured with the following line in " + securityFile + ":")
            logger.error("    securerandom.source=file:/dev/urandom")
            logger.error("On Linux systems this may result in slow start time for all servers. Change the setting to: ")
            logger.error("    securerandom.source=file:/dev/./urandom")
            writeReturnProperty("weblogicLinuxJdkUrandom", "true")
            return true
        line = f.readline()
    finally:
      f.close()
    return false

def isCorrectNativeLibraryInstalled():
    wlsNativeLibDirName = weblogicHome + "/server/native";
    if osMachineType == 'win':
        wlsNativeLibDir = File(wlsNativeLibDirName + "/win/32")  
    elif osMachineType == 'wx6':
        wlsNativeLibDir = File(wlsNativeLibDirName + "/win/x64")
    elif osMachineType == 's64':
        wlsNativeLibDir = File(wlsNativeLibDirName + "/solaris/sparc64")
    elif osMachineType == 'sax':
        wlsNativeLibDir = File(wlsNativeLibDirName + "/solaris/x64")
    elif osMachineType == 'r64':
        wlsNativeLibDir = File(wlsNativeLibDirName + "/aix/ppc64")
    elif osMachineType == 'h6i':
        wlsNativeLibDir = File(wlsNativeLibDirName + "/hpux11/IPF64")
    elif osMachineType == 'lax':
        wlsNativeLibDir = File(wlsNativeLibDirName + "/linux/x86_64")
    logger.debug('WebLogic Native Libraries should be located at: ' + `wlsNativeLibDir`)
    return wlsNativeLibDir.exists()

def is64BitArchitectureSupported():
    if osMachineType == 'win':
        return false
    elif osMachineType == 'wx6' or osMachineType == 'r64':
        return System.getProperty("sun.arch.data.model") == '64'
    else:
        rc = execScript(javaHome + "/bin/java -d64 -version")
        if rc == "0":
            return true;
        else:
            return false;

        