#!/bin/sh
#*****************************************************************************************************
# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved
#*****************************************************************************************************
#
# Name:     WASDriver.bat
#
# Purpose:  Run the WASDriver.groovy program to perform the configuration task(s)
#           that prepare the WAS environment for SAS webapps.
# 
# Command-line options:
# ---------------------
#    -e = Execution type: "RUN" - Run a single task as specified by command-line options
#                         "FILE" - Run all tasks specified in the taskFile command-line option
#    -m = Execution mode: "AUTO" - Run task(s) automatically
#                         "NOAUTO" = Syntax check all task(s)
#    -p = Prompt mode: "GRAPHIC" - Display prompts for required credentials using pop-window
#                      "TEXT" - Display prompts for required credentials using text-mode
#    -t = Task file: <Fully-qualified path to task file> (-e FILE required)
#    -d = Scripting directory: <Fully-qualified path to directory containing all scripting components>
#    -o = Task operation: (-e RUN required)
#             create, delete, start, stop, deploy, undeploy, update, updateProps, 
#             validate, updateProviderEndpoints, reset, syncNode, getPort, getStatus, 
#             checkAppStatus, clear, encodeAll, removeDuplicates
#    -r = Resource type: (-e RUN required)  
#             ActivationSpec, Application, Credentials, DataSource, DmgrProfile,
#             JDBCProvider, JMSConnectionFactory, JMSQueue, JMSTopic, LoginModule,
#             MailSession, MultiCast, Node, NodeProfile, ObjectCache, Server, SIBus
#    -n = Resource name: (-e RUN required)
#             <Name of resource to be configured>
#    -s = Scope: (-e RUN required)
#             CELL, NODE, SERVER
#***************************************************************************************************** 

#********************************************************
# Defaults for common variables for the WASDriver program
WD_JAVA=@javaLocation@
WD_CLASSPATH=@groovyClasspath@
WD_TASKFILE=@taskFile@
WD_SCRIPTINGDIR=@scriptingDirectory@
WD_EXECMODE=AUTO
WD_EXECTYPE=FILE
WD_PROMPT=TEXT
WD_HELP=0

#********************************************************
# Variables used to RUN single task 
WD_OPERATION=
WD_RESOURCETYPE=
WD_RESOURCENAME=
WD_SCOPE=

#************************************************************************************
# Process the input command-line options
echo "** Input command-line options:"
while [ $# -gt 0 ]
do
  case "$1" in
    -h)  WD_HELP=1;            echo "** Display WASDriver.sh help output: ";          break;;
    -e)  WD_EXECTYPE="$2";     echo "** Execution type:      \"$WD_EXECTYPE\"";       shift;;
    -m)  WD_EXECMODE="$2";     echo "** Execution mode:      \"$WD_EXECMODE\"";       shift;;
    -p)  WD_PROMPT="$2";       echo "** Prompt mode:         \"$WD_PROMPT\"";         shift;;
    -d)  WD_SCRIPTINGDIR="$2"; echo "** Scripting directory: \"$WD_SCRIPTINGDIR\"";   shift;;
    -t)  WD_TASKFILE="$2";     echo "** Task file:           \"$WD_TASKFILE\"";       shift;;
    -o)  WD_OPERATION="$2";    echo "** Operation:           \"$WD_OPERATION\"";      shift;;
    -r)  WD_RESOURCETYPE="$2"; echo "** Resource type:       \"$WD_RESOURCETYPE\"";   shift;;
    -n)  WD_RESOURCENAME="$2"; echo "** Resource name:       \"$WD_RESOURCENAME\"";   shift;;
    -s)  WD_SCOPE="$2";        echo "** Scope:               \"$WD_SCOPE\"";          shift;;
    *)   WD_HELP=1;            echo "**ERROR** Extraneous/unknown option \"$1\".";    break;;
  esac
  shift
done

#************************************************************************************
# Determine if help for WASDriver program should be produced
if [ $WD_HELP = 1 ]; then
  "$WD_JAVA/java" -cp "$WD_CLASSPATH" com.sas.config.websphere.WASDriver --help
  exit 0
fi

#***************************************************************************
# Run WASDriver Groovy program after determining type of execution requested
echo "** Run the WASDriver program with the following options:"
echo "** WD_EXECMODE=\"$WD_EXECMODE\""
echo "** WD_EXECTYPE=\"$WD_EXECTYPE\""
echo "** WD_PROMPT=\"$WD_PROMPT\""
echo "** WD_SCRIPTINGDIR=\"$WD_SCRIPTINGDIR\""
if [ ! -d $WD_SCRIPTINGDIR ]; then
  	echo "**ERROR** Scripting directory \"$WD_SCRIPTINGDIR\" does not exist."
   	exit 99  
fi

#*******************************************************
# Determine execution type for WASDriver Groovy program:
if [ "$WD_EXECTYPE" = "FILE" ]; then
	# Run the WASDriver Groovy program to configure WAS using a task file:
	echo "** WD_TASKFILE=\"$WD_TASKFILE\""
	if [ -f $WD_SCRIPTINGDIR/tasks/$WD_TASKFILE ]; then
	  echo "$WD_JAVA/java" -cp "$WD_CLASSPATH" com.sas.config.websphere.WASDriver -e $WD_EXECTYPE -m $WD_EXECMODE -p $WD_PROMPT -d "$WD_SCRIPTINGDIR" -t "$WD_TASKFILE"
      "$WD_JAVA/java" -cp "$WD_CLASSPATH" com.sas.config.websphere.WASDriver -e $WD_EXECTYPE -m $WD_EXECMODE -p $WD_PROMPT -d "$WD_SCRIPTINGDIR" -t "$WD_TASKFILE"
    else
      echo "**ERROR** Task file \"$WD_SCRIPTINGDIR/tasks/$WD_TASKFILE\" does not exist."
      exit 99
    fi	
  elif [ "$WD_EXECTYPE" = "RUN" ]; then 
  	# Run the WASDriver Groovy program for a single configuration task:
    echo "** WD_OPERATION="$WD_OPERATION
    echo "** WD_RESOURCENAME="$WD_RESOURCENAME
    echo "** WD_RESOURCETYPE="$WD_RESOURCETYPE
    echo "** WD_SCOPE="$WD_SCOPE
    WD_PROPERTYFILE=$WD_SCRIPTINGDIR/props/$WD_RESOURCETYPE.$WD_SCOPE.$WD_RESOURCENAME.properties
    if [ -f $WD_PROPERTYFILE ]; then
   	  echo "$WD_JAVA/java" -cp "$WD_CLASSPATH" com.sas.config.websphere.WASDriver -e $WD_EXECTYPE -m $WD_EXECMODE -p $WD_PROMPT -d "$WD_SCRIPTINGDIR" -o $WD_OPERATION -r $WD_RESOURCETYPE -n $WD_RESOURCENAME -s $WD_SCOPE
	  "$WD_JAVA/java" -cp "$WD_CLASSPATH" com.sas.config.websphere.WASDriver -e $WD_EXECTYPE -m $WD_EXECMODE -p $WD_PROMPT -d "$WD_SCRIPTINGDIR" -o $WD_OPERATION -r $WD_RESOURCETYPE -n $WD_RESOURCENAME -s $WD_SCOPE
    else
      echo "**ERROR** Required property file \"$WD_PROPERTYFILE\" does not exist."
      exit 99
    fi 	  
  else
    echo "**ERROR** Unknown -e option \"$WD_EXECTYPE\", must be \"RUN\" or \"FILE\"."
    exit 99	
fi
exit 0